"use strict";(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[1362],{85310:function(e,t,n){n.d(t,{Z:function(){return i}});/**
 * @license lucide-react v0.407.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */let i=(0,n(60998).Z)("Briefcase",[["path",{d:"M16 20V4a2 2 0 0 0-2-2h-4a2 2 0 0 0-2 2v16",key:"jecpp"}],["rect",{width:"20",height:"14",x:"2",y:"6",rx:"2",key:"i6l2r4"}]])},33551:function(e,t,n){n.d(t,{Z:function(){return i}});/**
 * @license lucide-react v0.407.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */let i=(0,n(60998).Z)("Database",[["ellipse",{cx:"12",cy:"5",rx:"9",ry:"3",key:"msslwz"}],["path",{d:"M3 5V19A9 3 0 0 0 21 19V5",key:"1wlel7"}],["path",{d:"M3 12A9 3 0 0 0 21 12",key:"mv7ke4"}]])},172:function(e,t,n){n.d(t,{Z:function(){return i}});/**
 * @license lucide-react v0.407.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */let i=(0,n(60998).Z)("Layers",[["path",{d:"m12.83 2.18a2 2 0 0 0-1.66 0L2.6 6.08a1 1 0 0 0 0 1.83l8.58 3.91a2 2 0 0 0 1.66 0l8.58-3.9a1 1 0 0 0 0-1.83Z",key:"8b97xw"}],["path",{d:"m22 17.65-9.17 4.16a2 2 0 0 1-1.66 0L2 17.65",key:"dd6zsq"}],["path",{d:"m22 12.65-9.17 4.16a2 2 0 0 1-1.66 0L2 12.65",key:"ep9fru"}]])},24544:function(e,t,n){n.d(t,{Z:function(){return i}});/**
 * @license lucide-react v0.407.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */let i=(0,n(60998).Z)("Pin",[["path",{d:"M12 17v5",key:"bb1du9"}],["path",{d:"M9 10.76a2 2 0 0 1-1.11 1.79l-1.78.9A2 2 0 0 0 5 15.24V16a1 1 0 0 0 1 1h12a1 1 0 0 0 1-1v-.76a2 2 0 0 0-1.11-1.79l-1.78-.9A2 2 0 0 1 15 10.76V7a1 1 0 0 1 1-1 2 2 0 0 0 0-4H8a2 2 0 0 0 0 4 1 1 0 0 1 1 1z",key:"1nkz8b"}]])},90598:function(e,t,n){n.d(t,{Z:function(){return i}});/**
 * @license lucide-react v0.407.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */let i=(0,n(60998).Z)("Server",[["rect",{width:"20",height:"8",x:"2",y:"2",rx:"2",ry:"2",key:"ngkwjq"}],["rect",{width:"20",height:"8",x:"2",y:"14",rx:"2",ry:"2",key:"iecqi9"}],["line",{x1:"6",x2:"6.01",y1:"6",y2:"6",key:"16zg32"}],["line",{x1:"6",x2:"6.01",y1:"18",y2:"18",key:"nzw8ys"}]])},43767:function(e,t,n){n.d(t,{Z:function(){return i}});/**
 * @license lucide-react v0.407.0 - ISC
 *
 * This source code is licensed under the ISC license.
 * See the LICENSE file in the root directory of this source tree.
 */let i=(0,n(60998).Z)("X",[["path",{d:"M18 6 6 18",key:"1bl5f8"}],["path",{d:"m6 6 12 12",key:"d8bk6v"}]])},53464:function(e,t,n){n.d(t,{rV:function(){return et}});var i=n(41113),r=n(83454);class s{constructor(e,t,n,i,r,s,o,l,a,h=0,c){this.p=e,this.stack=t,this.state=n,this.reducePos=i,this.pos=r,this.score=s,this.buffer=o,this.bufferBase=l,this.curContext=a,this.lookAhead=h,this.parent=c}toString(){return`[${this.stack.filter((e,t)=>t%3==0).concat(this.state)}]@${this.pos}${this.score?"!"+this.score:""}`}static start(e,t,n=0){let i=e.parser.context;return new s(e,[],t,n,n,0,[],0,i?new o(i,i.start):null,0,null)}get context(){return this.curContext?this.curContext.context:null}pushState(e,t){this.stack.push(this.state,t,this.bufferBase+this.buffer.length),this.state=e}reduce(e){var t;let n=e>>19,i=65535&e,{parser:r}=this.p,s=this.reducePos<this.pos-25&&this.setLookAhead(this.pos),o=r.dynamicPrecedence(i);if(o&&(this.score+=o),0==n){this.pushState(r.getGoto(this.state,i,!0),this.reducePos),i<r.minRepeatTerm&&this.storeNode(i,this.reducePos,this.reducePos,s?8:4,!0),this.reduceContext(i,this.reducePos);return}let l=this.stack.length-(n-1)*3-(262144&e?6:0),a=l?this.stack[l-2]:this.p.ranges[0].from,h=this.reducePos-a;h>=2e3&&!(null===(t=this.p.parser.nodeSet.types[i])||void 0===t?void 0:t.isAnonymous)&&(a==this.p.lastBigReductionStart?(this.p.bigReductionCount++,this.p.lastBigReductionSize=h):this.p.lastBigReductionSize<h&&(this.p.bigReductionCount=1,this.p.lastBigReductionStart=a,this.p.lastBigReductionSize=h));let c=l?this.stack[l-1]:0,u=this.bufferBase+this.buffer.length-c;if(i<r.minRepeatTerm||131072&e){let e=r.stateFlag(this.state,1)?this.pos:this.reducePos;this.storeNode(i,a,e,u+4,!0)}if(262144&e)this.state=this.stack[l];else{let e=this.stack[l-3];this.state=r.getGoto(e,i,!0)}for(;this.stack.length>l;)this.stack.pop();this.reduceContext(i,a)}storeNode(e,t,n,i=4,r=!1){if(0==e&&(!this.stack.length||this.stack[this.stack.length-1]<this.buffer.length+this.bufferBase)){let e=this,i=this.buffer.length;if(0==i&&e.parent&&(i=e.bufferBase-e.parent.bufferBase,e=e.parent),i>0&&0==e.buffer[i-4]&&e.buffer[i-1]>-1){if(t==n)return;if(e.buffer[i-2]>=t){e.buffer[i-2]=n;return}}}if(r&&this.pos!=n){let r=this.buffer.length;if(r>0&&(0!=this.buffer[r-4]||this.buffer[r-1]<0)){let e=!1;for(let t=r;t>0&&this.buffer[t-2]>n;t-=4)if(this.buffer[t-1]>=0){e=!0;break}if(e)for(;r>0&&this.buffer[r-2]>n;)this.buffer[r]=this.buffer[r-4],this.buffer[r+1]=this.buffer[r-3],this.buffer[r+2]=this.buffer[r-2],this.buffer[r+3]=this.buffer[r-1],r-=4,i>4&&(i-=4)}this.buffer[r]=e,this.buffer[r+1]=t,this.buffer[r+2]=n,this.buffer[r+3]=i}else this.buffer.push(e,t,n,i)}shift(e,t,n,i){if(131072&e)this.pushState(65535&e,this.pos);else if((262144&e)==0){let{parser:r}=this.p;this.pos=i;let s=r.stateFlag(e,1);!s&&(i>n||t<=r.maxNode)&&(this.reducePos=i),this.pushState(e,s?n:Math.min(n,this.reducePos)),this.shiftContext(t,n),t<=r.maxNode&&this.buffer.push(t,n,i,4)}else this.pos=i,this.shiftContext(t,n),t<=this.p.parser.maxNode&&this.buffer.push(t,n,i,4)}apply(e,t,n,i){65536&e?this.reduce(e):this.shift(e,t,n,i)}useNode(e,t){let n=this.p.reused.length-1;(n<0||this.p.reused[n]!=e)&&(this.p.reused.push(e),n++);let i=this.pos;this.reducePos=this.pos=i+e.length,this.pushState(t,i),this.buffer.push(n,i,this.reducePos,-1),this.curContext&&this.updateContext(this.curContext.tracker.reuse(this.curContext.context,e,this,this.p.stream.reset(this.pos-e.length)))}split(){let e=this,t=e.buffer.length;for(;t>0&&e.buffer[t-2]>e.reducePos;)t-=4;let n=e.buffer.slice(t),i=e.bufferBase+t;for(;e&&i==e.bufferBase;)e=e.parent;return new s(this.p,this.stack.slice(),this.state,this.reducePos,this.pos,this.score,n,i,this.curContext,this.lookAhead,e)}recoverByDelete(e,t){let n=e<=this.p.parser.maxNode;n&&this.storeNode(e,this.pos,t,4),this.storeNode(0,this.pos,t,n?8:4),this.pos=this.reducePos=t,this.score-=190}canShift(e){for(let t=new l(this);;){let n=this.p.parser.stateSlot(t.state,4)||this.p.parser.hasAction(t.state,e);if(0==n)return!1;if((65536&n)==0)return!0;t.reduce(n)}}recoverByInsert(e){if(this.stack.length>=300)return[];let t=this.p.parser.nextStates(this.state);if(t.length>8||this.stack.length>=120){let n=[];for(let i=0,r;i<t.length;i+=2)(r=t[i+1])!=this.state&&this.p.parser.hasAction(r,e)&&n.push(t[i],r);if(this.stack.length<120)for(let e=0;n.length<8&&e<t.length;e+=2){let i=t[e+1];n.some((e,t)=>1&t&&e==i)||n.push(t[e],i)}t=n}let n=[];for(let e=0;e<t.length&&n.length<4;e+=2){let i=t[e+1];if(i==this.state)continue;let r=this.split();r.pushState(i,this.pos),r.storeNode(0,r.pos,r.pos,4,!0),r.shiftContext(t[e],this.pos),r.reducePos=this.pos,r.score-=200,n.push(r)}return n}forceReduce(){let{parser:e}=this.p,t=e.stateSlot(this.state,5);if((65536&t)==0)return!1;if(!e.validAction(this.state,t)){let n=t>>19,i=65535&t,r=this.stack.length-3*n;if(r<0||0>e.getGoto(this.stack[r],i,!1)){let e=this.findForcedReduction();if(null==e)return!1;t=e}this.storeNode(0,this.pos,this.pos,4,!0),this.score-=100}return this.reducePos=this.pos,this.reduce(t),!0}findForcedReduction(){let{parser:e}=this.p,t=[],n=(i,r)=>{if(!t.includes(i))return t.push(i),e.allActions(i,t=>{if(393216&t);else if(65536&t){let n=(t>>19)-r;if(n>1){let i=65535&t,r=this.stack.length-3*n;if(r>=0&&e.getGoto(this.stack[r],i,!1)>=0)return n<<19|65536|i}}else{let e=n(t,r+1);if(null!=e)return e}})};return n(this.state,0)}forceAll(){for(;!this.p.parser.stateFlag(this.state,2);)if(!this.forceReduce()){this.storeNode(0,this.pos,this.pos,4,!0);break}return this}get deadEnd(){if(3!=this.stack.length)return!1;let{parser:e}=this.p;return 65535==e.data[e.stateSlot(this.state,1)]&&!e.stateSlot(this.state,4)}restart(){this.storeNode(0,this.pos,this.pos,4,!0),this.state=this.stack[0],this.stack.length=0}sameState(e){if(this.state!=e.state||this.stack.length!=e.stack.length)return!1;for(let t=0;t<this.stack.length;t+=3)if(this.stack[t]!=e.stack[t])return!1;return!0}get parser(){return this.p.parser}dialectEnabled(e){return this.p.parser.dialect.flags[e]}shiftContext(e,t){this.curContext&&this.updateContext(this.curContext.tracker.shift(this.curContext.context,e,this,this.p.stream.reset(t)))}reduceContext(e,t){this.curContext&&this.updateContext(this.curContext.tracker.reduce(this.curContext.context,e,this,this.p.stream.reset(t)))}emitContext(){let e=this.buffer.length-1;(e<0||-3!=this.buffer[e])&&this.buffer.push(this.curContext.hash,this.pos,this.pos,-3)}emitLookAhead(){let e=this.buffer.length-1;(e<0||-4!=this.buffer[e])&&this.buffer.push(this.lookAhead,this.pos,this.pos,-4)}updateContext(e){if(e!=this.curContext.context){let t=new o(this.curContext.tracker,e);t.hash!=this.curContext.hash&&this.emitContext(),this.curContext=t}}setLookAhead(e){return!(e<=this.lookAhead)&&(this.emitLookAhead(),this.lookAhead=e,!0)}close(){this.curContext&&this.curContext.tracker.strict&&this.emitContext(),this.lookAhead>0&&this.emitLookAhead()}}class o{constructor(e,t){this.tracker=e,this.context=t,this.hash=e.strict?e.hash(t):0}}class l{constructor(e){this.start=e,this.state=e.state,this.stack=e.stack,this.base=this.stack.length}reduce(e){let t=e>>19;0==t?(this.stack==this.start.stack&&(this.stack=this.stack.slice()),this.stack.push(this.state,0,0),this.base+=3):this.base-=(t-1)*3;let n=this.start.p.parser.getGoto(this.stack[this.base-3],65535&e,!0);this.state=n}}class a{constructor(e,t,n){this.stack=e,this.pos=t,this.index=n,this.buffer=e.buffer,0==this.index&&this.maybeNext()}static create(e,t=e.bufferBase+e.buffer.length){return new a(e,t,t-e.bufferBase)}maybeNext(){let e=this.stack.parent;null!=e&&(this.index=this.stack.bufferBase-e.bufferBase,this.stack=e,this.buffer=e.buffer)}get id(){return this.buffer[this.index-4]}get start(){return this.buffer[this.index-3]}get end(){return this.buffer[this.index-2]}get size(){return this.buffer[this.index-1]}next(){this.index-=4,this.pos-=4,0==this.index&&this.maybeNext()}fork(){return new a(this.stack,this.pos,this.index)}}function h(e,t=Uint16Array){if("string"!=typeof e)return e;let n=null;for(let i=0,r=0;i<e.length;){let s=0;for(;;){let t=e.charCodeAt(i++),n=!1;if(126==t){s=65535;break}t>=92&&t--,t>=34&&t--;let r=t-32;if(r>=46&&(r-=46,n=!0),s+=r,n)break;s*=46}n?n[r++]=s:n=new t(s)}return n}class c{constructor(){this.start=-1,this.value=-1,this.end=-1,this.extended=-1,this.lookAhead=0,this.mask=0,this.context=0}}let u=new c;class f{constructor(e,t){this.input=e,this.ranges=t,this.chunk="",this.chunkOff=0,this.chunk2="",this.chunk2Pos=0,this.next=-1,this.token=u,this.rangeIndex=0,this.pos=this.chunkPos=t[0].from,this.range=t[0],this.end=t[t.length-1].to,this.readNext()}resolveOffset(e,t){let n=this.range,i=this.rangeIndex,r=this.pos+e;for(;r<n.from;){if(!i)return null;let e=this.ranges[--i];r-=n.from-e.to,n=e}for(;t<0?r>n.to:r>=n.to;){if(i==this.ranges.length-1)return null;let e=this.ranges[++i];r+=e.from-n.to,n=e}return r}clipPos(e){if(e>=this.range.from&&e<this.range.to)return e;for(let t of this.ranges)if(t.to>e)return Math.max(e,t.from);return this.end}peek(e){let t=this.chunkOff+e,n,i;if(t>=0&&t<this.chunk.length)n=this.pos+e,i=this.chunk.charCodeAt(t);else{let t=this.resolveOffset(e,1);if(null==t)return -1;if((n=t)>=this.chunk2Pos&&n<this.chunk2Pos+this.chunk2.length)i=this.chunk2.charCodeAt(n-this.chunk2Pos);else{let e=this.rangeIndex,t=this.range;for(;t.to<=n;)t=this.ranges[++e];this.chunk2=this.input.chunk(this.chunk2Pos=n),n+this.chunk2.length>t.to&&(this.chunk2=this.chunk2.slice(0,t.to-n)),i=this.chunk2.charCodeAt(0)}}return n>=this.token.lookAhead&&(this.token.lookAhead=n+1),i}acceptToken(e,t=0){let n=t?this.resolveOffset(t,-1):this.pos;if(null==n||n<this.token.start)throw RangeError("Token end out of bounds");this.token.value=e,this.token.end=n}acceptTokenTo(e,t){this.token.value=e,this.token.end=t}getChunk(){if(this.pos>=this.chunk2Pos&&this.pos<this.chunk2Pos+this.chunk2.length){let{chunk:e,chunkPos:t}=this;this.chunk=this.chunk2,this.chunkPos=this.chunk2Pos,this.chunk2=e,this.chunk2Pos=t,this.chunkOff=this.pos-this.chunkPos}else{this.chunk2=this.chunk,this.chunk2Pos=this.chunkPos;let e=this.input.chunk(this.pos),t=this.pos+e.length;this.chunk=t>this.range.to?e.slice(0,this.range.to-this.pos):e,this.chunkPos=this.pos,this.chunkOff=0}}readNext(){return this.chunkOff>=this.chunk.length&&(this.getChunk(),this.chunkOff==this.chunk.length)?this.next=-1:this.next=this.chunk.charCodeAt(this.chunkOff)}advance(e=1){for(this.chunkOff+=e;this.pos+e>=this.range.to;){if(this.rangeIndex==this.ranges.length-1)return this.setDone();e-=this.range.to-this.pos,this.range=this.ranges[++this.rangeIndex],this.pos=this.range.from}return this.pos+=e,this.pos>=this.token.lookAhead&&(this.token.lookAhead=this.pos+1),this.readNext()}setDone(){return this.pos=this.chunkPos=this.end,this.range=this.ranges[this.rangeIndex=this.ranges.length-1],this.chunk="",this.next=-1}reset(e,t){if(t?(this.token=t,t.start=e,t.lookAhead=e+1,t.value=t.extended=-1):this.token=u,this.pos!=e){if(this.pos=e,e==this.end)return this.setDone(),this;for(;e<this.range.from;)this.range=this.ranges[--this.rangeIndex];for(;e>=this.range.to;)this.range=this.ranges[++this.rangeIndex];e>=this.chunkPos&&e<this.chunkPos+this.chunk.length?this.chunkOff=e-this.chunkPos:(this.chunk="",this.chunkOff=0),this.readNext()}return this}read(e,t){if(e>=this.chunkPos&&t<=this.chunkPos+this.chunk.length)return this.chunk.slice(e-this.chunkPos,t-this.chunkPos);if(e>=this.chunk2Pos&&t<=this.chunk2Pos+this.chunk2.length)return this.chunk2.slice(e-this.chunk2Pos,t-this.chunk2Pos);if(e>=this.range.from&&t<=this.range.to)return this.input.read(e,t);let n="";for(let i of this.ranges){if(i.from>=t)break;i.to>e&&(n+=this.input.read(Math.max(i.from,e),Math.min(i.to,t)))}return n}}class d{constructor(e,t){this.data=e,this.id=t}token(e,t){let{parser:n}=t.p;g(this.data,e,t,this.id,n.data,n.tokenPrecTable)}}d.prototype.contextual=d.prototype.fallback=d.prototype.extend=!1;class p{constructor(e,t,n){this.precTable=t,this.elseToken=n,this.data="string"==typeof e?h(e):e}token(e,t){let n=e.pos,i=0;for(;;){let n=e.next<0,r=e.resolveOffset(1,1);if(g(this.data,e,t,0,this.data,this.precTable),e.token.value>-1)break;if(null==this.elseToken)return;if(!n&&i++,null==r)break;e.reset(r,e.token)}i&&(e.reset(n,e.token),e.acceptToken(this.elseToken,i))}}p.prototype.contextual=d.prototype.fallback=d.prototype.extend=!1;class m{constructor(e,t={}){this.token=e,this.contextual=!!t.contextual,this.fallback=!!t.fallback,this.extend=!!t.extend}}function g(e,t,n,i,r,s){let o=0,l=1<<i,{dialect:a}=n.p.parser;e:for(;(l&e[o])!=0;){let n=e[o+1];for(let i=o+3;i<n;i+=2)if((e[i+1]&l)>0){let n=e[i];if(a.allows(n)&&(-1==t.token.value||t.token.value==n||function(e,t,n,i){let r=k(n,i,t);return r<0||k(n,i,e)<r}(n,t.token.value,r,s))){t.acceptToken(n);break}}let i=t.next,h=0,c=e[o+2];if(t.next<0&&c>h&&65535==e[n+3*c-3]){o=e[n+3*c-1];continue}for(;h<c;){let r=h+c>>1,s=n+r+(r<<1),l=e[s],a=e[s+1]||65536;if(i<l)c=r;else if(i>=a)h=r+1;else{o=e[s+2],t.advance();continue e}}break}}function k(e,t,n){for(let i=t,r;65535!=(r=e[i]);i++)if(r==n)return i-t;return -1}let v=void 0!==r&&r.env&&/\bparse\b/.test(r.env.LOG),y=null;function x(e,t,n){let r=e.cursor(i.vj.IncludeAnonymous);for(r.moveTo(t);;)if(!(n<0?r.childBefore(t):r.childAfter(t)))for(;;){if((n<0?r.to<t:r.from>t)&&!r.type.isError)return n<0?Math.max(0,Math.min(r.to-1,t-25)):Math.min(e.length,Math.max(r.from+1,t+25));if(n<0?r.prevSibling():r.nextSibling())break;if(!r.parent())return n<0?0:e.length}}class b{constructor(e,t){this.fragments=e,this.nodeSet=t,this.i=0,this.fragment=null,this.safeFrom=-1,this.safeTo=-1,this.trees=[],this.start=[],this.index=[],this.nextFragment()}nextFragment(){let e=this.fragment=this.i==this.fragments.length?null:this.fragments[this.i++];if(e){for(this.safeFrom=e.openStart?x(e.tree,e.from+e.offset,1)-e.offset:e.from,this.safeTo=e.openEnd?x(e.tree,e.to+e.offset,-1)-e.offset:e.to;this.trees.length;)this.trees.pop(),this.start.pop(),this.index.pop();this.trees.push(e.tree),this.start.push(-e.offset),this.index.push(0),this.nextStart=this.safeFrom}else this.nextStart=1e9}nodeAt(e){if(e<this.nextStart)return null;for(;this.fragment&&this.safeTo<=e;)this.nextFragment();if(!this.fragment)return null;for(;;){let t=this.trees.length-1;if(t<0)return this.nextFragment(),null;let n=this.trees[t],r=this.index[t];if(r==n.children.length){this.trees.pop(),this.start.pop(),this.index.pop();continue}let s=n.children[r],o=this.start[t]+n.positions[r];if(o>e)return this.nextStart=o,null;if(s instanceof i.mp){if(o==e){if(o<this.safeFrom)return null;let e=o+s.length;if(e<=this.safeTo){let t=s.prop(i.md.lookAhead);if(!t||e+t<this.fragment.to)return s}}this.index[t]++,o+s.length>=Math.max(this.safeFrom,e)&&(this.trees.push(s),this.start.push(o),this.index.push(0))}else this.index[t]++,this.nextStart=o+s.length}}}class O{constructor(e,t){this.stream=t,this.tokens=[],this.mainToken=null,this.actions=[],this.tokens=e.tokenizers.map(e=>new c)}getActions(e){let t=0,n=null,{parser:i}=e.p,{tokenizers:r}=i,s=i.stateSlot(e.state,3),o=e.curContext?e.curContext.hash:0,l=0;for(let i=0;i<r.length;i++){if((1<<i&s)==0)continue;let a=r[i],h=this.tokens[i];if((!n||a.fallback)&&((a.contextual||h.start!=e.pos||h.mask!=s||h.context!=o)&&(this.updateCachedToken(h,a,e),h.mask=s,h.context=o),h.lookAhead>h.end+25&&(l=Math.max(h.lookAhead,l)),0!=h.value)){let i=t;if(h.extended>-1&&(t=this.addActions(e,h.extended,h.end,t)),t=this.addActions(e,h.value,h.end,t),!a.extend&&(n=h,t>i))break}}for(;this.actions.length>t;)this.actions.pop();return l&&e.setLookAhead(l),n||e.pos!=this.stream.end||((n=new c).value=e.p.parser.eofTerm,n.start=n.end=e.pos,t=this.addActions(e,n.value,n.end,t)),this.mainToken=n,this.actions}getMainToken(e){if(this.mainToken)return this.mainToken;let t=new c,{pos:n,p:i}=e;return t.start=n,t.end=Math.min(n+1,i.stream.end),t.value=n==i.stream.end?i.parser.eofTerm:0,t}updateCachedToken(e,t,n){let i=this.stream.clipPos(n.pos);if(t.token(this.stream.reset(i,e),n),e.value>-1){let{parser:t}=n.p;for(let i=0;i<t.specialized.length;i++)if(t.specialized[i]==e.value){let r=t.specializers[i](this.stream.read(e.start,e.end),n);if(r>=0&&n.p.parser.dialect.allows(r>>1)){(1&r)==0?e.value=r>>1:e.extended=r>>1;break}}}else e.value=0,e.end=this.stream.clipPos(i+1)}putAction(e,t,n,i){for(let t=0;t<i;t+=3)if(this.actions[t]==e)return i;return this.actions[i++]=e,this.actions[i++]=t,this.actions[i++]=n,i}addActions(e,t,n,i){let{state:r}=e,{parser:s}=e.p,{data:o}=s;for(let e=0;e<2;e++)for(let l=s.stateSlot(r,e?2:1);;l+=3){if(65535==o[l]){if(1==o[l+1])l=R(o,l+2);else{0==i&&2==o[l+1]&&(i=this.putAction(R(o,l+2),t,n,i));break}}o[l]==t&&(i=this.putAction(R(o,l+1),t,n,i))}return i}}class w{constructor(e,t,n,i){this.parser=e,this.input=t,this.ranges=i,this.recovering=0,this.nextStackID=9812,this.minStackPos=0,this.reused=[],this.stoppedAt=null,this.lastBigReductionStart=-1,this.lastBigReductionSize=0,this.bigReductionCount=0,this.stream=new f(t,i),this.tokens=new O(e,this.stream),this.topTerm=e.top[1];let{from:r}=i[0];this.stacks=[s.start(this,e.top[0],r)],this.fragments=n.length&&this.stream.end-r>4*e.bufferLength?new b(n,e.nodeSet):null}get parsedPos(){return this.minStackPos}advance(){let e,t,n=this.stacks,i=this.minStackPos,r=this.stacks=[];if(this.bigReductionCount>300&&1==n.length){let[e]=n;for(;e.forceReduce()&&e.stack.length&&e.stack[e.stack.length-2]>=this.lastBigReductionStart;);this.bigReductionCount=this.lastBigReductionSize=0}for(let s=0;s<n.length;s++){let o=n[s];for(;;){if(this.tokens.mainToken=null,o.pos>i)r.push(o);else{if(this.advanceStack(o,r,n))continue;e||(e=[],t=[]),e.push(o);let i=this.tokens.getMainToken(o);t.push(i.value,i.end)}break}}if(!r.length){let t=e&&function(e){let t=null;for(let n of e){let e=n.p.stoppedAt;(n.pos==n.p.stream.end||null!=e&&n.pos>e)&&n.p.parser.stateFlag(n.state,2)&&(!t||t.score<n.score)&&(t=n)}return t}(e);if(t)return v&&console.log("Finish with "+this.stackID(t)),this.stackToTree(t);if(this.parser.strict)throw v&&e&&console.log("Stuck with token "+(this.tokens.mainToken?this.parser.getName(this.tokens.mainToken.value):"none")),SyntaxError("No parse at "+i);this.recovering||(this.recovering=5)}if(this.recovering&&e){let n=null!=this.stoppedAt&&e[0].pos>this.stoppedAt?e[0]:this.runRecovery(e,t,r);if(n)return v&&console.log("Force-finish "+this.stackID(n)),this.stackToTree(n.forceAll())}if(this.recovering){let e=1==this.recovering?1:3*this.recovering;if(r.length>e)for(r.sort((e,t)=>t.score-e.score);r.length>e;)r.pop();r.some(e=>e.reducePos>i)&&this.recovering--}else if(r.length>1){t:for(let e=0;e<r.length-1;e++){let t=r[e];for(let n=e+1;n<r.length;n++){let i=r[n];if(t.sameState(i)||t.buffer.length>500&&i.buffer.length>500){if((t.score-i.score||t.buffer.length-i.buffer.length)>0)r.splice(n--,1);else{r.splice(e--,1);continue t}}}}r.length>12&&(r.sort((e,t)=>t.score-e.score),r.splice(12,r.length-12))}this.minStackPos=r[0].pos;for(let e=1;e<r.length;e++)r[e].pos<this.minStackPos&&(this.minStackPos=r[e].pos);return null}stopAt(e){if(null!=this.stoppedAt&&this.stoppedAt<e)throw RangeError("Can't move stoppedAt forward");this.stoppedAt=e}advanceStack(e,t,n){let r=e.pos,{parser:s}=this,o=v?this.stackID(e)+" -> ":"";if(null!=this.stoppedAt&&r>this.stoppedAt)return e.forceReduce()?e:null;if(this.fragments){let t=e.curContext&&e.curContext.tracker.strict,n=t?e.curContext.hash:0;for(let l=this.fragments.nodeAt(r);l;){let r=this.parser.nodeSet.types[l.type.id]==l.type?s.getGoto(e.state,l.type.id):-1;if(r>-1&&l.length&&(!t||(l.prop(i.md.contextHash)||0)==n))return e.useNode(l,r),v&&console.log(o+this.stackID(e)+` (via reuse of ${s.getName(l.type.id)})`),!0;if(!(l instanceof i.mp)||0==l.children.length||l.positions[0]>0)break;let a=l.children[0];if(a instanceof i.mp&&0==l.positions[0])l=a;else break}}let l=s.stateSlot(e.state,4);if(l>0)return e.reduce(l),v&&console.log(o+this.stackID(e)+` (via always-reduce ${s.getName(65535&l)})`),!0;if(e.stack.length>=8400)for(;e.stack.length>6e3&&e.forceReduce(););let a=this.tokens.getActions(e);for(let i=0;i<a.length;){let l=a[i++],h=a[i++],c=a[i++],u=i==a.length||!n,f=u?e:e.split(),d=this.tokens.mainToken;if(f.apply(l,h,d?d.start:f.pos,c),v&&console.log(o+this.stackID(f)+` (via ${(65536&l)==0?"shift":`reduce of ${s.getName(65535&l)}`} for ${s.getName(h)} @ ${r}${f==e?"":", split"})`),u)return!0;f.pos>r?t.push(f):n.push(f)}return!1}advanceFully(e,t){let n=e.pos;for(;;){if(!this.advanceStack(e,null,null))return!1;if(e.pos>n)return S(e,t),!0}}runRecovery(e,t,n){let i=null,r=!1;for(let s=0;s<e.length;s++){let o=e[s],l=t[s<<1],a=t[(s<<1)+1],h=v?this.stackID(o)+" -> ":"";if(o.deadEnd&&(r||(r=!0,o.restart(),v&&console.log(h+this.stackID(o)+" (restarted)"),this.advanceFully(o,n))))continue;let c=o.split(),u=h;for(let e=0;e<10&&c.forceReduce()&&(v&&console.log(u+this.stackID(c)+" (via force-reduce)"),!this.advanceFully(c,n));e++)v&&(u=this.stackID(c)+" -> ");for(let e of o.recoverByInsert(l))v&&console.log(h+this.stackID(e)+" (via recover-insert)"),this.advanceFully(e,n);this.stream.end>o.pos?(a==o.pos&&(a++,l=0),o.recoverByDelete(l,a),v&&console.log(h+this.stackID(o)+` (via recover-delete ${this.parser.getName(l)})`),S(o,n)):(!i||i.score<c.score)&&(i=c)}return i}stackToTree(e){return e.close(),i.mp.build({buffer:a.create(e),nodeSet:this.parser.nodeSet,topID:this.topTerm,maxBufferLength:this.parser.bufferLength,reused:this.reused,start:this.ranges[0].from,length:e.pos-this.ranges[0].from,minRepeatType:this.parser.minRepeatTerm})}stackID(e){let t=(y||(y=new WeakMap)).get(e);return t||y.set(e,t=String.fromCodePoint(this.nextStackID++)),t+e}}function S(e,t){for(let n=0;n<t.length;n++){let i=t[n];if(i.pos==e.pos&&i.sameState(e)){t[n].score<e.score&&(t[n]=e);return}}t.push(e)}class P{constructor(e,t,n){this.source=e,this.flags=t,this.disabled=n}allows(e){return!this.disabled||0==this.disabled[e]}}let C=e=>e;class A{constructor(e){this.start=e.start,this.shift=e.shift||C,this.reduce=e.reduce||C,this.reuse=e.reuse||C,this.hash=e.hash||(()=>0),this.strict=!1!==e.strict}}class T extends i._b{constructor(e){if(super(),this.wrappers=[],14!=e.version)throw RangeError(`Parser version (${e.version}) doesn't match runtime version (14)`);let t=e.nodeNames.split(" ");this.minRepeatTerm=t.length;for(let n=0;n<e.repeatNodeCount;n++)t.push("");let n=Object.keys(e.topRules).map(t=>e.topRules[t][1]),r=[];for(let e=0;e<t.length;e++)r.push([]);function s(e,t,n){r[e].push([t,t.deserialize(String(n))])}if(e.nodeProps)for(let t of e.nodeProps){let e=t[0];"string"==typeof e&&(e=i.md[e]);for(let n=1;n<t.length;){let i=t[n++];if(i>=0)s(i,e,t[n++]);else{let r=t[n+-i];for(let o=-i;o>0;o--)s(t[n++],e,r);n++}}}this.nodeSet=new i.Lj(t.map((t,s)=>i.Jq.define({name:s>=this.minRepeatTerm?void 0:t,id:s,props:r[s],top:n.indexOf(s)>-1,error:0==s,skipped:e.skippedNodes&&e.skippedNodes.indexOf(s)>-1}))),e.propSources&&(this.nodeSet=this.nodeSet.extend(...e.propSources)),this.strict=!1,this.bufferLength=i.L3;let o=h(e.tokenData);this.context=e.context,this.specializerSpecs=e.specialized||[],this.specialized=new Uint16Array(this.specializerSpecs.length);for(let e=0;e<this.specializerSpecs.length;e++)this.specialized[e]=this.specializerSpecs[e].term;this.specializers=this.specializerSpecs.map(D),this.states=h(e.states,Uint32Array),this.data=h(e.stateData),this.goto=h(e.goto),this.maxTerm=e.maxTerm,this.tokenizers=e.tokenizers.map(e=>"number"==typeof e?new d(o,e):e),this.topRules=e.topRules,this.dialects=e.dialects||{},this.dynamicPrecedences=e.dynamicPrecedences||null,this.tokenPrecTable=e.tokenPrec,this.termNames=e.termNames||null,this.maxNode=this.nodeSet.types.length-1,this.dialect=this.parseDialect(),this.top=this.topRules[Object.keys(this.topRules)[0]]}createParse(e,t,n){let i=new w(this,e,t,n);for(let r of this.wrappers)i=r(i,e,t,n);return i}getGoto(e,t,n=!1){let i=this.goto;if(t>=i[0])return -1;for(let r=i[t+1];;){let t=i[r++],s=1&t,o=i[r++];if(s&&n)return o;for(let n=r+(t>>1);r<n;r++)if(i[r]==e)return o;if(s)return -1}}hasAction(e,t){let n=this.data;for(let i=0;i<2;i++)for(let r=this.stateSlot(e,i?2:1),s;;r+=3){if(65535==(s=n[r])){if(1==n[r+1])s=n[r=R(n,r+2)];else if(2==n[r+1])return R(n,r+2);else break}if(s==t||0==s)return R(n,r+1)}return 0}stateSlot(e,t){return this.states[6*e+t]}stateFlag(e,t){return(this.stateSlot(e,0)&t)>0}validAction(e,t){return!!this.allActions(e,e=>e==t||null)}allActions(e,t){let n=this.stateSlot(e,4),i=n?t(n):void 0;for(let n=this.stateSlot(e,1);null==i;n+=3){if(65535==this.data[n]){if(1==this.data[n+1])n=R(this.data,n+2);else break}i=t(R(this.data,n+1))}return i}nextStates(e){let t=[];for(let n=this.stateSlot(e,1);;n+=3){if(65535==this.data[n]){if(1==this.data[n+1])n=R(this.data,n+2);else break}if((1&this.data[n+2])==0){let e=this.data[n+1];t.some((t,n)=>1&n&&t==e)||t.push(this.data[n],e)}}return t}configure(e){let t=Object.assign(Object.create(T.prototype),this);if(e.props&&(t.nodeSet=this.nodeSet.extend(...e.props)),e.top){let n=this.topRules[e.top];if(!n)throw RangeError(`Invalid top rule name ${e.top}`);t.top=n}return e.tokenizers&&(t.tokenizers=this.tokenizers.map(t=>{let n=e.tokenizers.find(e=>e.from==t);return n?n.to:t})),e.specializers&&(t.specializers=this.specializers.slice(),t.specializerSpecs=this.specializerSpecs.map((n,i)=>{let r=e.specializers.find(e=>e.from==n.external);if(!r)return n;let s=Object.assign(Object.assign({},n),{external:r.to});return t.specializers[i]=D(s),s})),e.contextTracker&&(t.context=e.contextTracker),e.dialect&&(t.dialect=this.parseDialect(e.dialect)),null!=e.strict&&(t.strict=e.strict),e.wrap&&(t.wrappers=t.wrappers.concat(e.wrap)),null!=e.bufferLength&&(t.bufferLength=e.bufferLength),t}hasWrappers(){return this.wrappers.length>0}getName(e){return this.termNames?this.termNames[e]:String(e<=this.maxNode&&this.nodeSet.types[e].name||e)}get eofTerm(){return this.maxNode+1}get topNode(){return this.nodeSet.types[this.top[1]]}dynamicPrecedence(e){let t=this.dynamicPrecedences;return null==t?0:t[e]||0}parseDialect(e){let t=Object.keys(this.dialects),n=t.map(()=>!1);if(e)for(let i of e.split(" ")){let e=t.indexOf(i);e>=0&&(n[e]=!0)}let i=null;for(let e=0;e<t.length;e++)if(!n[e])for(let n=this.dialects[t[e]],r;65535!=(r=this.data[n++]);)(i||(i=new Uint8Array(this.maxTerm+1)))[r]=1;return new P(e,n,i)}static deserialize(e){return new T(e)}}function R(e,t){return e[t]|e[t+1]<<16}function D(e){if(e.external){let t=e.extend?1:0;return(n,i)=>e.external(n,i)<<1|t}return e.get}var M=n(35524);class I{constructor(e,t,n){this.parent=e,this.depth=t,this.type=n,this.hash=(e?e.hash+e.hash<<8:0)+t+(t<<4)+n}}function Q(e,t){for(let n=0,i=t-e.pos-1;;i--,n++){let t=e.peek(i);if(E(t)||-1==t)return n}}function N(e){return 32==e||9==e}function E(e){return 10==e||13==e}function B(e){var t;return e<0||N(t=e)||E(t)}I.top=new I(null,-1,0);let L=new A({start:I.top,reduce:(e,t)=>3==e.type&&(20==t||34==t)?e.parent:e,shift(e,t,n,i){if(3==t)return new I(e,Q(i,i.pos),1);if(65==t||5==t)return new I(e,Q(i,i.pos),2);if(63==t)return e.parent;if(19==t||33==t)return new I(e,0,3);if(13==t&&4==e.type)return e.parent;if(47==t){let t=/[1-9]/.exec(i.read(i.pos,n.pos));if(t)return new I(e,e.depth+ +t[0],4)}return e},hash:e=>e.hash});function $(e,t,n=0){return e.peek(n)==t&&e.peek(n+1)==t&&e.peek(n+2)==t&&B(e.peek(n+3))}let j=new m((e,t)=>{if(-1==e.next&&t.canShift(64))return e.acceptToken(64);let n=e.peek(-1);if((E(n)||n<0)&&3!=t.context.type){if($(e,45)){if(!t.canShift(63))return e.acceptToken(1,3);e.acceptToken(63)}if($(e,46)){if(!t.canShift(63))return e.acceptToken(2,3);e.acceptToken(63)}let n=0;for(;32==e.next;)n++,e.advance();!(n<t.context.depth)&&(n!=t.context.depth||1!=t.context.type||45==e.next&&B(e.peek(1)))||-1==e.next||E(e.next)||35==e.next||e.acceptToken(63,-n)}},{contextual:!0}),z=new m((e,t)=>{if(3==t.context.type){63==e.next&&(e.advance(),B(e.next)&&e.acceptToken(7));return}if(45==e.next)e.advance(),B(e.next)&&e.acceptToken(1==t.context.type&&t.context.depth==Q(e,e.pos-1)?4:3);else if(63==e.next)e.advance(),B(e.next)&&e.acceptToken(2==t.context.type&&t.context.depth==Q(e,e.pos-1)?6:5);else{let n=e.pos;for(;;)if(N(e.next)){if(e.pos==n)return;e.advance()}else if(33==e.next)J(e);else if(38==e.next)W(e);else if(42==e.next){W(e);break}else if(39==e.next||34==e.next){if(_(e,!0))break;return}else if(91==e.next||123==e.next){if(!function(e){for(let t=[],n=e.pos+1024;;)if(91==e.next||123==e.next)t.push(e.next),e.advance();else if(39==e.next||34==e.next){if(!_(e,!0))return!1}else if(93==e.next||125==e.next){if(t[t.length-1]!=e.next-2)return!1;if(t.pop(),e.advance(),!t.length)return!0}else{if(e.next<0||e.pos>n||E(e.next))return!1;e.advance()}}(e))return;break}else{V(e,!0,!1,0);break}for(;N(e.next);)e.advance();if(58==e.next){if(e.pos==n&&t.canShift(29))return;B(e.peek(1))&&e.acceptTokenTo(2==t.context.type&&t.context.depth==Q(e,n)?66:65,n)}}},{contextual:!0});function X(e){return e>=48&&e<=57||e>=97&&e<=102||e>=65&&e<=70}function q(e,t){var n;return 37==e.next?(e.advance(),X(e.next)&&e.advance(),X(e.next)&&e.advance(),!0):((n=e.next)>32&&n<127&&34!=n&&37!=n&&44!=n&&60!=n&&62!=n&&92!=n&&94!=n&&96!=n&&123!=n&&124!=n&&125!=n||!!t&&44==e.next)&&(e.advance(),!0)}function J(e){if(e.advance(),60==e.next){for(e.advance();;)if(!q(e,!0)){62==e.next&&e.advance();break}}else for(;q(e,!1););}function W(e){for(e.advance();!B(e.next)&&"f"!=F(e.tag);)e.advance()}function _(e,t){let n=e.next,i=!1,r=e.pos;for(e.advance();;){let s=e.next;if(s<0)break;if(e.advance(),s==n){if(39==s){if(39==e.next)e.advance();else break}else break}else if(92==s&&34==n)e.next>=0&&e.advance();else if(E(s)){if(t)return!1;i=!0}else if(t&&e.pos>=r+1024)return!1}return!i}function F(e){return e<33?"u":e>125?"s":"iiisiiissisfissssssssssssisssiiissssssssssssssssssssssssssfsfssissssssssssssssssssssssssssfif"[e-33]}function U(e,t){let n=F(e);return"u"!=n&&!(t&&"f"==n)}function V(e,t,n,i){if(!("s"==F(e.next)||(63==e.next||58==e.next||45==e.next)&&U(e.peek(1),n)))return!1;e.advance();let r=e.pos;for(;;){var s;let o=e.next,l=0,a=i+1;for(;N(s=o)||E(s);){if(E(o)){if(t)return!1;a=0}else a++;o=e.peek(++l)}if(!(o>=0&&(58==o?U(e.peek(l+1),n):35==o?32!=e.peek(l-1):U(o,n)))||!n&&a<=i||0==a&&!n&&($(e,45,l)||$(e,46,l)))break;if(t&&"f"==F(o))return!1;for(let t=l;t>=0;t--)e.advance();if(t&&e.pos>r+1024)return!1}return!0}let Z=new m((e,t)=>{if(33==e.next)J(e),e.acceptToken(12);else if(38==e.next||42==e.next){let t=38==e.next?10:11;W(e),e.acceptToken(t)}else 39==e.next||34==e.next?(_(e,!1),e.acceptToken(9)):V(e,!1,3==t.context.type,t.context.depth)&&e.acceptToken(8)}),G=new m((e,t)=>{let n=4==t.context.type?t.context.depth:-1,i=e.pos;e:for(;;){let r=0,s=e.next;for(;32==s;)s=e.peek(++r);if(!r&&($(e,45,r)||$(e,46,r))||!E(s)&&(n<0&&(n=Math.max(t.context.depth+1,r)),r<n))break;for(;;){if(e.next<0)break e;let t=E(e.next);if(e.advance(),t)continue e;i=e.pos}}e.acceptTokenTo(13,i)}),Y=(0,M.Gv)({DirectiveName:M.pJ.keyword,DirectiveContent:M.pJ.attributeValue,"DirectiveEnd DocEnd":M.pJ.meta,QuotedLiteral:M.pJ.string,BlockLiteralHeader:M.pJ.special(M.pJ.string),BlockLiteralContent:M.pJ.content,Literal:M.pJ.content,"Key/Literal Key/QuotedLiteral":M.pJ.definition(M.pJ.propertyName),"Anchor Alias":M.pJ.labelName,Tag:M.pJ.typeName,Comment:M.pJ.lineComment,": , -":M.pJ.separator,"?":M.pJ.punctuation,"[ ]":M.pJ.squareBracket,"{ }":M.pJ.brace}),H=T.deserialize({version:14,states:"5lQ!ZQgOOO#PQfO'#CpO#uQfO'#DOOOQR'#Dv'#DvO$qQgO'#DRO%gQdO'#DUO%nQgO'#DUO&ROaO'#D[OOQR'#Du'#DuO&{QgO'#D^O'rQgO'#D`OOQR'#Dt'#DtO(iOqO'#DbOOQP'#Dj'#DjO(zQaO'#CmO)YQgO'#CmOOQP'#Cm'#CmQ)jQaOOQ)uQgOOQ]QgOOO*PQdO'#CrO*nQdO'#CtOOQO'#Dw'#DwO+]Q`O'#CxO+hQdO'#CwO+rQ`O'#CwOOQO'#Cv'#CvO+wQdO'#CvOOQO'#Cq'#CqO,UQ`O,59[O,^QfO,59[OOQR,59[,59[OOQO'#Cx'#CxO,eQ`O'#DPO,pQdO'#DPOOQO'#Dx'#DxO,zQdO'#DxO-XQ`O,59jO-aQfO,59jOOQR,59j,59jOOQR'#DS'#DSO-hQcO,59mO-sQgO'#DVO.TQ`O'#DVO.YQcO,59pOOQR'#DX'#DXO#|QfO'#DWO.hQcO'#DWOOQR,59v,59vO.yOWO,59vO/OOaO,59vO/WOaO,59vO/cQgO'#D_OOQR,59x,59xO0VQgO'#DaOOQR,59z,59zOOQP,59|,59|O0yOaO,59|O1ROaO,59|O1aOqO,59|OOQP-E7h-E7hO1oQgO,59XOOQP,59X,59XO2PQaO'#DeO2_QgO'#DeO2oQgO'#DkOOQP'#Dk'#DkQ)jQaOOO3PQdO'#CsOOQO,59^,59^O3kQdO'#CuOOQO,59`,59`OOQO,59c,59cO4VQdO,59cO4aQdO'#CzO4kQ`O'#CzOOQO,59b,59bOOQU,5:Q,5:QOOQR1G.v1G.vO4pQ`O1G.vOOQU-E7d-E7dO4xQdO,59kOOQO,59k,59kO5SQdO'#DQO5^Q`O'#DQOOQO,5:d,5:dOOQU,5:R,5:ROOQR1G/U1G/UO5cQ`O1G/UOOQU-E7e-E7eO5kQgO'#DhO5xQcO1G/XOOQR1G/X1G/XOOQR,59q,59qO6TQgO,59qO6eQdO'#DiO6lQgO'#DiO7PQcO1G/[OOQR1G/[1G/[OOQR,59r,59rO#|QfO,59rOOQR1G/b1G/bO7_OWO1G/bO7dOaO1G/bOOQR,59y,59yOOQR,59{,59{OOQP1G/h1G/hO7lOaO1G/hO7tOaO1G/hO8POaO1G/hOOQP1G.s1G.sO8_QgO,5:POOQP,5:P,5:POOQP,5:V,5:VOOQP-E7i-E7iOOQO,59_,59_OOQO,59a,59aOOQO1G.}1G.}OOQO,59f,59fO8oQdO,59fOOQR7+$b7+$bP,XQ`O'#DfOOQO1G/V1G/VOOQO,59l,59lO8yQdO,59lOOQR7+$p7+$pP9TQ`O'#DgOOQR'#DT'#DTOOQR,5:S,5:SOOQR-E7f-E7fOOQR7+$s7+$sOOQR1G/]1G/]O9YQgO'#DYO9jQ`O'#DYOOQR,5:T,5:TO#|QfO'#DZO9oQcO'#DZOOQR-E7g-E7gOOQR7+$v7+$vOOQR1G/^1G/^OOQR7+$|7+$|O:QOWO7+$|OOQP7+%S7+%SO:VOaO7+%SO:_OaO7+%SOOQP1G/k1G/kOOQO1G/Q1G/QOOQO1G/W1G/WOOQR,59t,59tO:jQgO,59tOOQR,59u,59uO#|QfO,59uOOQR<<Hh<<HhOOQP<<Hn<<HnO:zOaO<<HnOOQR1G/`1G/`OOQR1G/a1G/aOOQPAN>YAN>Y",stateData:";S~O!fOS!gOS^OS~OP_OQbORSOTUOWROXROYYOZZO[XOcPOqQO!PVO!V[O!cTO~O`cO~P]OVkOWROXROYeOZfO[dOcPOmhOqQO~OboO~P!bOVtOWROXROYeOZfO[dOcPOmrOqQO~OpwO~P#WORSOTUOWROXROYYOZZO[XOcPOqQO!PVO!cTO~OSvP!avP!bvP~P#|OWROXROYeOZfO[dOcPOqQO~OmzO~P%OOm!OOUzP!azP!bzP!dzP~P#|O^!SO!b!QO!f!TO!g!RO~ORSOTUOWROXROcPOqQO!PVO!cTO~OY!UOP!QXQ!QX!V!QX!`!QXS!QX!a!QX!b!QXU!QXm!QX!d!QX~P&aO[!WOP!SXQ!SX!V!SX!`!SXS!SX!a!SX!b!SXU!SXm!SX!d!SX~P&aO^!ZO!W![O!b!YO!f!]O!g!YO~OP!_O!V[OQaX!`aX~OPaXQaX!VaX!`aX~P#|OP!bOQ!cO!V[O~OP_O!V[O~P#|OWROXROY!fOcPOqQObfXmfXofXpfX~OWROXRO[!hOcPOqQObhXmhXohXphX~ObeXmlXoeX~ObkXokX~P%OOm!kO~Om!lObnPonP~P%OOb!pOo!oO~Ob!pO~P!bOm!sOosXpsX~OosXpsX~P%OOm!uOotPptP~P%OOo!xOp!yO~Op!yO~P#WOS!|O!a#OO!b#OO~OUyX!ayX!byX!dyX~P#|Om#QO~OU#SO!a#UO!b#UO!d#RO~Om#WOUzX!azX!bzX!dzX~O]#XO~O!b#XO!g#YO~O^#ZO!b#XO!g#YO~OP!RXQ!RX!V!RX!`!RXS!RX!a!RX!b!RXU!RXm!RX!d!RX~P&aOP!TXQ!TX!V!TX!`!TXS!TX!a!TX!b!TXU!TXm!TX!d!TX~P&aO!b#^O!g#^O~O^#_O!b#^O!f#`O!g#^O~O^#_O!W#aO!b#^O!g#^O~OPaaQaa!Vaa!`aa~P#|OP#cO!V[OQ!XX!`!XX~OP!XXQ!XX!V!XX!`!XX~P#|OP_O!V[OQ!_X!`!_X~P#|OWROXROcPOqQObgXmgXogXpgX~OWROXROcPOqQObiXmiXoiXpiX~Obkaoka~P%OObnXonX~P%OOm#kO~Ob#lOo!oO~Oosapsa~P%OOotXptX~P%OOm#pO~Oo!xOp#qO~OSwP!awP!bwP~P#|OS!|O!a#vO!b#vO~OUya!aya!bya!dya~P#|Om#xO~P%OOm#{OU}P!a}P!b}P!d}P~P#|OU#SO!a$OO!b$OO!d#RO~O]$QO~O!b$QO!g$RO~O!b$SO!g$SO~O^$TO!b$SO!g$SO~O^$TO!b$SO!f$UO!g$SO~OP!XaQ!Xa!V!Xa!`!Xa~P#|Obnaona~P%OOotapta~P%OOo!xO~OU|X!a|X!b|X!d|X~P#|Om$ZO~Om$]OU}X!a}X!b}X!d}X~O]$^O~O!b$_O!g$_O~O^$`O!b$_O!g$_O~OU|a!a|a!b|a!d|a~P#|O!b$cO!g$cO~O",goto:",]!mPPPPPPPPPPPPPPPPP!nPP!v#v#|$`#|$c$f$j$nP%VPPP!v%Y%^%a%{&O%a&R&U&X&_&b%aP&e&{&e'O'RPP']'a'g'm's'y(XPPPPPPPP(_)e*X+c,VUaObcR#e!c!{ROPQSTUXY_bcdehknrtvz!O!U!W!_!b!c!f!h!k!l!s!u!|#Q#R#S#W#c#k#p#x#{$Z$]QmPR!qnqfPQThknrtv!k!l!s!u#R#k#pR!gdR!ieTlPnTjPnSiPnSqQvQ{TQ!mkQ!trQ!vtR#y#RR!nkTsQvR!wt!RWOSUXY_bcz!O!U!W!_!b!c!|#Q#S#W#c#x#{$Z$]RySR#t!|R|TR|UQ!PUR#|#SR#z#RR#z#SyZOSU_bcz!O!_!b!c!|#Q#S#W#c#x#{$Z$]R!VXR!XYa]O^abc!a!c!eT!da!eQnPR!rnQvQR!{vQ!}yR#u!}Q#T|R#}#TW^Obc!cS!^^!aT!aa!eQ!eaR#f!eW`Obc!cQxSS}U#SQ!`_Q#PzQ#V!OQ#b!_Q#d!bQ#s!|Q#w#QQ$P#WQ$V#cQ$Y#xQ$[#{Q$a$ZR$b$]xZOSU_bcz!O!_!b!c!|#Q#S#W#c#x#{$Z$]Q!VXQ!XYQ#[!UR#]!W!QWOSUXY_bcz!O!U!W!_!b!c!|#Q#S#W#c#x#{$Z$]pfPQThknrtv!k!l!s!u#R#k#pQ!gdQ!ieQ#g!fR#h!hSgPn^pQTkrtv#RQ!jhQ#i!kQ#j!lQ#n!sQ#o!uQ$W#kR$X#pQuQR!zv",nodeNames:"⚠ DirectiveEnd DocEnd - - ? ? ? Literal QuotedLiteral Anchor Alias Tag BlockLiteralContent Comment Stream BOM Document ] [ FlowSequence Item Tagged Anchored Anchored Tagged FlowMapping Pair Key : Pair , } { FlowMapping Pair Pair BlockSequence Item Item BlockMapping Pair Pair Key Pair Pair BlockLiteral BlockLiteralHeader Tagged Anchored Anchored Tagged Directive DirectiveName DirectiveContent Document",maxTerm:74,context:L,nodeProps:[["isolate",-3,8,9,14,""],["openedBy",18,"[",32,"{"],["closedBy",19,"]",33,"}"]],propSources:[Y],skippedNodes:[0],repeatNodeCount:6,tokenData:"-Y~RnOX#PXY$QYZ$]Z]#P]^$]^p#Ppq$Qqs#Pst$btu#Puv$yv|#P|}&e}![#P![!]'O!]!`#P!`!a'i!a!}#P!}#O*g#O#P#P#P#Q+Q#Q#o#P#o#p+k#p#q'i#q#r,U#r;'S#P;'S;=`#z<%l?HT#P?HT?HU,o?HUO#PQ#UU!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PQ#kTOY#PZs#Pt;'S#P;'S;=`#z<%lO#PQ#}P;=`<%l#P~$VQ!f~XY$Qpq$Q~$bO!g~~$gS^~OY$bZ;'S$b;'S;=`$s<%lO$b~$vP;=`<%l$bR%OX!WQOX%kXY#PZ]%k]^#P^p%kpq#hq;'S%k;'S;=`&_<%lO%kR%rX!WQ!VPOX%kXY#PZ]%k]^#P^p%kpq#hq;'S%k;'S;=`&_<%lO%kR&bP;=`<%l%kR&lUoP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR'VUmP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR'p[!PP!WQOY#PZp#Ppq#hq{#P{|(f|}#P}!O(f!O!R#P!R![)p![;'S#P;'S;=`#z<%lO#PR(mW!PP!WQOY#PZp#Ppq#hq!R#P!R![)V![;'S#P;'S;=`#z<%lO#PR)^U!PP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR)wY!PP!WQOY#PZp#Ppq#hq{#P{|)V|}#P}!O)V!O;'S#P;'S;=`#z<%lO#PR*nUcP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR+XUbP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR+rUqP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR,]UpP!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#PR,vU`P!WQOY#PZp#Ppq#hq;'S#P;'S;=`#z<%lO#P",tokenizers:[j,z,Z,G,0,1],topRules:{Stream:[0,15]},tokenPrec:0});var K=n(59119);let ee=K.qp.define({name:"yaml",parser:H.configure({props:[K.uj.add({Stream:e=>{for(let t=e.node.resolve(e.pos,-1);t&&t.to>=e.pos;t=t.parent){if("BlockLiteralContent"==t.name&&t.from<t.to)return e.baseIndentFor(t);if("BlockLiteral"==t.name)return e.baseIndentFor(t)+e.unit;if("BlockSequence"==t.name||"BlockMapping"==t.name)return e.column(t.from,1);if("QuotedLiteral"==t.name)break;if("Literal"==t.name){let n=e.column(t.from,1);if(n==e.lineIndent(t.from,1))return n;if(t.to>e.pos)break}}return null},FlowMapping:(0,K.vw)({closing:"}"}),FlowSequence:(0,K.vw)({closing:"]"})}),K.x0.add({"FlowMapping FlowSequence":K.Dv,"Item Pair BlockLiteral":(e,t)=>({from:t.doc.lineAt(e.from).to,to:e.to})})]}),languageData:{commentTokens:{line:"#"},indentOnInput:/^\s*[\]\}]$/}});function et(){return new K.ri(ee)}M.pJ.meta},59119:function(e,t,n){n.d(t,{Dv:function(){return z},Gn:function(){return D},K0:function(){return R},Qf:function(){return ea},R_:function(){return em},SS:function(){return T},Um:function(){return eA},c:function(){return C},e7:function(){return Y},mi:function(){return eo},n$:function(){return ew},nF:function(){return ef},nY:function(){return L},qp:function(){return d},qz:function(){return p},ri:function(){return S},uj:function(){return M},vw:function(){return E},x0:function(){return j},y1:function(){return A}});var i,r=n(41113),s=n(37205),o=n(47421),l=n(35524),a=n(28699);let h=new r.md,c=new r.md;class u{constructor(e,t,n=[],i=""){this.data=e,this.name=i,s.yy.prototype.hasOwnProperty("tree")||Object.defineProperty(s.yy.prototype,"tree",{get(){return p(this)}}),this.parser=t,this.extension=[w.of(this),s.yy.languageData.of((e,t,n)=>{let i=f(e,t,n),r=i.type.prop(h);if(!r)return[];let s=e.facet(r),o=i.type.prop(c);if(o){let r=i.resolve(t-i.from,n);for(let t of o)if(t.test(r,e)){let n=e.facet(t.facet);return"replace"==t.type?n:n.concat(s)}}return s})].concat(n)}isActiveAt(e,t,n=-1){return f(e,t,n).type.prop(h)==this.data}findRegions(e){let t=e.facet(w);if((null==t?void 0:t.data)==this.data)return[{from:0,to:e.doc.length}];if(!t||!t.allowsNesting)return[];let n=[],i=(e,t)=>{if(e.prop(h)==this.data){n.push({from:t,to:t+e.length});return}let s=e.prop(r.md.mounted);if(s){if(s.tree.prop(h)==this.data){if(s.overlay)for(let e of s.overlay)n.push({from:e.from+t,to:e.to+t});else n.push({from:t,to:t+e.length});return}if(s.overlay){let e=n.length;if(i(s.tree,s.overlay[0].from+t),n.length>e)return}}for(let n=0;n<e.children.length;n++){let s=e.children[n];s instanceof r.mp&&i(s,e.positions[n]+t)}};return i(p(e),0),n}get allowsNesting(){return!0}}function f(e,t,n){let i=e.facet(w),s=p(e).topNode;if(!i||i.allowsNesting)for(let e=s;e;e=e.enter(t,n,r.vj.ExcludeBuffers|r.vj.EnterBracketed))e.type.isTop&&(s=e);return s}u.setState=s.Py.define();class d extends u{constructor(e,t,n){super(e,t,[],n),this.parser=t}static define(e){var t;let n=(t=e.languageData,s.r$.define({combine:t?e=>e.concat(t):void 0}));return new d(n,e.parser.configure({props:[h.add(e=>e.isTop?n:void 0)]}),e.name)}configure(e,t){return new d(this.data,this.parser.configure(e),t||this.name)}get allowsNesting(){return this.parser.hasWrappers()}}function p(e){let t=e.field(u.state,!1);return t?t.tree:r.mp.empty}class m{constructor(e){this.doc=e,this.cursorPos=0,this.string="",this.cursor=e.iter()}get length(){return this.doc.length}syncTo(e){return this.string=this.cursor.next(e-this.cursorPos).value,this.cursorPos=e+this.string.length,this.cursorPos-this.string.length}chunk(e){return this.syncTo(e),this.string}get lineChunks(){return!0}read(e,t){let n=this.cursorPos-this.string.length;return e<n||t>=this.cursorPos?this.doc.sliceString(e,t):this.string.slice(e-n,t-n)}}let g=null;class k{constructor(e,t,n=[],i,r,s,o,l){this.parser=e,this.state=t,this.fragments=n,this.tree=i,this.treeLen=r,this.viewport=s,this.skipped=o,this.scheduleOn=l,this.parse=null,this.tempSkipped=[]}static create(e,t,n){return new k(e,t,[],r.mp.empty,0,n,[],null)}startParse(){return this.parser.startParse(new m(this.state.doc),this.fragments)}work(e,t){return(null!=t&&t>=this.state.doc.length&&(t=void 0),this.tree!=r.mp.empty&&this.isDone(null!=t?t:this.state.doc.length))?(this.takeTree(),!0):this.withContext(()=>{var n;if("number"==typeof e){let t=Date.now()+e;e=()=>Date.now()>t}for(this.parse||(this.parse=this.startParse()),null!=t&&(null==this.parse.stoppedAt||this.parse.stoppedAt>t)&&t<this.state.doc.length&&this.parse.stopAt(t);;){let i=this.parse.advance();if(i){if(this.fragments=this.withoutTempSkipped(r.i9.addTree(i,this.fragments,null!=this.parse.stoppedAt)),this.treeLen=null!==(n=this.parse.stoppedAt)&&void 0!==n?n:this.state.doc.length,this.tree=i,this.parse=null,!(this.treeLen<(null!=t?t:this.state.doc.length)))return!0;this.parse=this.startParse()}if(e())return!1}})}takeTree(){let e,t;this.parse&&(e=this.parse.parsedPos)>=this.treeLen&&((null==this.parse.stoppedAt||this.parse.stoppedAt>e)&&this.parse.stopAt(e),this.withContext(()=>{for(;!(t=this.parse.advance()););}),this.treeLen=e,this.tree=t,this.fragments=this.withoutTempSkipped(r.i9.addTree(this.tree,this.fragments,!0)),this.parse=null)}withContext(e){let t=g;g=this;try{return e()}finally{g=t}}withoutTempSkipped(e){for(let t;t=this.tempSkipped.pop();)e=v(e,t.from,t.to);return e}changes(e,t){let{fragments:n,tree:i,treeLen:s,viewport:o,skipped:l}=this;if(this.takeTree(),!e.empty){let t=[];if(e.iterChangedRanges((e,n,i,r)=>t.push({fromA:e,toA:n,fromB:i,toB:r})),n=r.i9.applyChanges(n,t),i=r.mp.empty,s=0,o={from:e.mapPos(o.from,-1),to:e.mapPos(o.to,1)},this.skipped.length)for(let t of(l=[],this.skipped)){let n=e.mapPos(t.from,1),i=e.mapPos(t.to,-1);n<i&&l.push({from:n,to:i})}}return new k(this.parser,t,n,i,s,o,l,this.scheduleOn)}updateViewport(e){if(this.viewport.from==e.from&&this.viewport.to==e.to)return!1;this.viewport=e;let t=this.skipped.length;for(let t=0;t<this.skipped.length;t++){let{from:n,to:i}=this.skipped[t];n<e.to&&i>e.from&&(this.fragments=v(this.fragments,n,i),this.skipped.splice(t--,1))}return!(this.skipped.length>=t)&&(this.reset(),!0)}reset(){this.parse&&(this.takeTree(),this.parse=null)}skipUntilInView(e,t){this.skipped.push({from:e,to:t})}static getSkippingParser(e){return new class extends r._b{createParse(t,n,i){let s=i[0].from,o=i[i.length-1].to;return{parsedPos:s,advance(){let t=g;if(t){for(let e of i)t.tempSkipped.push(e);e&&(t.scheduleOn=t.scheduleOn?Promise.all([t.scheduleOn,e]):e)}return this.parsedPos=o,new r.mp(r.Jq.none,[],[],o-s)},stoppedAt:null,stopAt(){}}}}}isDone(e){e=Math.min(e,this.state.doc.length);let t=this.fragments;return this.treeLen>=e&&t.length&&0==t[0].from&&t[0].to>=e}static get(){return g}}function v(e,t,n){return r.i9.applyChanges(e,[{fromA:t,toA:n,fromB:t,toB:n}])}class y{constructor(e){this.context=e,this.tree=e.tree}apply(e){if(!e.docChanged&&this.tree==this.context.tree)return this;let t=this.context.changes(e.changes,e.state),n=this.context.treeLen==e.startState.doc.length?void 0:Math.max(e.changes.mapPos(this.context.treeLen),t.viewport.to);return t.work(20,n)||t.takeTree(),new y(t)}static init(e){let t=Math.min(3e3,e.doc.length),n=k.create(e.facet(w).parser,e,{from:0,to:t});return n.work(20,t)||n.takeTree(),new y(n)}}u.state=s.QQ.define({create:y.init,update(e,t){for(let e of t.effects)if(e.is(u.setState))return e.value;return t.startState.facet(w)!=t.state.facet(w)?y.init(t.state):e.apply(t)}});let x=e=>{let t=setTimeout(()=>e(),500);return()=>clearTimeout(t)};"undefined"!=typeof requestIdleCallback&&(x=e=>{let t=-1,n=setTimeout(()=>{t=requestIdleCallback(e,{timeout:400})},100);return()=>t<0?clearTimeout(n):cancelIdleCallback(t)});let b="undefined"!=typeof navigator&&(null===(i=navigator.scheduling)||void 0===i?void 0:i.isInputPending)?()=>navigator.scheduling.isInputPending():null,O=o.lg.fromClass(class{constructor(e){this.view=e,this.working=null,this.workScheduled=0,this.chunkEnd=-1,this.chunkBudget=-1,this.work=this.work.bind(this),this.scheduleWork()}update(e){let t=this.view.state.field(u.state).context;(t.updateViewport(e.view.viewport)||this.view.viewport.to>t.treeLen)&&this.scheduleWork(),(e.docChanged||e.selectionSet)&&(this.view.hasFocus&&(this.chunkBudget+=50),this.scheduleWork()),this.checkAsyncSchedule(t)}scheduleWork(){if(this.working)return;let{state:e}=this.view,t=e.field(u.state);t.tree==t.context.tree&&t.context.isDone(e.doc.length)||(this.working=x(this.work))}work(e){this.working=null;let t=Date.now();if(this.chunkEnd<t&&(this.chunkEnd<0||this.view.hasFocus)&&(this.chunkEnd=t+3e4,this.chunkBudget=3e3),this.chunkBudget<=0)return;let{state:n,viewport:{to:i}}=this.view,r=n.field(u.state);if(r.tree==r.context.tree&&r.context.isDone(i+1e5))return;let s=Date.now()+Math.min(this.chunkBudget,100,e&&!b?Math.max(25,e.timeRemaining()-5):1e9),o=r.context.treeLen<i&&n.doc.length>i+1e3,l=r.context.work(()=>b&&b()||Date.now()>s,i+(o?0:1e5));this.chunkBudget-=Date.now()-t,(l||this.chunkBudget<=0)&&(r.context.takeTree(),this.view.dispatch({effects:u.setState.of(new y(r.context))})),this.chunkBudget>0&&!(l&&!o)&&this.scheduleWork(),this.checkAsyncSchedule(r.context)}checkAsyncSchedule(e){e.scheduleOn&&(this.workScheduled++,e.scheduleOn.then(()=>this.scheduleWork()).catch(e=>(0,o.OO)(this.view.state,e)).then(()=>this.workScheduled--),e.scheduleOn=null)}destroy(){this.working&&this.working()}isWorking(){return!!(this.working||this.workScheduled>0)}},{eventHandlers:{focus(){this.scheduleWork()}}}),w=s.r$.define({combine:e=>e.length?e[0]:null,enables:e=>[u.state,O,o.tk.contentAttributes.compute([e],t=>{let n=t.facet(e);return n&&n.name?{"data-language":n.name}:{}})]});class S{constructor(e,t=[]){this.language=e,this.support=t,this.extension=[e,t]}}let P=s.r$.define(),C=s.r$.define({combine:e=>{if(!e.length)return"  ";let t=e[0];if(!t||/\S/.test(t)||Array.from(t).some(e=>e!=t[0]))throw Error("Invalid indent unit: "+JSON.stringify(e[0]));return t}});function A(e){let t=e.facet(C);return 9==t.charCodeAt(0)?e.tabSize*t.length:t.length}function T(e,t){let n="",i=e.tabSize,r=e.facet(C)[0];if("	"==r){for(;t>=i;)n+="	",t-=i;r=" "}for(let e=0;e<t;e++)n+=r;return n}function R(e,t){for(let n of(e instanceof s.yy&&(e=new D(e)),e.state.facet(P))){let i=n(e,t);if(void 0!==i)return i}let n=p(e.state);return n.length>=t?function(e,t,n){let i=t.resolveStack(n),r=t.resolveInner(n,-1).resolve(n,0).enterUnfinishedNodesBefore(n);if(r!=i.node){let e=[];for(let t=r;t&&!(t.from<i.node.from||t.to>i.node.to||t.from==i.node.from&&t.type==i.node.type);t=t.parent)e.push(t);for(let t=e.length-1;t>=0;t--)i={node:e[t],next:i}}return I(i,e,n)}(e,n,t):null}class D{constructor(e,t={}){this.state=e,this.options=t,this.unit=A(e)}lineAt(e,t=1){let n=this.state.doc.lineAt(e),{simulateBreak:i,simulateDoubleBreak:r}=this.options;return null!=i&&i>=n.from&&i<=n.to?r&&i==e?{text:"",from:e}:(t<0?i<e:i<=e)?{text:n.text.slice(i-n.from),from:i}:{text:n.text.slice(0,i-n.from),from:n.from}:n}textAfterPos(e,t=1){if(this.options.simulateDoubleBreak&&e==this.options.simulateBreak)return"";let{text:n,from:i}=this.lineAt(e,t);return n.slice(e-i,Math.min(n.length,e+100-i))}column(e,t=1){let{text:n,from:i}=this.lineAt(e,t),r=this.countColumn(n,e-i),s=this.options.overrideIndentation?this.options.overrideIndentation(i):-1;return s>-1&&(r+=s-this.countColumn(n,n.search(/\S|$/))),r}countColumn(e,t=e.length){return(0,s.IS)(e,this.state.tabSize,t)}lineIndent(e,t=1){let{text:n,from:i}=this.lineAt(e,t),r=this.options.overrideIndentation;if(r){let e=r(i);if(e>-1)return e}return this.countColumn(n,n.search(/\S|$/))}get simulatedBreak(){return this.options.simulateBreak||null}}let M=new r.md;function I(e,t,n){for(let i=e;i;i=i.next){let e=function(e){let t=e.type.prop(M);if(t)return t;let n=e.firstChild,i;if(n&&(i=n.type.prop(r.md.closedBy))){let t=e.lastChild,n=t&&i.indexOf(t.name)>-1;return e=>B(e,!0,1,void 0,n&&!(e.pos==e.options.simulateBreak&&e.options.simulateDoubleBreak)?t.from:void 0)}return null==e.parent?Q:null}(i.node);if(e)return e(N.create(t,n,i))}return 0}function Q(){return 0}class N extends D{constructor(e,t,n){super(e.state,e.options),this.base=e,this.pos=t,this.context=n}get node(){return this.context.node}static create(e,t,n){return new N(e,t,n)}get textAfter(){return this.textAfterPos(this.pos)}get baseIndent(){return this.baseIndentFor(this.node)}baseIndentFor(e){let t=this.state.doc.lineAt(e.from);for(;;){let n=e.resolve(t.from);for(;n.parent&&n.parent.from==n.from;)n=n.parent;if(function(e,t){for(let n=t;n;n=n.parent)if(e==n)return!0;return!1}(n,e))break;t=this.state.doc.lineAt(n.from)}return this.lineIndent(t.from)}continue(){return I(this.context.next,this.base,this.pos)}}function E({closing:e,align:t=!0,units:n=1}){return i=>B(i,t,n,e)}function B(e,t,n,i,r){let s=e.textAfter,o=s.match(/^\s*/)[0].length,l=i&&s.slice(o,o+i.length)==i||r==e.pos+o,a=t?function(e){let t=e.node,n=t.childAfter(t.from),i=t.lastChild;if(!n)return null;let r=e.options.simulateBreak,s=e.state.doc.lineAt(n.from),o=null==r||r<=s.from?s.to:Math.min(s.to,r);for(let e=n.to;;){let r=t.childAfter(e);if(!r||r==i)return null;if(!r.type.isSkipped){if(r.from>=o)return null;let e=/^ */.exec(s.text.slice(n.to-s.from))[0].length;return{from:n.from,to:n.to+e}}e=r.to}}(e):null;return a?l?e.column(a.from):e.column(a.to):e.baseIndent+(l?0:e.unit*n)}function L(){return s.yy.transactionFilter.of(e=>{if(!e.docChanged||!e.isUserEvent("input.type")&&!e.isUserEvent("input.complete"))return e;let t=e.startState.languageDataAt("indentOnInput",e.startState.selection.main.head);if(!t.length)return e;let n=e.newDoc,{head:i}=e.newSelection.main,r=n.lineAt(i);if(i>r.from+200)return e;let s=n.sliceString(r.from,i);if(!t.some(e=>e.test(s)))return e;let{state:o}=e,l=-1,a=[];for(let{head:e}of o.selection.ranges){let t=o.doc.lineAt(e);if(t.from==l)continue;l=t.from;let n=R(o,t.from);if(null==n)continue;let i=/^\s*/.exec(t.text)[0],r=T(o,n);i!=r&&a.push({from:t.from,to:t.from+i.length,insert:r})}return a.length?[e,{changes:a,sequential:!0}]:e})}let $=s.r$.define(),j=new r.md;function z(e){let t=e.firstChild,n=e.lastChild;return t&&t.to<n.from?{from:t.to,to:n.type.isError?e.to:n.from}:null}function X(e,t,n){for(let i of e.facet($)){let r=i(e,t,n);if(r)return r}return function(e,t,n){let i=p(e);if(i.length<n)return null;let r=i.resolveStack(n,1),s=null;for(let o=r;o;o=o.next){let r=o.node;if(r.to<=n||r.from>n)continue;if(s&&r.from<t)break;let l=r.type.prop(j);if(l&&(r.to<i.length-50||i.length==e.doc.length||!function(e){let t=e.lastChild;return t&&t.to==e.to&&t.type.isError}(r))){let i=l(r,e);i&&i.from<=n&&i.from>=t&&i.to>n&&(s=i)}}return s}(e,t,n)}function q(e,t){let n=t.mapPos(e.from,1),i=t.mapPos(e.to,-1);return n>=i?void 0:{from:n,to:i}}let J=s.Py.define({map:q}),W=s.Py.define({map:q});function _(e){let t=[];for(let{head:n}of e.state.selection.ranges)t.some(e=>e.from<=n&&e.to>=n)||t.push(e.lineBlockAt(n));return t}let F=s.QQ.define({create:()=>o.p.none,update(e,t){for(let n of(t.isUserEvent("delete")&&t.changes.iterChangedRanges((t,n)=>e=U(e,t,n)),e=e.map(t.changes),t.effects))if(n.is(J)&&!function(e,t,n){let i=!1;return e.between(t,t,(e,r)=>{e==t&&r==n&&(i=!0)}),i}(e,n.value.from,n.value.to)){let{preparePlaceholder:i}=t.state.facet(K),r=i?o.p.replace({widget:new ei(i(t.state,n.value))}):en;e=e.update({add:[r.range(n.value.from,n.value.to)]})}else n.is(W)&&(e=e.update({filter:(e,t)=>n.value.from!=e||n.value.to!=t,filterFrom:n.value.from,filterTo:n.value.to}));return t.selection&&(e=U(e,t.selection.main.head)),e},provide:e=>o.tk.decorations.from(e),toJSON(e,t){let n=[];return e.between(0,t.doc.length,(e,t)=>{n.push(e,t)}),n},fromJSON(e){if(!Array.isArray(e)||e.length%2)throw RangeError("Invalid JSON for fold state");let t=[];for(let n=0;n<e.length;){let i=e[n++],r=e[n++];if("number"!=typeof i||"number"!=typeof r)throw RangeError("Invalid JSON for fold state");t.push(en.range(i,r))}return o.p.set(t,!0)}});function U(e,t,n=t){let i=!1;return e.between(t,n,(e,r)=>{e<n&&r>t&&(i=!0)}),i?e.update({filterFrom:t,filterTo:n,filter:(e,i)=>e>=n||i<=t}):e}function V(e,t,n){var i;let r=null;return null===(i=e.field(F,!1))||void 0===i||i.between(t,n,(e,t)=>{(!r||r.from>e)&&(r={from:e,to:t})}),r}function Z(e,t){return e.field(F,!1)?t:t.concat(s.Py.appendConfig.of(ee()))}function G(e,t,n=!0){let i=e.state.doc.lineAt(t.from).number,r=e.state.doc.lineAt(t.to).number;return o.tk.announce.of(`${e.state.phrase(n?"Folded lines":"Unfolded lines")} ${i} ${e.state.phrase("to")} ${r}.`)}let Y=[{key:"Ctrl-Shift-[",mac:"Cmd-Alt-[",run:e=>{for(let t of _(e)){let n=X(e.state,t.from,t.to);if(n)return e.dispatch({effects:Z(e.state,[J.of(n),G(e,n)])}),!0}return!1}},{key:"Ctrl-Shift-]",mac:"Cmd-Alt-]",run:e=>{if(!e.state.field(F,!1))return!1;let t=[];for(let n of _(e)){let i=V(e.state,n.from,n.to);i&&t.push(W.of(i),G(e,i,!1))}return t.length&&e.dispatch({effects:t}),t.length>0}},{key:"Ctrl-Alt-[",run:e=>{let{state:t}=e,n=[];for(let i=0;i<t.doc.length;){let r=e.lineBlockAt(i),s=X(t,r.from,r.to);s&&n.push(J.of(s)),i=(s?e.lineBlockAt(s.to):r).to+1}return n.length&&e.dispatch({effects:Z(e.state,n)}),!!n.length}},{key:"Ctrl-Alt-]",run:e=>{let t=e.state.field(F,!1);if(!t||!t.size)return!1;let n=[];return t.between(0,e.state.doc.length,(e,t)=>{n.push(W.of({from:e,to:t}))}),e.dispatch({effects:n}),!0}}],H={placeholderDOM:null,preparePlaceholder:null,placeholderText:"…"},K=s.r$.define({combine:e=>(0,s.BO)(e,H)});function ee(e){let t=[F,el];return e&&t.push(K.of(e)),t}function et(e,t){let{state:n}=e,i=n.facet(K),r=t=>{let n=e.lineBlockAt(e.posAtDOM(t.target)),i=V(e.state,n.from,n.to);i&&e.dispatch({effects:W.of(i)}),t.preventDefault()};if(i.placeholderDOM)return i.placeholderDOM(e,r,t);let s=document.createElement("span");return s.textContent=i.placeholderText,s.setAttribute("aria-label",n.phrase("folded code")),s.title=n.phrase("unfold"),s.className="cm-foldPlaceholder",s.onclick=r,s}let en=o.p.replace({widget:new class extends o.l9{toDOM(e){return et(e,null)}}});class ei extends o.l9{constructor(e){super(),this.value=e}eq(e){return this.value==e.value}toDOM(e){return et(e,this.value)}}let er={openText:"⌄",closedText:"›",markerDOM:null,domEventHandlers:{},foldingChanged:()=>!1};class es extends o.SJ{constructor(e,t){super(),this.config=e,this.open=t}eq(e){return this.config==e.config&&this.open==e.open}toDOM(e){if(this.config.markerDOM)return this.config.markerDOM(this.open);let t=document.createElement("span");return t.textContent=this.open?this.config.openText:this.config.closedText,t.title=e.state.phrase(this.open?"Fold line":"Unfold line"),t}}function eo(e={}){let t={...er,...e},n=new es(t,!0),i=new es(t,!1),r=o.lg.fromClass(class{constructor(e){this.from=e.viewport.from,this.markers=this.buildMarkers(e)}update(e){(e.docChanged||e.viewportChanged||e.startState.facet(w)!=e.state.facet(w)||e.startState.field(F,!1)!=e.state.field(F,!1)||p(e.startState)!=p(e.state)||t.foldingChanged(e))&&(this.markers=this.buildMarkers(e.view))}buildMarkers(e){let t=new s.f_;for(let r of e.viewportLineBlocks){let s=V(e.state,r.from,r.to)?i:X(e.state,r.from,r.to)?n:null;s&&t.add(r.from,r.from,s)}return t.finish()}}),{domEventHandlers:l}=t;return[r,(0,o.v5)({class:"cm-foldGutter",markers(e){var t;return(null===(t=e.plugin(r))||void 0===t?void 0:t.markers)||s.Xs.empty},initialSpacer:()=>new es(t,!1),domEventHandlers:{...l,click:(e,t,n)=>{if(l.click&&l.click(e,t,n))return!0;let i=V(e.state,t.from,t.to);if(i)return e.dispatch({effects:W.of(i)}),!0;let r=X(e.state,t.from,t.to);return!!r&&(e.dispatch({effects:J.of(r)}),!0)}}}),ee()]}let el=o.tk.baseTheme({".cm-foldPlaceholder":{backgroundColor:"#eee",border:"1px solid #ddd",color:"#888",borderRadius:".2em",margin:"0 1px",padding:"0 1px",cursor:"pointer"},".cm-foldGutter span":{padding:"0 1px",cursor:"pointer"}});class ea{constructor(e,t){let n;function i(e){let t=a.V.newName();return(n||(n=Object.create(null)))["."+t]=e,t}this.specs=e;let r="string"==typeof t.all?t.all:t.all?i(t.all):void 0,s=t.scope;this.scope=s instanceof u?e=>e.prop(h)==s.data:s?e=>e==s:void 0,this.style=(0,l.QR)(e.map(e=>({tag:e.tag,class:e.class||i(Object.assign({},e,{tag:null}))})),{all:r}).style,this.module=n?new a.V(n):null,this.themeType=t.themeType}static define(e,t){return new ea(e,t||{})}}let eh=s.r$.define(),ec=s.r$.define({combine:e=>e.length?[e[0]]:null});function eu(e){let t=e.facet(eh);return t.length?t:e.facet(ec)}function ef(e,t){let n=[ep],i;return e instanceof ea&&(e.module&&n.push(o.tk.styleModule.of(e.module)),i=e.themeType),(null==t?void 0:t.fallback)?n.push(ec.of(e)):i?n.push(eh.computeN([o.tk.darkTheme],t=>t.facet(o.tk.darkTheme)==("dark"==i)?[e]:[])):n.push(eh.of(e)),n}class ed{constructor(e){this.markCache=Object.create(null),this.tree=p(e.state),this.decorations=this.buildDeco(e,eu(e.state)),this.decoratedTo=e.viewport.to}update(e){let t=p(e.state),n=eu(e.state),i=n!=eu(e.startState),{viewport:r}=e.view,s=e.changes.mapPos(this.decoratedTo,1);t.length<r.to&&!i&&t.type==this.tree.type&&s>=r.to?(this.decorations=this.decorations.map(e.changes),this.decoratedTo=s):(t!=this.tree||e.viewportChanged||i)&&(this.tree=t,this.decorations=this.buildDeco(e.view,n),this.decoratedTo=r.to)}buildDeco(e,t){if(!t||!this.tree.length)return o.p.none;let n=new s.f_;for(let{from:i,to:r}of e.visibleRanges)(0,l.bW)(this.tree,t,(e,t,i)=>{n.add(e,t,this.markCache[i]||(this.markCache[i]=o.p.mark({class:i})))},i,r);return n.finish()}}let ep=s.Wl.high(o.lg.fromClass(ed,{decorations:e=>e.decorations})),em=ea.define([{tag:l.pJ.meta,color:"#404740"},{tag:l.pJ.link,textDecoration:"underline"},{tag:l.pJ.heading,textDecoration:"underline",fontWeight:"bold"},{tag:l.pJ.emphasis,fontStyle:"italic"},{tag:l.pJ.strong,fontWeight:"bold"},{tag:l.pJ.strikethrough,textDecoration:"line-through"},{tag:l.pJ.keyword,color:"#708"},{tag:[l.pJ.atom,l.pJ.bool,l.pJ.url,l.pJ.contentSeparator,l.pJ.labelName],color:"#219"},{tag:[l.pJ.literal,l.pJ.inserted],color:"#164"},{tag:[l.pJ.string,l.pJ.deleted],color:"#a11"},{tag:[l.pJ.regexp,l.pJ.escape,l.pJ.special(l.pJ.string)],color:"#e40"},{tag:l.pJ.definition(l.pJ.variableName),color:"#00f"},{tag:l.pJ.local(l.pJ.variableName),color:"#30a"},{tag:[l.pJ.typeName,l.pJ.namespace],color:"#085"},{tag:l.pJ.className,color:"#167"},{tag:[l.pJ.special(l.pJ.variableName),l.pJ.macroName],color:"#256"},{tag:l.pJ.definition(l.pJ.propertyName),color:"#00c"},{tag:l.pJ.comment,color:"#940"},{tag:l.pJ.invalid,color:"#f00"}]),eg=o.tk.baseTheme({"&.cm-focused .cm-matchingBracket":{backgroundColor:"#328c8252"},"&.cm-focused .cm-nonmatchingBracket":{backgroundColor:"#bb555544"}}),ek="()[]{}",ev=s.r$.define({combine:e=>(0,s.BO)(e,{afterCursor:!0,brackets:ek,maxScanDistance:1e4,renderMatch:eb})}),ey=o.p.mark({class:"cm-matchingBracket"}),ex=o.p.mark({class:"cm-nonmatchingBracket"});function eb(e){let t=[],n=e.matched?ey:ex;return t.push(n.range(e.start.from,e.start.to)),e.end&&t.push(n.range(e.end.from,e.end.to)),t}let eO=[s.QQ.define({create:()=>o.p.none,update(e,t){if(!t.docChanged&&!t.selection)return e;let n=[],i=t.state.facet(ev);for(let e of t.state.selection.ranges){if(!e.empty)continue;let r=eA(t.state,e.head,-1,i)||e.head>0&&eA(t.state,e.head-1,1,i)||i.afterCursor&&(eA(t.state,e.head,1,i)||e.head<t.state.doc.length&&eA(t.state,e.head+1,-1,i));r&&(n=n.concat(i.renderMatch(r,t.state)))}return o.p.set(n,!0)},provide:e=>o.tk.decorations.from(e)}),eg];function ew(e={}){return[ev.of(e),eO]}let eS=new r.md;function eP(e,t,n){let i=e.prop(t<0?r.md.openedBy:r.md.closedBy);if(i)return i;if(1==e.name.length){let i=n.indexOf(e.name);if(i>-1&&i%2==(t<0?1:0))return[n[i+t]]}return null}function eC(e){let t=e.type.prop(eS);return t?t(e.node):e}function eA(e,t,n,i={}){let r=i.maxScanDistance||1e4,s=i.brackets||ek,o=p(e),l=o.resolveInner(t,n);for(let e=l;e;e=e.parent){let i=eP(e.type,n,s);if(i&&e.from<e.to){let r=eC(e);if(r&&(n>0?t>=r.from&&t<r.to:t>r.from&&t<=r.to))return function(e,t,n,i,r,s,o){let l=i.parent,a={from:r.from,to:r.to},h=0,c=null==l?void 0:l.cursor();if(c&&(n<0?c.childBefore(i.from):c.childAfter(i.to)))do if(n<0?c.to<=i.from:c.from>=i.to){if(0==h&&s.indexOf(c.type.name)>-1&&c.from<c.to){let e=eC(c);return{start:a,end:e?{from:e.from,to:e.to}:void 0,matched:!0}}if(eP(c.type,n,o))h++;else if(eP(c.type,-n,o)){if(0==h){let e=eC(c);return{start:a,end:e&&e.from<e.to?{from:e.from,to:e.to}:void 0,matched:!1}}h--}}while(n<0?c.prevSibling():c.nextSibling());return{start:a,matched:!1}}(0,0,n,e,r,i,s)}}return function(e,t,n,i,r,s,o){let l=n<0?e.sliceDoc(t-1,t):e.sliceDoc(t,t+1),a=o.indexOf(l);if(a<0||a%2==0!=n>0)return null;let h={from:n<0?t-1:t,to:n>0?t+1:t},c=e.doc.iterRange(t,n>0?e.doc.length:0),u=0;for(let e=0;!c.next().done&&e<=s;){let s=c.value;n<0&&(e+=s.length);let l=t+e*n;for(let e=n>0?0:s.length-1,t=n>0?s.length:-1;e!=t;e+=n){let t=o.indexOf(s[e]);if(!(t<0)&&i.resolveInner(l+e,1).type==r){if(t%2==0==n>0)u++;else{if(1==u)return{start:h,end:{from:l+e,to:l+e+1},matched:t>>1==a>>1};u--}}}n>0&&(e+=s.length)}return c.done?{start:h,matched:!1}:null}(e,t,n,o,l.type,r,s)}let eT=Object.create(null),eR=[r.Jq.none],eD=[],eM=Object.create(null),eI=Object.create(null);for(let[e,t]of[["variable","variableName"],["variable-2","variableName.special"],["string-2","string.special"],["def","variableName.definition"],["tag","tagName"],["attribute","attributeName"],["type","typeName"],["builtin","variableName.standard"],["qualifier","modifier"],["error","invalid"],["header","heading"],["property","propertyName"]])eI[e]=function(e,t){let n=[];for(let i of t.split(" ")){let t=[];for(let n of i.split(".")){let i=e[n]||l.pJ[n];i?"function"==typeof i?t.length?t=t.map(i):eQ(n,`Modifier ${n} used at start of tag`):t.length?eQ(n,`Tag ${n} used as modifier`):t=Array.isArray(i)?i:[i]:eQ(n,`Unknown highlighting tag ${n}`)}for(let e of t)n.push(e)}if(!n.length)return 0;let i=t.replace(/ /g,"_"),s=i+" "+n.map(e=>e.id),o=eM[s];if(o)return o.id;let a=eM[s]=r.Jq.define({id:eR.length,name:i,props:[(0,l.Gv)({[i]:n})]});return eR.push(a),a.id}(eT,t);function eQ(e,t){eD.indexOf(e)>-1||(eD.push(e),console.warn(t))}o.Nm.RTL,o.Nm.LTR},37205:function(e,t,n){let i;n.d(t,{q6:function(){return ec},n0:function(){return R},as:function(){return D},D0:function(){return ey},jT:function(){return $},yy:function(){return eb},r$:function(){return X},gc:function(){return T},Wl:function(){return G},Xs:function(){return eT},f_:function(){return eR},uU:function(){return ew},Py:function(){return ed},QQ:function(){return U},xv:function(){return f},YW:function(){return ep},gm:function(){return O},nZ:function(){return S},BO:function(){return eO},IS:function(){return ez},cp:function(){return b},Gz:function(){return eX},bg:function(){return w}});let r=[],s=[];function o(e){return e>=127462&&e<=127487}function l(e,t,n){if(t==e.length)return t;t&&h(e.charCodeAt(t))&&c(e.charCodeAt(t-1))&&t--;let i=a(e,t);for(t+=u(i);t<e.length;){let l=a(e,t);if(8205==i||8205==l||n&&function(e){if(e<768)return!1;for(let t=0,n=r.length;;){let i=t+n>>1;if(e<r[i])n=i;else{if(!(e>=s[i]))return!0;t=i+1}if(t==n)return!1}}(l))t+=u(l),i=l;else if(o(l)){let n=0,i=t-2;for(;i>=0&&o(a(e,i));)n++,i-=2;if(n%2==0)break;t+=2}else break}return t}function a(e,t){let n=e.charCodeAt(t);if(!c(n)||t+1==e.length)return n;let i=e.charCodeAt(t+1);return h(i)?(n-55296<<10)+(i-56320)+65536:n}function h(e){return e>=56320&&e<57344}function c(e){return e>=55296&&e<56320}function u(e){return e<65536?1:2}(()=>{let e="lc,34,7n,7,7b,19,,,,2,,2,,,20,b,1c,l,g,,2t,7,2,6,2,2,,4,z,,u,r,2j,b,1m,9,9,,o,4,,9,,3,,5,17,3,3b,f,,w,1j,,,,4,8,4,,3,7,a,2,t,,1m,,,,2,4,8,,9,,a,2,q,,2,2,1l,,4,2,4,2,2,3,3,,u,2,3,,b,2,1l,,4,5,,2,4,,k,2,m,6,,,1m,,,2,,4,8,,7,3,a,2,u,,1n,,,,c,,9,,14,,3,,1l,3,5,3,,4,7,2,b,2,t,,1m,,2,,2,,3,,5,2,7,2,b,2,s,2,1l,2,,,2,4,8,,9,,a,2,t,,20,,4,,2,3,,,8,,29,,2,7,c,8,2q,,2,9,b,6,22,2,r,,,,,,1j,e,,5,,2,5,b,,10,9,,2u,4,,6,,2,2,2,p,2,4,3,g,4,d,,2,2,6,,f,,jj,3,qa,3,t,3,t,2,u,2,1s,2,,7,8,,2,b,9,,19,3,3b,2,y,,3a,3,4,2,9,,6,3,63,2,2,,1m,,,7,,,,,2,8,6,a,2,,1c,h,1r,4,1c,7,,,5,,14,9,c,2,w,4,2,2,,3,1k,,,2,3,,,3,1m,8,2,2,48,3,,d,,7,4,,6,,3,2,5i,1m,,5,ek,,5f,x,2da,3,3x,,2o,w,fe,6,2x,2,n9w,4,,a,w,2,28,2,7k,,3,,4,,p,2,5,,47,2,q,i,d,,12,8,p,b,1a,3,1c,,2,4,2,2,13,,1v,6,2,2,2,2,c,,8,,1b,,1f,,,3,2,2,5,2,,,16,2,8,,6m,,2,,4,,fn4,,kh,g,g,g,a6,2,gt,,6a,,45,5,1ae,3,,2,5,4,14,3,4,,4l,2,fx,4,ar,2,49,b,4w,,1i,f,1k,3,1d,4,2,2,1x,3,10,5,,8,1q,,c,2,1g,9,a,4,2,,2n,3,2,,,2,6,,4g,,3,8,l,2,1l,2,,,,,m,,e,7,3,5,5f,8,2,3,,,n,,29,,2,6,,,2,,,2,,2,6j,,2,4,6,2,,2,r,2,2d,8,2,,,2,2y,,,,2,6,,,2t,3,2,4,,5,77,9,,2,6t,,a,2,,,4,,40,4,2,2,4,,w,a,14,6,2,4,8,,9,6,2,3,1a,d,,2,ba,7,,6,,,2a,m,2,7,,2,,2,3e,6,3,,,2,,7,,,20,2,3,,,,9n,2,f0b,5,1n,7,t4,,1r,4,29,,f5k,2,43q,,,3,4,5,8,8,2,7,u,4,44,3,1iz,1j,4,1e,8,,e,,m,5,,f,11s,7,,h,2,7,,2,,5,79,7,c5,4,15s,7,31,7,240,5,gx7k,2o,3k,6o".split(",").map(e=>e?parseInt(e,36):1);for(let t=0,n=0;t<e.length;t++)(t%2?s:r).push(n+=e[t])})();class f{lineAt(e){if(e<0||e>this.length)throw RangeError(`Invalid position ${e} in document of length ${this.length}`);return this.lineInner(e,!1,1,0)}line(e){if(e<1||e>this.lines)throw RangeError(`Invalid line number ${e} in ${this.lines}-line document`);return this.lineInner(e,!0,1,0)}replace(e,t,n){[e,t]=x(this,e,t);let i=[];return this.decompose(0,e,i,2),n.length&&n.decompose(0,n.length,i,3),this.decompose(t,this.length,i,1),p.from(i,this.length-(t-e)+n.length)}append(e){return this.replace(this.length,this.length,e)}slice(e,t=this.length){[e,t]=x(this,e,t);let n=[];return this.decompose(e,t,n,0),p.from(n,t-e)}eq(e){if(e==this)return!0;if(e.length!=this.length||e.lines!=this.lines)return!1;let t=this.scanIdentical(e,1),n=this.length-this.scanIdentical(e,-1),i=new g(this),r=new g(e);for(let e=t,s=t;;){if(i.next(e),r.next(e),e=0,i.lineBreak!=r.lineBreak||i.done!=r.done||i.value!=r.value)return!1;if(s+=i.value.length,i.done||s>=n)return!0}}iter(e=1){return new g(this,e)}iterRange(e,t=this.length){return new k(this,e,t)}iterLines(e,t){let n;if(null==e)n=this.iter();else{null==t&&(t=this.lines+1);let i=this.line(e).from;n=this.iterRange(i,Math.max(i,t==this.lines+1?this.length:t<=1?0:this.line(t-1).to))}return new v(n)}toString(){return this.sliceString(0)}toJSON(){let e=[];return this.flatten(e),e}constructor(){}static of(e){if(0==e.length)throw RangeError("A document must have at least one line");return 1!=e.length||e[0]?e.length<=32?new d(e):p.from(d.split(e,[])):f.empty}}class d extends f{constructor(e,t=function(e){let t=-1;for(let n of e)t+=n.length+1;return t}(e)){super(),this.text=e,this.length=t}get lines(){return this.text.length}get children(){return null}lineInner(e,t,n,i){for(let r=0;;r++){let s=this.text[r],o=i+s.length;if((t?n:o)>=e)return new y(i,o,n,s);i=o+1,n++}}decompose(e,t,n,i){let r=e<=0&&t>=this.length?this:new d(m(this.text,[""],e,t),Math.min(t,this.length)-Math.max(0,e));if(1&i){let e=n.pop(),t=m(r.text,e.text.slice(),0,r.length);if(t.length<=32)n.push(new d(t,e.length+r.length));else{let e=t.length>>1;n.push(new d(t.slice(0,e)),new d(t.slice(e)))}}else n.push(r)}replace(e,t,n){if(!(n instanceof d))return super.replace(e,t,n);[e,t]=x(this,e,t);let i=m(this.text,m(n.text,m(this.text,[""],0,e)),t),r=this.length+n.length-(t-e);return i.length<=32?new d(i,r):p.from(d.split(i,[]),r)}sliceString(e,t=this.length,n="\n"){[e,t]=x(this,e,t);let i="";for(let r=0,s=0;r<=t&&s<this.text.length;s++){let o=this.text[s],l=r+o.length;r>e&&s&&(i+=n),e<l&&t>r&&(i+=o.slice(Math.max(0,e-r),t-r)),r=l+1}return i}flatten(e){for(let t of this.text)e.push(t)}scanIdentical(){return 0}static split(e,t){let n=[],i=-1;for(let r of e)n.push(r),i+=r.length+1,32==n.length&&(t.push(new d(n,i)),n=[],i=-1);return i>-1&&t.push(new d(n,i)),t}}class p extends f{constructor(e,t){for(let n of(super(),this.children=e,this.length=t,this.lines=0,e))this.lines+=n.lines}lineInner(e,t,n,i){for(let r=0;;r++){let s=this.children[r],o=i+s.length,l=n+s.lines-1;if((t?l:o)>=e)return s.lineInner(e,t,n,i);i=o+1,n=l+1}}decompose(e,t,n,i){for(let r=0,s=0;s<=t&&r<this.children.length;r++){let o=this.children[r],l=s+o.length;if(e<=l&&t>=s){let r=i&((s<=e?1:0)|(l>=t?2:0));s>=e&&l<=t&&!r?n.push(o):o.decompose(e-s,t-s,n,r)}s=l+1}}replace(e,t,n){if([e,t]=x(this,e,t),n.lines<this.lines)for(let i=0,r=0;i<this.children.length;i++){let s=this.children[i],o=r+s.length;if(e>=r&&t<=o){let l=s.replace(e-r,t-r,n),a=this.lines-s.lines+l.lines;if(l.lines<a>>4&&l.lines>a>>6){let r=this.children.slice();return r[i]=l,new p(r,this.length-(t-e)+n.length)}return super.replace(r,o,l)}r=o+1}return super.replace(e,t,n)}sliceString(e,t=this.length,n="\n"){[e,t]=x(this,e,t);let i="";for(let r=0,s=0;r<this.children.length&&s<=t;r++){let o=this.children[r],l=s+o.length;s>e&&r&&(i+=n),e<l&&t>s&&(i+=o.sliceString(e-s,t-s,n)),s=l+1}return i}flatten(e){for(let t of this.children)t.flatten(e)}scanIdentical(e,t){if(!(e instanceof p))return 0;let n=0,[i,r,s,o]=t>0?[0,0,this.children.length,e.children.length]:[this.children.length-1,e.children.length-1,-1,-1];for(;;i+=t,r+=t){if(i==s||r==o)return n;let l=this.children[i],a=e.children[r];if(l!=a)return n+l.scanIdentical(a,t);n+=l.length+1}}static from(e,t=e.reduce((e,t)=>e+t.length+1,-1)){let n=0;for(let t of e)n+=t.lines;if(n<32){let n=[];for(let t of e)t.flatten(n);return new d(n,t)}let i=Math.max(32,n>>5),r=i<<1,s=i>>1,o=[],l=0,a=-1,h=[];function c(){0!=l&&(o.push(1==h.length?h[0]:p.from(h,a)),a=-1,l=h.length=0)}for(let t of e)!function e(t){let n;if(t.lines>r&&t instanceof p)for(let n of t.children)e(n);else t.lines>s&&(l>s||!l)?(c(),o.push(t)):t instanceof d&&l&&(n=h[h.length-1])instanceof d&&t.lines+n.lines<=32?(l+=t.lines,a+=t.length+1,h[h.length-1]=new d(n.text.concat(t.text),n.length+1+t.length)):(l+t.lines>i&&c(),l+=t.lines,a+=t.length+1,h.push(t))}(t);return c(),1==o.length?o[0]:new p(o,t)}}function m(e,t,n=0,i=1e9){for(let r=0,s=0,o=!0;s<e.length&&r<=i;s++){let l=e[s],a=r+l.length;a>=n&&(a>i&&(l=l.slice(0,i-r)),r<n&&(l=l.slice(n-r)),o?(t[t.length-1]+=l,o=!1):t.push(l)),r=a+1}return t}f.empty=new d([""],0);class g{constructor(e,t=1){this.dir=t,this.done=!1,this.lineBreak=!1,this.value="",this.nodes=[e],this.offsets=[t>0?1:(e instanceof d?e.text.length:e.children.length)<<1]}nextInner(e,t){for(this.done=this.lineBreak=!1;;){let n=this.nodes.length-1,i=this.nodes[n],r=this.offsets[n],s=r>>1,o=i instanceof d?i.text.length:i.children.length;if(s==(t>0?o:0)){if(0==n)return this.done=!0,this.value="",this;t>0&&this.offsets[n-1]++,this.nodes.pop(),this.offsets.pop()}else if((1&r)==(t>0?0:1)){if(this.offsets[n]+=t,0==e)return this.lineBreak=!0,this.value="\n",this;e--}else if(i instanceof d){let r=i.text[s+(t<0?-1:0)];if(this.offsets[n]+=t,r.length>Math.max(0,e))return this.value=0==e?r:t>0?r.slice(e):r.slice(0,r.length-e),this;e-=r.length}else{let r=i.children[s+(t<0?-1:0)];e>r.length?(e-=r.length,this.offsets[n]+=t):(t<0&&this.offsets[n]--,this.nodes.push(r),this.offsets.push(t>0?1:(r instanceof d?r.text.length:r.children.length)<<1))}}}next(e=0){return e<0&&(this.nextInner(-e,-this.dir),e=this.value.length),this.nextInner(e,this.dir)}}class k{constructor(e,t,n){this.value="",this.done=!1,this.cursor=new g(e,t>n?-1:1),this.pos=t>n?e.length:0,this.from=Math.min(t,n),this.to=Math.max(t,n)}nextInner(e,t){if(t<0?this.pos<=this.from:this.pos>=this.to)return this.value="",this.done=!0,this;e+=Math.max(0,t<0?this.pos-this.to:this.from-this.pos);let n=t<0?this.pos-this.from:this.to-this.pos;e>n&&(e=n),n-=e;let{value:i}=this.cursor.next(e);return this.pos+=(i.length+e)*t,this.value=i.length<=n?i:t<0?i.slice(i.length-n):i.slice(0,n),this.done=!this.value,this}next(e=0){return e<0?e=Math.max(e,this.from-this.pos):e>0&&(e=Math.min(e,this.to-this.pos)),this.nextInner(e,this.cursor.dir)}get lineBreak(){return this.cursor.lineBreak&&""!=this.value}}class v{constructor(e){this.inner=e,this.afterBreak=!0,this.value="",this.done=!1}next(e=0){let{done:t,lineBreak:n,value:i}=this.inner.next(e);return t&&this.afterBreak?(this.value="",this.afterBreak=!1):t?(this.done=!0,this.value=""):n?this.afterBreak?this.value="":(this.afterBreak=!0,this.next()):(this.value=i,this.afterBreak=!1),this}get lineBreak(){return!1}}"undefined"!=typeof Symbol&&(f.prototype[Symbol.iterator]=function(){return this.iter()},g.prototype[Symbol.iterator]=k.prototype[Symbol.iterator]=v.prototype[Symbol.iterator]=function(){return this});class y{constructor(e,t,n,i){this.from=e,this.to=t,this.number=n,this.text=i}get length(){return this.to-this.from}}function x(e,t,n){return[t=Math.max(0,Math.min(e.length,t)),Math.max(t,Math.min(e.length,n))]}function b(e,t,n=!0,i=!0){return function(e,t,n=!0,i=!0){return(n?l:function(e,t,n){for(;t>0;){let i=l(e,t-2,n);if(i<t)return i;t--}return 0})(e,t,i)}(e,t,n,i)}function O(e,t){let n=e.charCodeAt(t);if(!(n>=55296&&n<56320)||t+1==e.length)return n;let i=e.charCodeAt(t+1);return i>=56320&&i<57344?(n-55296<<10)+(i-56320)+65536:n}function w(e){return e<=65535?String.fromCharCode(e):String.fromCharCode(((e-=65536)>>10)+55296,(1023&e)+56320)}function S(e){return e<65536?1:2}let P=/\r\n?|\n/;var C,A,T=((C=T||(T={}))[C.Simple=0]="Simple",C[C.TrackDel=1]="TrackDel",C[C.TrackBefore=2]="TrackBefore",C[C.TrackAfter=3]="TrackAfter",C);class R{constructor(e){this.sections=e}get length(){let e=0;for(let t=0;t<this.sections.length;t+=2)e+=this.sections[t];return e}get newLength(){let e=0;for(let t=0;t<this.sections.length;t+=2){let n=this.sections[t+1];e+=n<0?this.sections[t]:n}return e}get empty(){return 0==this.sections.length||2==this.sections.length&&this.sections[1]<0}iterGaps(e){for(let t=0,n=0,i=0;t<this.sections.length;){let r=this.sections[t++],s=this.sections[t++];s<0?(e(n,i,r),i+=r):i+=s,n+=r}}iterChangedRanges(e,t=!1){Q(this,e,t)}get invertedDesc(){let e=[];for(let t=0;t<this.sections.length;){let n=this.sections[t++],i=this.sections[t++];i<0?e.push(n,i):e.push(i,n)}return new R(e)}composeDesc(e){return this.empty?e:e.empty?this:E(this,e)}mapDesc(e,t=!1){return e.empty?this:N(this,e,t)}mapPos(e,t=-1,n=T.Simple){let i=0,r=0;for(let s=0;s<this.sections.length;){let o=this.sections[s++],l=this.sections[s++],a=i+o;if(l<0){if(a>e)return r+(e-i);r+=o}else{if(n!=T.Simple&&a>=e&&(n==T.TrackDel&&i<e&&a>e||n==T.TrackBefore&&i<e||n==T.TrackAfter&&a>e))return null;if(a>e||a==e&&t<0&&!o)return e==i||t<0?r:r+l;r+=l}i=a}if(e>i)throw RangeError(`Position ${e} is out of range for changeset of length ${i}`);return r}touchesRange(e,t=e){for(let n=0,i=0;n<this.sections.length&&i<=t;){let r=this.sections[n++],s=this.sections[n++],o=i+r;if(s>=0&&i<=t&&o>=e)return!(i<e)||!(o>t)||"cover";i=o}return!1}toString(){let e="";for(let t=0;t<this.sections.length;){let n=this.sections[t++],i=this.sections[t++];e+=(e?" ":"")+n+(i>=0?":"+i:"")}return e}toJSON(){return this.sections}static fromJSON(e){if(!Array.isArray(e)||e.length%2||e.some(e=>"number"!=typeof e))throw RangeError("Invalid JSON representation of ChangeDesc");return new R(e)}static create(e){return new R(e)}}class D extends R{constructor(e,t){super(e),this.inserted=t}apply(e){if(this.length!=e.length)throw RangeError("Applying change set to a document with the wrong length");return Q(this,(t,n,i,r,s)=>e=e.replace(i,i+(n-t),s),!1),e}mapDesc(e,t=!1){return N(this,e,t,!0)}invert(e){let t=this.sections.slice(),n=[];for(let i=0,r=0;i<t.length;i+=2){let s=t[i],o=t[i+1];if(o>=0){t[i]=o,t[i+1]=s;let l=i>>1;for(;n.length<l;)n.push(f.empty);n.push(s?e.slice(r,r+s):f.empty)}r+=s}return new D(t,n)}compose(e){return this.empty?e:e.empty?this:E(this,e,!0)}map(e,t=!1){return e.empty?this:N(this,e,t,!0)}iterChanges(e,t=!1){Q(this,e,t)}get desc(){return R.create(this.sections)}filter(e){let t=[],n=[],i=[],r=new B(this);n:for(let s=0,o=0;;){let l=s==e.length?1e9:e[s++];for(;o<l||o==l&&0==r.len;){if(r.done)break n;let e=Math.min(r.len,l-o);M(i,e,-1);let s=-1==r.ins?-1:0==r.off?r.ins:0;M(t,e,s),s>0&&I(n,t,r.text),r.forward(e),o+=e}let a=e[s++];for(;o<a;){if(r.done)break n;let e=Math.min(r.len,a-o);M(t,e,-1),M(i,e,-1==r.ins?-1:0==r.off?r.ins:0),r.forward(e),o+=e}}return{changes:new D(t,n),filtered:R.create(i)}}toJSON(){let e=[];for(let t=0;t<this.sections.length;t+=2){let n=this.sections[t],i=this.sections[t+1];i<0?e.push(n):0==i?e.push([n]):e.push([n].concat(this.inserted[t>>1].toJSON()))}return e}static of(e,t,n){let i=[],r=[],s=0,o=null;function l(e=!1){if(!e&&!i.length)return;s<t&&M(i,t-s,-1);let n=new D(i,r);o=o?o.compose(n.map(o)):n,i=[],r=[],s=0}return!function e(a){if(Array.isArray(a))for(let t of a)e(t);else if(a instanceof D){if(a.length!=t)throw RangeError(`Mismatched change set length (got ${a.length}, expected ${t})`);l(),o=o?o.compose(a.map(o)):a}else{let{from:e,to:o=e,insert:h}=a;if(e>o||e<0||o>t)throw RangeError(`Invalid change range ${e} to ${o} (in doc of length ${t})`);let c=h?"string"==typeof h?f.of(h.split(n||P)):h:f.empty,u=c.length;if(e==o&&0==u)return;e<s&&l(),e>s&&M(i,e-s,-1),M(i,o-e,u),I(r,i,c),s=o}}(e),l(!o),o}static empty(e){return new D(e?[e,-1]:[],[])}static fromJSON(e){if(!Array.isArray(e))throw RangeError("Invalid JSON representation of ChangeSet");let t=[],n=[];for(let i=0;i<e.length;i++){let r=e[i];if("number"==typeof r)t.push(r,-1);else if(!Array.isArray(r)||"number"!=typeof r[0]||r.some((e,t)=>t&&"string"!=typeof e))throw RangeError("Invalid JSON representation of ChangeSet");else if(1==r.length)t.push(r[0],0);else{for(;n.length<i;)n.push(f.empty);n[i]=f.of(r.slice(1)),t.push(r[0],n[i].length)}}return new D(t,n)}static createSet(e,t){return new D(e,t)}}function M(e,t,n,i=!1){if(0==t&&n<=0)return;let r=e.length-2;r>=0&&n<=0&&n==e[r+1]?e[r]+=t:r>=0&&0==t&&0==e[r]?e[r+1]+=n:i?(e[r]+=t,e[r+1]+=n):e.push(t,n)}function I(e,t,n){if(0==n.length)return;let i=t.length-2>>1;if(i<e.length)e[e.length-1]=e[e.length-1].append(n);else{for(;e.length<i;)e.push(f.empty);e.push(n)}}function Q(e,t,n){let i=e.inserted;for(let r=0,s=0,o=0;o<e.sections.length;){let l=e.sections[o++],a=e.sections[o++];if(a<0)r+=l,s+=l;else{let h=r,c=s,u=f.empty;for(;h+=l,c+=a,a&&i&&(u=u.append(i[o-2>>1])),!n&&o!=e.sections.length&&!(e.sections[o+1]<0);)l=e.sections[o++],a=e.sections[o++];t(r,h,s,c,u),r=h,s=c}}}function N(e,t,n,i=!1){let r=[],s=i?[]:null,o=new B(e),l=new B(t);for(let e=-1;;){if(o.done&&l.len||l.done&&o.len)throw Error("Mismatched change set lengths");if(-1==o.ins&&-1==l.ins){let e=Math.min(o.len,l.len);M(r,e,-1),o.forward(e),l.forward(e)}else if(l.ins>=0&&(o.ins<0||e==o.i||0==o.off&&(l.len<o.len||l.len==o.len&&!n))){let t=l.len;for(M(r,l.ins,-1);t;){let n=Math.min(o.len,t);o.ins>=0&&e<o.i&&o.len<=n&&(M(r,0,o.ins),s&&I(s,r,o.text),e=o.i),o.forward(n),t-=n}l.next()}else if(o.ins>=0){let t=0,n=o.len;for(;n;)if(-1==l.ins){let e=Math.min(n,l.len);t+=e,n-=e,l.forward(e)}else if(0==l.ins&&l.len<n)n-=l.len,l.next();else break;M(r,t,e<o.i?o.ins:0),s&&e<o.i&&I(s,r,o.text),e=o.i,o.forward(o.len-n)}else if(o.done&&l.done)return s?D.createSet(r,s):R.create(r);else throw Error("Mismatched change set lengths")}}function E(e,t,n=!1){let i=[],r=n?[]:null,s=new B(e),o=new B(t);for(let e=!1;;){if(s.done&&o.done)return r?D.createSet(i,r):R.create(i);if(0==s.ins)M(i,s.len,0,e),s.next();else if(0!=o.len||o.done){if(s.done||o.done)throw Error("Mismatched change set lengths");{let t=Math.min(s.len2,o.len),n=i.length;if(-1==s.ins){let n=-1==o.ins?-1:o.off?0:o.ins;M(i,t,n,e),r&&n&&I(r,i,o.text)}else -1==o.ins?(M(i,s.off?0:s.len,t,e),r&&I(r,i,s.textBit(t))):(M(i,s.off?0:s.len,o.off?0:o.ins,e),r&&!o.off&&I(r,i,o.text));e=(s.ins>t||o.ins>=0&&o.len>t)&&(e||i.length>n),s.forward2(t),o.forward(t)}}else M(i,0,o.ins,e),r&&I(r,i,o.text),o.next()}}class B{constructor(e){this.set=e,this.i=0,this.next()}next(){let{sections:e}=this.set;this.i<e.length?(this.len=e[this.i++],this.ins=e[this.i++]):(this.len=0,this.ins=-2),this.off=0}get done(){return -2==this.ins}get len2(){return this.ins<0?this.len:this.ins}get text(){let{inserted:e}=this.set,t=this.i-2>>1;return t>=e.length?f.empty:e[t]}textBit(e){let{inserted:t}=this.set,n=this.i-2>>1;return n>=t.length&&!e?f.empty:t[n].slice(this.off,null==e?void 0:this.off+e)}forward(e){e==this.len?this.next():(this.len-=e,this.off+=e)}forward2(e){-1==this.ins?this.forward(e):e==this.ins?this.next():(this.ins-=e,this.off+=e)}}class L{constructor(e,t,n){this.from=e,this.to=t,this.flags=n}get anchor(){return 32&this.flags?this.to:this.from}get head(){return 32&this.flags?this.from:this.to}get empty(){return this.from==this.to}get assoc(){return 8&this.flags?-1:16&this.flags?1:0}get bidiLevel(){let e=7&this.flags;return 7==e?null:e}get goalColumn(){let e=this.flags>>6;return 16777215==e?void 0:e}map(e,t=-1){let n,i;return this.empty?n=i=e.mapPos(this.from,t):(n=e.mapPos(this.from,1),i=e.mapPos(this.to,-1)),n==this.from&&i==this.to?this:new L(n,i,this.flags)}extend(e,t=e){if(e<=this.anchor&&t>=this.anchor)return $.range(e,t);let n=Math.abs(e-this.anchor)>Math.abs(t-this.anchor)?e:t;return $.range(this.anchor,n)}eq(e,t=!1){return this.anchor==e.anchor&&this.head==e.head&&this.goalColumn==e.goalColumn&&(!t||!this.empty||this.assoc==e.assoc)}toJSON(){return{anchor:this.anchor,head:this.head}}static fromJSON(e){if(!e||"number"!=typeof e.anchor||"number"!=typeof e.head)throw RangeError("Invalid JSON representation for SelectionRange");return $.range(e.anchor,e.head)}static create(e,t,n){return new L(e,t,n)}}class ${constructor(e,t){this.ranges=e,this.mainIndex=t}map(e,t=-1){return e.empty?this:$.create(this.ranges.map(n=>n.map(e,t)),this.mainIndex)}eq(e,t=!1){if(this.ranges.length!=e.ranges.length||this.mainIndex!=e.mainIndex)return!1;for(let n=0;n<this.ranges.length;n++)if(!this.ranges[n].eq(e.ranges[n],t))return!1;return!0}get main(){return this.ranges[this.mainIndex]}asSingle(){return 1==this.ranges.length?this:new $([this.main],0)}addRange(e,t=!0){return $.create([e].concat(this.ranges),t?0:this.mainIndex+1)}replaceRange(e,t=this.mainIndex){let n=this.ranges.slice();return n[t]=e,$.create(n,this.mainIndex)}toJSON(){return{ranges:this.ranges.map(e=>e.toJSON()),main:this.mainIndex}}static fromJSON(e){if(!e||!Array.isArray(e.ranges)||"number"!=typeof e.main||e.main>=e.ranges.length)throw RangeError("Invalid JSON representation for EditorSelection");return new $(e.ranges.map(e=>L.fromJSON(e)),e.main)}static single(e,t=e){return new $([$.range(e,t)],0)}static create(e,t=0){if(0==e.length)throw RangeError("A selection needs at least one range");for(let n=0,i=0;i<e.length;i++){let r=e[i];if(r.empty?r.from<=n:r.from<n)return $.normalized(e.slice(),t);n=r.to}return new $(e,t)}static cursor(e,t=0,n,i){return L.create(e,e,(0==t?0:t<0?8:16)|(null==n?7:Math.min(6,n))|(null!=i?i:16777215)<<6)}static range(e,t,n,i){let r=(null!=n?n:16777215)<<6|(null==i?7:Math.min(6,i));return t<e?L.create(t,e,48|r):L.create(e,t,(t>e?8:0)|r)}static normalized(e,t=0){let n=e[t];e.sort((e,t)=>e.from-t.from),t=e.indexOf(n);for(let n=1;n<e.length;n++){let i=e[n],r=e[n-1];if(i.empty?i.from<=r.to:i.from<r.to){let s=r.from,o=Math.max(i.to,r.to);n<=t&&t--,e.splice(--n,2,i.anchor>i.head?$.range(o,s):$.range(s,o))}}return new $(e,t)}}function j(e,t){for(let n of e.ranges)if(n.to>t)throw RangeError("Selection points outside of document")}let z=0;class X{constructor(e,t,n,i,r){this.combine=e,this.compareInput=t,this.compare=n,this.isStatic=i,this.id=z++,this.default=e([]),this.extensions="function"==typeof r?r(this):r}get reader(){return this}static define(e={}){return new X(e.combine||(e=>e),e.compareInput||((e,t)=>e===t),e.compare||(e.combine?(e,t)=>e===t:q),!!e.static,e.enables)}of(e){return new J([],this,0,e)}compute(e,t){if(this.isStatic)throw Error("Can't compute a static facet");return new J(e,this,1,t)}computeN(e,t){if(this.isStatic)throw Error("Can't compute a static facet");return new J(e,this,2,t)}from(e,t){return t||(t=e=>e),this.compute([e],n=>t(n.field(e)))}}function q(e,t){return e==t||e.length==t.length&&e.every((e,n)=>e===t[n])}class J{constructor(e,t,n,i){this.dependencies=e,this.facet=t,this.type=n,this.value=i,this.id=z++}dynamicSlot(e){var t;let n=this.value,i=this.facet.compareInput,r=this.id,s=e[r]>>1,o=2==this.type,l=!1,a=!1,h=[];for(let n of this.dependencies)"doc"==n?l=!0:"selection"==n?a=!0:((null!==(t=e[n.id])&&void 0!==t?t:1)&1)==0&&h.push(e[n.id]);return{create:e=>(e.values[s]=n(e),1),update(e,t){if(l&&t.docChanged||a&&(t.docChanged||t.selection)||_(e,h)){let t=n(e);if(o?!W(t,e.values[s],i):!i(t,e.values[s]))return e.values[s]=t,1}return 0},reconfigure:(e,t)=>{let l,a=t.config.address[r];if(null!=a){let r=en(t,a);if(this.dependencies.every(n=>n instanceof X?t.facet(n)===e.facet(n):!(n instanceof U)||t.field(n,!1)==e.field(n,!1))||(o?W(l=n(e),r,i):i(l=n(e),r)))return e.values[s]=r,0}else l=n(e);return e.values[s]=l,1}}}}function W(e,t,n){if(e.length!=t.length)return!1;for(let i=0;i<e.length;i++)if(!n(e[i],t[i]))return!1;return!0}function _(e,t){let n=!1;for(let i of t)1&et(e,i)&&(n=!0);return n}let F=X.define({static:!0});class U{constructor(e,t,n,i,r){this.id=e,this.createF=t,this.updateF=n,this.compareF=i,this.spec=r,this.provides=void 0}static define(e){let t=new U(z++,e.create,e.update,e.compare||((e,t)=>e===t),e);return e.provide&&(t.provides=e.provide(t)),t}create(e){let t=e.facet(F).find(e=>e.field==this);return((null==t?void 0:t.create)||this.createF)(e)}slot(e){let t=e[this.id]>>1;return{create:e=>(e.values[t]=this.create(e),1),update:(e,n)=>{let i=e.values[t],r=this.updateF(i,n);return this.compareF(i,r)?0:(e.values[t]=r,1)},reconfigure:(e,n)=>{let i=e.facet(F),r=n.facet(F),s;return(s=i.find(e=>e.field==this))&&s!=r.find(e=>e.field==this)?(e.values[t]=s.create(e),1):null!=n.config.address[this.id]?(e.values[t]=n.field(this),0):(e.values[t]=this.create(e),1)}}}init(e){return[this,F.of({field:this,create:e})]}get extension(){return this}}let V={lowest:4,low:3,default:2,high:1,highest:0};function Z(e){return t=>new Y(t,e)}let G={highest:Z(V.highest),high:Z(V.high),default:Z(V.default),low:Z(V.low),lowest:Z(V.lowest)};class Y{constructor(e,t){this.inner=e,this.prec=t}}class H{of(e){return new K(this,e)}reconfigure(e){return H.reconfigure.of({compartment:this,extension:e})}get(e){return e.config.compartments.get(this)}}class K{constructor(e,t){this.compartment=e,this.inner=t}}class ee{constructor(e,t,n,i,r,s){for(this.base=e,this.compartments=t,this.dynamicSlots=n,this.address=i,this.staticValues=r,this.facets=s,this.statusTemplate=[];this.statusTemplate.length<n.length;)this.statusTemplate.push(0)}staticFacet(e){let t=this.address[e.id];return null==t?e.default:this.staticValues[t>>1]}static resolve(e,t,n){let i,r,s=[],o=Object.create(null),l=new Map;for(let n of(i=[[],[],[],[],[]],r=new Map,!function e(n,s){let o=r.get(n);if(null!=o){if(o<=s)return;let e=i[o].indexOf(n);e>-1&&i[o].splice(e,1),n instanceof K&&l.delete(n.compartment)}if(r.set(n,s),Array.isArray(n))for(let t of n)e(t,s);else if(n instanceof K){if(l.has(n.compartment))throw RangeError("Duplicate use of compartment in extensions");let i=t.get(n.compartment)||n.inner;l.set(n.compartment,i),e(i,s)}else if(n instanceof Y)e(n.inner,n.prec);else if(n instanceof U)i[s].push(n),n.provides&&e(n.provides,s);else if(n instanceof J)i[s].push(n),n.facet.extensions&&e(n.facet.extensions,V.default);else{let t=n.extension;if(!t)throw Error(`Unrecognized extension value in extension set (${n}). This sometimes happens because multiple instances of @codemirror/state are loaded, breaking instanceof checks.`);e(t,s)}}(e,V.default),i.reduce((e,t)=>e.concat(t))))n instanceof U?s.push(n):(o[n.facet.id]||(o[n.facet.id]=[])).push(n);let a=Object.create(null),h=[],c=[];for(let e of s)a[e.id]=c.length<<1,c.push(t=>e.slot(t));let u=null==n?void 0:n.config.facets;for(let e in o){let t=o[e],i=t[0].facet,r=u&&u[e]||[];if(t.every(e=>0==e.type)){if(a[i.id]=h.length<<1|1,q(r,t))h.push(n.facet(i));else{let e=i.combine(t.map(e=>e.value));h.push(n&&i.compare(e,n.facet(i))?n.facet(i):e)}}else{for(let e of t)0==e.type?(a[e.id]=h.length<<1|1,h.push(e.value)):(a[e.id]=c.length<<1,c.push(t=>e.dynamicSlot(t)));a[i.id]=c.length<<1,c.push(e=>(function(e,t,n){let i=n.map(t=>e[t.id]),r=n.map(e=>e.type),s=i.filter(e=>!(1&e)),o=e[t.id]>>1;function l(e){let n=[];for(let t=0;t<i.length;t++){let s=en(e,i[t]);if(2==r[t])for(let e of s)n.push(e);else n.push(s)}return t.combine(n)}return{create(e){for(let t of i)et(e,t);return e.values[o]=l(e),1},update(e,n){if(!_(e,s))return 0;let i=l(e);return t.compare(i,e.values[o])?0:(e.values[o]=i,1)},reconfigure(e,r){let s=_(e,i),a=r.config.facets[t.id],h=r.facet(t);if(a&&!s&&q(n,a))return e.values[o]=h,0;let c=l(e);return t.compare(c,h)?(e.values[o]=h,0):(e.values[o]=c,1)}}})(e,i,t))}}return new ee(e,l,c.map(e=>e(a)),a,h,o)}}function et(e,t){if(1&t)return 2;let n=t>>1,i=e.status[n];if(4==i)throw Error("Cyclic dependency between fields and/or facets");if(2&i)return i;e.status[n]=4;let r=e.computeSlot(e,e.config.dynamicSlots[n]);return e.status[n]=2|r}function en(e,t){return 1&t?e.config.staticValues[t>>1]:e.values[t>>1]}let ei=X.define(),er=X.define({combine:e=>e.some(e=>e),static:!0}),es=X.define({combine:e=>e.length?e[0]:void 0,static:!0}),eo=X.define(),el=X.define(),ea=X.define(),eh=X.define({combine:e=>!!e.length&&e[0]});class ec{constructor(e,t){this.type=e,this.value=t}static define(){return new eu}}class eu{of(e){return new ec(this,e)}}class ef{constructor(e){this.map=e}of(e){return new ed(this,e)}}class ed{constructor(e,t){this.type=e,this.value=t}map(e){let t=this.type.map(this.value,e);return void 0===t?void 0:t==this.value?this:new ed(this.type,t)}is(e){return this.type==e}static define(e={}){return new ef(e.map||(e=>e))}static mapEffects(e,t){if(!e.length)return e;let n=[];for(let i of e){let e=i.map(t);e&&n.push(e)}return n}}ed.reconfigure=ed.define(),ed.appendConfig=ed.define();class ep{constructor(e,t,n,i,r,s){this.startState=e,this.changes=t,this.selection=n,this.effects=i,this.annotations=r,this.scrollIntoView=s,this._doc=null,this._state=null,n&&j(n,t.newLength),r.some(e=>e.type==ep.time)||(this.annotations=r.concat(ep.time.of(Date.now())))}static create(e,t,n,i,r,s){return new ep(e,t,n,i,r,s)}get newDoc(){return this._doc||(this._doc=this.changes.apply(this.startState.doc))}get newSelection(){return this.selection||this.startState.selection.map(this.changes)}get state(){return this._state||this.startState.applyTransaction(this),this._state}annotation(e){for(let t of this.annotations)if(t.type==e)return t.value}get docChanged(){return!this.changes.empty}get reconfigured(){return this.startState.config!=this.state.config}isUserEvent(e){let t=this.annotation(ep.userEvent);return!!(t&&(t==e||t.length>e.length&&t.slice(0,e.length)==e&&"."==t[e.length]))}}function em(e,t,n){var i;let r,s,o;return n?(r=t.changes,s=D.empty(t.changes.length),o=e.changes.compose(t.changes)):(r=t.changes.map(e.changes),s=e.changes.mapDesc(t.changes,!0),o=e.changes.compose(r)),{changes:o,selection:t.selection?t.selection.map(s):null===(i=e.selection)||void 0===i?void 0:i.map(r),effects:ed.mapEffects(e.effects,r).concat(ed.mapEffects(t.effects,s)),annotations:e.annotations.length?e.annotations.concat(t.annotations):t.annotations,scrollIntoView:e.scrollIntoView||t.scrollIntoView}}function eg(e,t,n){let i=t.selection,r=ev(t.annotations);return t.userEvent&&(r=r.concat(ep.userEvent.of(t.userEvent))),{changes:t.changes instanceof D?t.changes:D.of(t.changes||[],n,e.facet(es)),selection:i&&(i instanceof $?i:$.single(i.anchor,i.head)),effects:ev(t.effects),annotations:r,scrollIntoView:!!t.scrollIntoView}}ep.time=ec.define(),ep.userEvent=ec.define(),ep.addToHistory=ec.define(),ep.remote=ec.define();let ek=[];function ev(e){return null==e?ek:Array.isArray(e)?e:[e]}var ey=((A=ey||(ey={}))[A.Word=0]="Word",A[A.Space=1]="Space",A[A.Other=2]="Other",A);let ex=/[\u00df\u0587\u0590-\u05f4\u0600-\u06ff\u3040-\u309f\u30a0-\u30ff\u3400-\u4db5\u4e00-\u9fcc\uac00-\ud7af]/;try{i=RegExp("[\\p{Alphabetic}\\p{Number}_]","u")}catch(e){}class eb{constructor(e,t,n,i,r,s){this.config=e,this.doc=t,this.selection=n,this.values=i,this.status=e.statusTemplate.slice(),this.computeSlot=r,s&&(s._state=this);for(let e=0;e<this.config.dynamicSlots.length;e++)et(this,e<<1);this.computeSlot=null}field(e,t=!0){let n=this.config.address[e.id];if(null==n){if(t)throw RangeError("Field is not present in this state");return}return et(this,n),en(this,n)}update(...e){return function e(t,n,i){let r=eg(t,n.length?n[0]:{},t.doc.length);n.length&&!1===n[0].filter&&(i=!1);for(let e=1;e<n.length;e++){!1===n[e].filter&&(i=!1);let s=!!n[e].sequential;r=em(r,eg(t,n[e],s?r.changes.newLength:t.doc.length),s)}let s=ep.create(t,r.changes,r.selection,r.effects,r.annotations,r.scrollIntoView);return function(e){let t=e.startState,n=t.facet(ea),i=e;for(let r=n.length-1;r>=0;r--){let s=n[r](e);s&&Object.keys(s).length&&(i=em(i,eg(t,s,e.changes.newLength),!0))}return i==e?e:ep.create(t,e.changes,e.selection,i.effects,i.annotations,i.scrollIntoView)}(i?function(t){let n=t.startState,i=!0;for(let e of n.facet(eo)){let n=e(t);if(!1===n){i=!1;break}Array.isArray(n)&&(i=!0===i?n:function(e,t){let n=[];for(let i=0,r=0;;){let s,o;if(i<e.length&&(r==t.length||t[r]>=e[i]))s=e[i++],o=e[i++];else{if(!(r<t.length))return n;s=t[r++],o=t[r++]}!n.length||n[n.length-1]<s?n.push(s,o):n[n.length-1]<o&&(n[n.length-1]=o)}}(i,n))}if(!0!==i){let e,r;if(!1===i)r=t.changes.invertedDesc,e=D.empty(n.doc.length);else{let n=t.changes.filter(i);e=n.changes,r=n.filtered.mapDesc(n.changes).invertedDesc}t=ep.create(n,e,t.selection&&t.selection.map(r),ed.mapEffects(t.effects,r),t.annotations,t.scrollIntoView)}let r=n.facet(el);for(let i=r.length-1;i>=0;i--){let s=r[i](t);t=s instanceof ep?s:Array.isArray(s)&&1==s.length&&s[0]instanceof ep?s[0]:e(n,ev(s),!1)}return t}(s):s)}(this,e,!0)}applyTransaction(e){let t,n=this.config,{base:i,compartments:r}=n;for(let t of e.effects)t.is(H.reconfigure)?(n&&(r=new Map,n.compartments.forEach((e,t)=>r.set(t,e)),n=null),r.set(t.value.compartment,t.value.extension)):t.is(ed.reconfigure)?(n=null,i=t.value):t.is(ed.appendConfig)&&(n=null,i=ev(i).concat(t.value));t=n?e.startState.values.slice():new eb(n=ee.resolve(i,r,this),this.doc,this.selection,n.dynamicSlots.map(()=>null),(e,t)=>t.reconfigure(e,this),null).values;let s=e.startState.facet(er)?e.newSelection:e.newSelection.asSingle();new eb(n,e.newDoc,s,t,(t,n)=>n.update(t,e),e)}replaceSelection(e){return"string"==typeof e&&(e=this.toText(e)),this.changeByRange(t=>({changes:{from:t.from,to:t.to,insert:e},range:$.cursor(t.from+e.length)}))}changeByRange(e){let t=this.selection,n=e(t.ranges[0]),i=this.changes(n.changes),r=[n.range],s=ev(n.effects);for(let n=1;n<t.ranges.length;n++){let o=e(t.ranges[n]),l=this.changes(o.changes),a=l.map(i);for(let e=0;e<n;e++)r[e]=r[e].map(a);let h=i.mapDesc(l,!0);r.push(o.range.map(h)),i=i.compose(a),s=ed.mapEffects(s,a).concat(ed.mapEffects(ev(o.effects),h))}return{changes:i,selection:$.create(r,t.mainIndex),effects:s}}changes(e=[]){return e instanceof D?e:D.of(e,this.doc.length,this.facet(eb.lineSeparator))}toText(e){return f.of(e.split(this.facet(eb.lineSeparator)||P))}sliceDoc(e=0,t=this.doc.length){return this.doc.sliceString(e,t,this.lineBreak)}facet(e){let t=this.config.address[e.id];return null==t?e.default:(et(this,t),en(this,t))}toJSON(e){let t={doc:this.sliceDoc(),selection:this.selection.toJSON()};if(e)for(let n in e){let i=e[n];i instanceof U&&null!=this.config.address[i.id]&&(t[n]=i.spec.toJSON(this.field(e[n]),this))}return t}static fromJSON(e,t={},n){if(!e||"string"!=typeof e.doc)throw RangeError("Invalid JSON representation for EditorState");let i=[];if(n){for(let t in n)if(Object.prototype.hasOwnProperty.call(e,t)){let r=n[t],s=e[t];i.push(r.init(e=>r.spec.fromJSON(s,e)))}}return eb.create({doc:e.doc,selection:$.fromJSON(e.selection),extensions:t.extensions?i.concat([t.extensions]):i})}static create(e={}){let t=ee.resolve(e.extensions||[],new Map),n=e.doc instanceof f?e.doc:f.of((e.doc||"").split(t.staticFacet(eb.lineSeparator)||P)),i=e.selection?e.selection instanceof $?e.selection:$.single(e.selection.anchor,e.selection.head):$.single(0);return j(i,n.length),t.staticFacet(er)||(i=i.asSingle()),new eb(t,n,i,t.dynamicSlots.map(()=>null),(e,t)=>t.create(e),null)}get tabSize(){return this.facet(eb.tabSize)}get lineBreak(){return this.facet(eb.lineSeparator)||"\n"}get readOnly(){return this.facet(eh)}phrase(e,...t){for(let t of this.facet(eb.phrases))if(Object.prototype.hasOwnProperty.call(t,e)){e=t[e];break}return t.length&&(e=e.replace(/\$(\$|\d*)/g,(e,n)=>{if("$"==n)return"$";let i=+(n||1);return!i||i>t.length?e:t[i-1]})),e}languageDataAt(e,t,n=-1){let i=[];for(let r of this.facet(ei))for(let s of r(this,t,n))Object.prototype.hasOwnProperty.call(s,e)&&i.push(s[e]);return i}charCategorizer(e){var t;let n=this.languageDataAt("wordChars",e);return t=n.length?n[0]:"",e=>{if(!/\S/.test(e))return ey.Space;if(function(e){if(i)return i.test(e);for(let t=0;t<e.length;t++){let n=e[t];if(/\w/.test(n)||n>"\x80"&&(n.toUpperCase()!=n.toLowerCase()||ex.test(n)))return!0}return!1}(e))return ey.Word;for(let n=0;n<t.length;n++)if(e.indexOf(t[n])>-1)return ey.Word;return ey.Other}}wordAt(e){let{text:t,from:n,length:i}=this.doc.lineAt(e),r=this.charCategorizer(e),s=e-n,o=e-n;for(;s>0;){let e=b(t,s,!1);if(r(t.slice(e,s))!=ey.Word)break;s=e}for(;o<i;){let e=b(t,o);if(r(t.slice(o,e))!=ey.Word)break;o=e}return s==o?null:$.range(s+n,o+n)}}function eO(e,t,n={}){let i={};for(let t of e)for(let e of Object.keys(t)){let r=t[e],s=i[e];if(void 0===s)i[e]=r;else if(s===r||void 0===r);else if(Object.hasOwnProperty.call(n,e))i[e]=n[e](s,r);else throw Error("Config merge conflict for field "+e)}for(let e in t)void 0===i[e]&&(i[e]=t[e]);return i}eb.allowMultipleSelections=er,eb.tabSize=X.define({combine:e=>e.length?e[0]:4}),eb.lineSeparator=es,eb.readOnly=eh,eb.phrases=X.define({compare(e,t){let n=Object.keys(e),i=Object.keys(t);return n.length==i.length&&n.every(n=>e[n]==t[n])}}),eb.languageData=ei,eb.changeFilter=eo,eb.transactionFilter=el,eb.transactionExtender=ea,H.reconfigure=ed.define();class ew{eq(e){return this==e}range(e,t=e){return eP.create(e,t,this)}}function eS(e,t){return e==t||e.constructor==t.constructor&&e.eq(t)}ew.prototype.startSide=ew.prototype.endSide=0,ew.prototype.point=!1,ew.prototype.mapMode=T.TrackDel;class eP{constructor(e,t,n){this.from=e,this.to=t,this.value=n}static create(e,t,n){return new eP(e,t,n)}}function eC(e,t){return e.from-t.from||e.value.startSide-t.value.startSide}class eA{constructor(e,t,n,i){this.from=e,this.to=t,this.value=n,this.maxPoint=i}get length(){return this.to[this.to.length-1]}findIndex(e,t,n,i=0){let r=n?this.to:this.from;for(let s=i,o=r.length;;){if(s==o)return s;let i=s+o>>1,l=r[i]-e||(n?this.value[i].endSide:this.value[i].startSide)-t;if(i==s)return l>=0?s:o;l>=0?o=i:s=i+1}}between(e,t,n,i){for(let r=this.findIndex(t,-1e9,!0),s=this.findIndex(n,1e9,!1,r);r<s;r++)if(!1===i(this.from[r]+e,this.to[r]+e,this.value[r]))return!1}map(e,t){let n=[],i=[],r=[],s=-1,o=-1;for(let l=0;l<this.value.length;l++){let a=this.value[l],h=this.from[l]+e,c=this.to[l]+e,u,f;if(h==c){let e=t.mapPos(h,a.startSide,a.mapMode);if(null==e||(u=f=e,a.startSide!=a.endSide&&(f=t.mapPos(h,a.endSide))<u))continue}else if((u=t.mapPos(h,a.startSide))>(f=t.mapPos(c,a.endSide))||u==f&&a.startSide>0&&a.endSide<=0)continue;0>(f-u||a.endSide-a.startSide)||(s<0&&(s=u),a.point&&(o=Math.max(o,f-u)),n.push(a),i.push(u-s),r.push(f-s))}return{mapped:n.length?new eA(i,r,n,o):null,pos:s}}}class eT{constructor(e,t,n,i){this.chunkPos=e,this.chunk=t,this.nextLayer=n,this.maxPoint=i}static create(e,t,n,i){return new eT(e,t,n,i)}get length(){let e=this.chunk.length-1;return e<0?0:Math.max(this.chunkEnd(e),this.nextLayer.length)}get size(){if(this.isEmpty)return 0;let e=this.nextLayer.size;for(let t of this.chunk)e+=t.value.length;return e}chunkEnd(e){return this.chunkPos[e]+this.chunk[e].length}update(e){let{add:t=[],sort:n=!1,filterFrom:i=0,filterTo:r=this.length}=e,s=e.filter;if(0==t.length&&!s)return this;if(n&&(t=t.slice().sort(eC)),this.isEmpty)return t.length?eT.of(t):this;let o=new eM(this,null,-1).goto(0),l=0,a=[],h=new eR;for(;o.value||l<t.length;)if(l<t.length&&(o.from-t[l].from||o.startSide-t[l].value.startSide)>=0){let e=t[l++];h.addInner(e.from,e.to,e.value)||a.push(e)}else 1==o.rangeIndex&&o.chunkIndex<this.chunk.length&&(l==t.length||this.chunkEnd(o.chunkIndex)<t[l].from)&&(!s||i>this.chunkEnd(o.chunkIndex)||r<this.chunkPos[o.chunkIndex])&&h.addChunk(this.chunkPos[o.chunkIndex],this.chunk[o.chunkIndex])?o.nextChunk():((!s||i>o.to||r<o.from||s(o.from,o.to,o.value))&&!h.addInner(o.from,o.to,o.value)&&a.push(eP.create(o.from,o.to,o.value)),o.next());return h.finishInner(this.nextLayer.isEmpty&&!a.length?eT.empty:this.nextLayer.update({add:a,filter:s,filterFrom:i,filterTo:r}))}map(e){if(e.empty||this.isEmpty)return this;let t=[],n=[],i=-1;for(let r=0;r<this.chunk.length;r++){let s=this.chunkPos[r],o=this.chunk[r],l=e.touchesRange(s,s+o.length);if(!1===l)i=Math.max(i,o.maxPoint),t.push(o),n.push(e.mapPos(s));else if(!0===l){let{mapped:r,pos:l}=o.map(s,e);r&&(i=Math.max(i,r.maxPoint),t.push(r),n.push(l))}}let r=this.nextLayer.map(e);return 0==t.length?r:new eT(n,t,r||eT.empty,i)}between(e,t,n){if(!this.isEmpty){for(let i=0;i<this.chunk.length;i++){let r=this.chunkPos[i],s=this.chunk[i];if(t>=r&&e<=r+s.length&&!1===s.between(r,e-r,t-r,n))return}this.nextLayer.between(e,t,n)}}iter(e=0){return eI.from([this]).goto(e)}get isEmpty(){return this.nextLayer==this}static iter(e,t=0){return eI.from(e).goto(t)}static compare(e,t,n,i,r=-1){let s=e.filter(e=>e.maxPoint>0||!e.isEmpty&&e.maxPoint>=r),o=t.filter(e=>e.maxPoint>0||!e.isEmpty&&e.maxPoint>=r),l=eD(s,o,n),a=new eN(s,l,r),h=new eN(o,l,r);n.iterGaps((e,t,n)=>eE(a,e,h,t,n,i)),n.empty&&0==n.length&&eE(a,0,h,0,0,i)}static eq(e,t,n=0,i){null==i&&(i=999999999);let r=e.filter(e=>!e.isEmpty&&0>t.indexOf(e)),s=t.filter(t=>!t.isEmpty&&0>e.indexOf(t));if(r.length!=s.length)return!1;if(!r.length)return!0;let o=eD(r,s),l=new eN(r,o,0).goto(n),a=new eN(s,o,0).goto(n);for(;;){if(l.to!=a.to||!eB(l.active,a.active)||l.point&&(!a.point||!eS(l.point,a.point)))return!1;if(l.to>i)return!0;l.next(),a.next()}}static spans(e,t,n,i,r=-1){let s=new eN(e,null,r).goto(t),o=t,l=s.openStart;for(;;){let e=Math.min(s.to,n);if(s.point){let n=s.activeForPoint(s.to),r=s.pointFrom<t?n.length+1:s.point.startSide<0?n.length:Math.min(n.length,l);i.point(o,e,s.point,n,r,s.pointRank),l=Math.min(s.openEnd(e),n.length)}else e>o&&(i.span(o,e,s.active,l),l=s.openEnd(e));if(s.to>n)return l+(s.point&&s.to>n?1:0);o=s.to,s.next()}}static of(e,t=!1){let n=new eR;for(let i of e instanceof eP?[e]:t?function(e){if(e.length>1)for(let t=e[0],n=1;n<e.length;n++){let i=e[n];if(eC(t,i)>0)return e.slice().sort(eC);t=i}return e}(e):e)n.add(i.from,i.to,i.value);return n.finish()}static join(e){if(!e.length)return eT.empty;let t=e[e.length-1];for(let n=e.length-2;n>=0;n--)for(let i=e[n];i!=eT.empty;i=i.nextLayer)t=new eT(i.chunkPos,i.chunk,t,Math.max(i.maxPoint,t.maxPoint));return t}}eT.empty=new eT([],[],null,-1),eT.empty.nextLayer=eT.empty;class eR{finishChunk(e){this.chunks.push(new eA(this.from,this.to,this.value,this.maxPoint)),this.chunkPos.push(this.chunkStart),this.chunkStart=-1,this.setMaxPoint=Math.max(this.setMaxPoint,this.maxPoint),this.maxPoint=-1,e&&(this.from=[],this.to=[],this.value=[])}constructor(){this.chunks=[],this.chunkPos=[],this.chunkStart=-1,this.last=null,this.lastFrom=-1e9,this.lastTo=-1e9,this.from=[],this.to=[],this.value=[],this.maxPoint=-1,this.setMaxPoint=-1,this.nextLayer=null}add(e,t,n){this.addInner(e,t,n)||(this.nextLayer||(this.nextLayer=new eR)).add(e,t,n)}addInner(e,t,n){let i=e-this.lastTo||n.startSide-this.last.endSide;if(i<=0&&0>(e-this.lastFrom||n.startSide-this.last.startSide))throw Error("Ranges must be added sorted by `from` position and `startSide`");return!(i<0)&&(250==this.from.length&&this.finishChunk(!0),this.chunkStart<0&&(this.chunkStart=e),this.from.push(e-this.chunkStart),this.to.push(t-this.chunkStart),this.last=n,this.lastFrom=e,this.lastTo=t,this.value.push(n),n.point&&(this.maxPoint=Math.max(this.maxPoint,t-e)),!0)}addChunk(e,t){if(0>(e-this.lastTo||t.value[0].startSide-this.last.endSide))return!1;this.from.length&&this.finishChunk(!0),this.setMaxPoint=Math.max(this.setMaxPoint,t.maxPoint),this.chunks.push(t),this.chunkPos.push(e);let n=t.value.length-1;return this.last=t.value[n],this.lastFrom=t.from[n]+e,this.lastTo=t.to[n]+e,!0}finish(){return this.finishInner(eT.empty)}finishInner(e){if(this.from.length&&this.finishChunk(!1),0==this.chunks.length)return e;let t=eT.create(this.chunkPos,this.chunks,this.nextLayer?this.nextLayer.finishInner(e):e,this.setMaxPoint);return this.from=null,t}}function eD(e,t,n){let i=new Map;for(let t of e)for(let e=0;e<t.chunk.length;e++)t.chunk[e].maxPoint<=0&&i.set(t.chunk[e],t.chunkPos[e]);let r=new Set;for(let e of t)for(let t=0;t<e.chunk.length;t++){let s=i.get(e.chunk[t]);null==s||(n?n.mapPos(s):s)!=e.chunkPos[t]||(null==n?void 0:n.touchesRange(s,s+e.chunk[t].length))||r.add(e.chunk[t])}return r}class eM{constructor(e,t,n,i=0){this.layer=e,this.skip=t,this.minPoint=n,this.rank=i}get startSide(){return this.value?this.value.startSide:0}get endSide(){return this.value?this.value.endSide:0}goto(e,t=-1e9){return this.chunkIndex=this.rangeIndex=0,this.gotoInner(e,t,!1),this}gotoInner(e,t,n){for(;this.chunkIndex<this.layer.chunk.length;){let t=this.layer.chunk[this.chunkIndex];if(!(this.skip&&this.skip.has(t)||this.layer.chunkEnd(this.chunkIndex)<e||t.maxPoint<this.minPoint))break;this.chunkIndex++,n=!1}if(this.chunkIndex<this.layer.chunk.length){let i=this.layer.chunk[this.chunkIndex].findIndex(e-this.layer.chunkPos[this.chunkIndex],t,!0);(!n||this.rangeIndex<i)&&this.setRangeIndex(i)}this.next()}forward(e,t){0>(this.to-e||this.endSide-t)&&this.gotoInner(e,t,!0)}next(){for(;;){if(this.chunkIndex==this.layer.chunk.length){this.from=this.to=1e9,this.value=null;break}{let e=this.layer.chunkPos[this.chunkIndex],t=this.layer.chunk[this.chunkIndex],n=e+t.from[this.rangeIndex];if(this.from=n,this.to=e+t.to[this.rangeIndex],this.value=t.value[this.rangeIndex],this.setRangeIndex(this.rangeIndex+1),this.minPoint<0||this.value.point&&this.to-this.from>=this.minPoint)break}}}setRangeIndex(e){if(e==this.layer.chunk[this.chunkIndex].value.length){if(this.chunkIndex++,this.skip)for(;this.chunkIndex<this.layer.chunk.length&&this.skip.has(this.layer.chunk[this.chunkIndex]);)this.chunkIndex++;this.rangeIndex=0}else this.rangeIndex=e}nextChunk(){this.chunkIndex++,this.rangeIndex=0,this.next()}compare(e){return this.from-e.from||this.startSide-e.startSide||this.rank-e.rank||this.to-e.to||this.endSide-e.endSide}}class eI{constructor(e){this.heap=e}static from(e,t=null,n=-1){let i=[];for(let r=0;r<e.length;r++)for(let s=e[r];!s.isEmpty;s=s.nextLayer)s.maxPoint>=n&&i.push(new eM(s,t,n,r));return 1==i.length?i[0]:new eI(i)}get startSide(){return this.value?this.value.startSide:0}goto(e,t=-1e9){for(let n of this.heap)n.goto(e,t);for(let e=this.heap.length>>1;e>=0;e--)eQ(this.heap,e);return this.next(),this}forward(e,t){for(let n of this.heap)n.forward(e,t);for(let e=this.heap.length>>1;e>=0;e--)eQ(this.heap,e);0>(this.to-e||this.value.endSide-t)&&this.next()}next(){if(0==this.heap.length)this.from=this.to=1e9,this.value=null,this.rank=-1;else{let e=this.heap[0];this.from=e.from,this.to=e.to,this.value=e.value,this.rank=e.rank,e.value&&e.next(),eQ(this.heap,0)}}}function eQ(e,t){for(let n=e[t];;){let i=(t<<1)+1;if(i>=e.length)break;let r=e[i];if(i+1<e.length&&r.compare(e[i+1])>=0&&(r=e[i+1],i++),0>n.compare(r))break;e[i]=n,e[t]=r,t=i}}class eN{constructor(e,t,n){this.minPoint=n,this.active=[],this.activeTo=[],this.activeRank=[],this.minActive=-1,this.point=null,this.pointFrom=0,this.pointRank=0,this.to=-1e9,this.endSide=0,this.openStart=-1,this.cursor=eI.from(e,t,n)}goto(e,t=-1e9){return this.cursor.goto(e,t),this.active.length=this.activeTo.length=this.activeRank.length=0,this.minActive=-1,this.to=e,this.endSide=t,this.openStart=-1,this.next(),this}forward(e,t){for(;this.minActive>-1&&0>(this.activeTo[this.minActive]-e||this.active[this.minActive].endSide-t);)this.removeActive(this.minActive);this.cursor.forward(e,t)}removeActive(e){eL(this.active,e),eL(this.activeTo,e),eL(this.activeRank,e),this.minActive=ej(this.active,this.activeTo)}addActive(e){let t=0,{value:n,to:i,rank:r}=this.cursor;for(;t<this.activeRank.length&&(r-this.activeRank[t]||i-this.activeTo[t])>0;)t++;e$(this.active,t,n),e$(this.activeTo,t,i),e$(this.activeRank,t,r),e&&e$(e,t,this.cursor.from),this.minActive=ej(this.active,this.activeTo)}next(){let e=this.to,t=this.point;this.point=null;let n=this.openStart<0?[]:null;for(;;){let i=this.minActive;if(i>-1&&0>(this.activeTo[i]-this.cursor.from||this.active[i].endSide-this.cursor.startSide)){if(this.activeTo[i]>e){this.to=this.activeTo[i],this.endSide=this.active[i].endSide;break}this.removeActive(i),n&&eL(n,i)}else if(this.cursor.value){if(this.cursor.from>e){this.to=this.cursor.from,this.endSide=this.cursor.startSide;break}{let e=this.cursor.value;if(e.point){if(t&&this.cursor.to==this.to&&this.cursor.from<this.cursor.to)this.cursor.next();else{this.point=e,this.pointFrom=this.cursor.from,this.pointRank=this.cursor.rank,this.to=this.cursor.to,this.endSide=e.endSide,this.cursor.next(),this.forward(this.to,this.endSide);break}}else this.addActive(n),this.cursor.next()}}else{this.to=this.endSide=1e9;break}}if(n){this.openStart=0;for(let t=n.length-1;t>=0&&n[t]<e;t--)this.openStart++}}activeForPoint(e){if(!this.active.length)return this.active;let t=[];for(let n=this.active.length-1;n>=0&&!(this.activeRank[n]<this.pointRank);n--)(this.activeTo[n]>e||this.activeTo[n]==e&&this.active[n].endSide>=this.point.endSide)&&t.push(this.active[n]);return t.reverse()}openEnd(e){let t=0;for(let n=this.activeTo.length-1;n>=0&&this.activeTo[n]>e;n--)t++;return t}}function eE(e,t,n,i,r,s){e.goto(t),n.goto(i);let o=i+r,l=i,a=i-t,h=!!s.boundChange;for(let t=!1;;){let i=e.to+a-n.to,r=i||e.endSide-n.endSide,c=r<0?e.to+a:n.to,u=Math.min(c,o);if(e.point||n.point?(e.point&&n.point&&eS(e.point,n.point)&&eB(e.activeForPoint(e.to),n.activeForPoint(n.to))||s.comparePoint(l,u,e.point,n.point),t=!1):(t&&s.boundChange(l),u>l&&!eB(e.active,n.active)&&s.compareRange(l,u,e.active,n.active),h&&u<o&&(i||e.openEnd(c)!=n.openEnd(c))&&(t=!0)),c>o)break;l=c,r<=0&&e.next(),r>=0&&n.next()}}function eB(e,t){if(e.length!=t.length)return!1;for(let n=0;n<e.length;n++)if(e[n]!=t[n]&&!eS(e[n],t[n]))return!1;return!0}function eL(e,t){for(let n=t,i=e.length-1;n<i;n++)e[n]=e[n+1];e.pop()}function e$(e,t,n){for(let n=e.length-1;n>=t;n--)e[n+1]=e[n];e[t]=n}function ej(e,t){let n=-1,i=1e9;for(let r=0;r<t.length;r++)0>(t[r]-i||e[r].endSide-e[n].endSide)&&(n=r,i=t[r]);return n}function ez(e,t,n=e.length){let i=0;for(let r=0;r<n&&r<e.length;)9==e.charCodeAt(r)?(i+=t-i%t,r++):(i++,r=b(e,r));return i}function eX(e,t,n,i){for(let i=0,r=0;;){if(r>=t)return i;if(i==e.length)break;r+=9==e.charCodeAt(i)?n-r%n:1,i=b(e,i)}return!0===i?-1:e.length}},41113:function(e,t,n){var i,r;n.d(t,{Jq:function(){return u},L3:function(){return s},Lj:function(){return f},_b:function(){return N},i9:function(){return Q},md:function(){return a},mp:function(){return m},vj:function(){return i}});let s=1024,o=0;class l{constructor(e,t){this.from=e,this.to=t}}class a{constructor(e={}){this.id=o++,this.perNode=!!e.perNode,this.deserialize=e.deserialize||(()=>{throw Error("This node type doesn't define a deserialize function")}),this.combine=e.combine||null}add(e){if(this.perNode)throw RangeError("Can't add per-node props to node types");return"function"!=typeof e&&(e=u.match(e)),t=>{let n=e(t);return void 0===n?null:[this,n]}}}a.closedBy=new a({deserialize:e=>e.split(" ")}),a.openedBy=new a({deserialize:e=>e.split(" ")}),a.group=new a({deserialize:e=>e.split(" ")}),a.isolate=new a({deserialize:e=>{if(e&&"rtl"!=e&&"ltr"!=e&&"auto"!=e)throw RangeError("Invalid value for isolate: "+e);return e||"auto"}}),a.contextHash=new a({perNode:!0}),a.lookAhead=new a({perNode:!0}),a.mounted=new a({perNode:!0});class h{constructor(e,t,n,i=!1){this.tree=e,this.overlay=t,this.parser=n,this.bracketed=i}static get(e){return e&&e.props&&e.props[a.mounted.id]}}let c=Object.create(null);class u{constructor(e,t,n,i=0){this.name=e,this.props=t,this.id=n,this.flags=i}static define(e){let t=e.props&&e.props.length?Object.create(null):c,n=(e.top?1:0)|(e.skipped?2:0)|(e.error?4:0)|(null==e.name?8:0),i=new u(e.name||"",t,e.id,n);if(e.props){for(let n of e.props)if(Array.isArray(n)||(n=n(i)),n){if(n[0].perNode)throw RangeError("Can't store a per-node prop on a node type");t[n[0].id]=n[1]}}return i}prop(e){return this.props[e.id]}get isTop(){return(1&this.flags)>0}get isSkipped(){return(2&this.flags)>0}get isError(){return(4&this.flags)>0}get isAnonymous(){return(8&this.flags)>0}is(e){if("string"==typeof e){if(this.name==e)return!0;let t=this.prop(a.group);return!!t&&t.indexOf(e)>-1}return this.id==e}static match(e){let t=Object.create(null);for(let n in e)for(let i of n.split(" "))t[i]=e[n];return e=>{for(let n=e.prop(a.group),i=-1;i<(n?n.length:0);i++){let r=t[i<0?e.name:n[i]];if(r)return r}}}}u.none=new u("",Object.create(null),0,8);class f{constructor(e){this.types=e;for(let t=0;t<e.length;t++)if(e[t].id!=t)throw RangeError("Node type ids should correspond to array positions when creating a node set")}extend(...e){let t=[];for(let n of this.types){let i=null;for(let t of e){let e=t(n);if(e){i||(i=Object.assign({},n.props));let t=e[1],r=e[0];r.combine&&r.id in i&&(t=r.combine(i[r.id],t)),i[r.id]=t}}t.push(i?new u(n.name,i,n.id,n.flags):n)}return new f(t)}}let d=new WeakMap,p=new WeakMap;(r=i||(i={}))[r.ExcludeBuffers=1]="ExcludeBuffers",r[r.IncludeAnonymous=2]="IncludeAnonymous",r[r.IgnoreMounts=4]="IgnoreMounts",r[r.IgnoreOverlays=8]="IgnoreOverlays",r[r.EnterBracketed=16]="EnterBracketed";class m{constructor(e,t,n,i,r){if(this.type=e,this.children=t,this.positions=n,this.length=i,this.props=null,r&&r.length)for(let[e,t]of(this.props=Object.create(null),r))this.props["number"==typeof e?e:e.id]=t}toString(){let e=h.get(this);if(e&&!e.overlay)return e.tree.toString();let t="";for(let e of this.children){let n=e.toString();n&&(t&&(t+=","),t+=n)}return this.type.name?(/\W/.test(this.type.name)&&!this.type.isError?JSON.stringify(this.type.name):this.type.name)+(t.length?"("+t+")":""):t}cursor(e=0){return new T(this.topNode,e)}cursorAt(e,t=0,n=0){let i=new T(d.get(this)||this.topNode);return i.moveTo(e,t),d.set(this,i._tree),i}get topNode(){return new b(this,0,0,null)}resolve(e,t=0){let n=y(d.get(this)||this.topNode,e,t,!1);return d.set(this,n),n}resolveInner(e,t=0){let n=y(p.get(this)||this.topNode,e,t,!0);return p.set(this,n),n}resolveStack(e,t=0){return function(e,t,n){let i=e.resolveInner(t,n),r=null;for(let e=i instanceof b?i:i.context.parent;e;e=e.parent)if(e.index<0){let s=e.parent;(r||(r=[i])).push(s.resolve(t,n)),e=s}else{let s=h.get(e.tree);if(s&&s.overlay&&s.overlay[0].from<=t&&s.overlay[s.overlay.length-1].to>=t){let o=new b(s.tree,s.overlay[0].from+e.from,-1,e);(r||(r=[i])).push(y(o,t,n,!1))}}return r?C(r):i}(this,e,t)}iterate(e){let{enter:t,leave:n,from:r=0,to:s=this.length}=e,o=e.mode||0,l=(o&i.IncludeAnonymous)>0;for(let e=this.cursor(o|i.IncludeAnonymous);;){let i=!1;if(e.from<=s&&e.to>=r&&(!l&&e.type.isAnonymous||!1!==t(e))){if(e.firstChild())continue;i=!0}for(;i&&n&&(l||!e.type.isAnonymous)&&n(e),!e.nextSibling();){if(!e.parent())return;i=!0}}}prop(e){return e.perNode?this.props?this.props[e.id]:void 0:this.type.prop(e)}get propValues(){let e=[];if(this.props)for(let t in this.props)e.push([+t,this.props[t]]);return e}balance(e={}){return this.children.length<=8?this:I(u.none,this.children,this.positions,0,this.children.length,0,this.length,(e,t,n)=>new m(this.type,e,t,n,this.propValues),e.makeTree||((e,t,n)=>new m(u.none,e,t,n)))}static build(e){return function(e){var t;let{buffer:n,nodeSet:i,maxBufferLength:r=s,reused:o=[],minRepeatType:l=i.types.length}=e,h=Array.isArray(n)?new g(n,n.length):n,c=i.types,u=0,f=0;function d(e,t,n,r,s,o,l,a,h){let c=[],u=[];for(;e.length>r;)c.push(e.pop()),u.push(t.pop()+n-s);e.push(p(i.types[l],c,u,o-s,a-o,h)),t.push(s-n)}function p(e,t,n,i,r,s,o){if(s){let e=[a.contextHash,s];o=o?[e].concat(o):[e]}if(r>25){let e=[a.lookAhead,r];o=o?[e].concat(o):[e]}return new m(e,t,n,i,o)}let v=[],y=[];for(;h.pos>0;)!function e(t,n,s,g,v,y){let{id:x,start:b,end:O,size:w}=h,S=f,P=u;if(w<0){if(h.next(),-1==w){let e=o[x];s.push(e),g.push(b-t);return}if(-3==w){u=x;return}if(-4==w){f=x;return}throw RangeError(`Unrecognized record size: ${w}`)}let C=c[x],A,T,R=b-t;if(O-b<=r&&(T=function(e,t){let n=h.fork(),i=0,s=0,o=0,a=n.end-r,c={size:0,start:0,skip:0};e:for(let r=n.pos-e;n.pos>r;){let e=n.size;if(n.id==t&&e>=0){c.size=i,c.start=s,c.skip=o,o+=4,i+=4,n.next();continue}let h=n.pos-e;if(e<0||h<r||n.start<a)break;let u=n.id>=l?4:0,f=n.start;for(n.next();n.pos>h;){if(n.size<0){if(-3==n.size||-4==n.size)u+=4;else break e}else n.id>=l&&(u+=4);n.next()}s=f,i+=e,o+=u}return(t<0||i==e)&&(c.size=i,c.start=s,c.skip=o),c.size>4?c:void 0}(h.pos-n,v))){let e=new Uint16Array(T.size-T.skip),n=h.pos-T.size,r=e.length;for(;h.pos>n;)r=function e(t,n,i){let{id:r,start:s,end:o,size:a}=h;if(h.next(),a>=0&&r<l){let l=i;if(a>4){let r=h.pos-(a-4);for(;h.pos>r;)i=e(t,n,i)}n[--i]=l,n[--i]=o-t,n[--i]=s-t,n[--i]=r}else -3==a?u=r:-4==a&&(f=r);return i}(T.start,e,r);A=new k(e,O-T.start,i),R=T.start-t}else{let t=h.pos-w;h.next();let n=[],s=[],o=x>=l?x:-1,c=0,u=O;for(;h.pos>t;)o>=0&&h.id==o&&h.size>=0?(h.end<=u-r&&(d(n,s,b,c,h.end,u,o,S,P),c=n.length,u=h.end),h.next()):y>2500?function(e,t,n,s){let o=[],l=0,a=-1;for(;h.pos>t;){let{id:e,start:t,end:n,size:i}=h;if(i>4)h.next();else if(a>-1&&t<a)break;else a<0&&(a=n-r),o.push(e,t,n),l++,h.next()}if(l){let t=new Uint16Array(4*l),r=o[o.length-2];for(let e=o.length-3,n=0;e>=0;e-=3)t[n++]=o[e],t[n++]=o[e+1]-r,t[n++]=o[e+2]-r,t[n++]=n;n.push(new k(t,o[2]-r,i)),s.push(r-e)}}(b,t,n,s):e(b,t,n,s,o,y+1);if(o>=0&&c>0&&c<n.length&&d(n,s,b,c,b,u,o,S,P),n.reverse(),s.reverse(),o>-1&&c>0){let e=function(e,t){return(n,i,r)=>{let s=0,o=n.length-1,l,h;if(o>=0&&(l=n[o])instanceof m){if(!o&&l.type==e&&l.length==r)return l;(h=l.prop(a.lookAhead))&&(s=i[o]+l.length+h)}return p(e,n,i,r,s,t)}}(C,P);A=I(C,n,s,0,n.length,0,O-b,e,e)}else A=p(C,n,s,O-b,S-O,P)}s.push(A),g.push(R)}(e.start||0,e.bufferStart||0,v,y,-1,0);let x=null!==(t=e.length)&&void 0!==t?t:v.length?y[0]+v[0].length:0;return new m(c[e.topID],v.reverse(),y.reverse(),x)}(e)}}m.empty=new m(u.none,[],[],0);class g{constructor(e,t){this.buffer=e,this.index=t}get id(){return this.buffer[this.index-4]}get start(){return this.buffer[this.index-3]}get end(){return this.buffer[this.index-2]}get size(){return this.buffer[this.index-1]}get pos(){return this.index}next(){this.index-=4}fork(){return new g(this.buffer,this.index)}}class k{constructor(e,t,n){this.buffer=e,this.length=t,this.set=n}get type(){return u.none}toString(){let e=[];for(let t=0;t<this.buffer.length;)e.push(this.childString(t)),t=this.buffer[t+3];return e.join(",")}childString(e){let t=this.buffer[e],n=this.buffer[e+3],i=this.set.types[t],r=i.name;if(/\W/.test(r)&&!i.isError&&(r=JSON.stringify(r)),n==(e+=4))return r;let s=[];for(;e<n;)s.push(this.childString(e)),e=this.buffer[e+3];return r+"("+s.join(",")+")"}findChild(e,t,n,i,r){let{buffer:s}=this,o=-1;for(let l=e;l!=t&&(!v(r,i,s[l+1],s[l+2])||(o=l,!(n>0)));l=s[l+3]);return o}slice(e,t,n){let i=this.buffer,r=new Uint16Array(t-e),s=0;for(let o=e,l=0;o<t;){r[l++]=i[o++],r[l++]=i[o++]-n;let t=r[l++]=i[o++]-n;r[l++]=i[o++]-e,s=Math.max(s,t)}return new k(r,s,this.set)}}function v(e,t,n,i){switch(e){case -2:return n<t;case -1:return i>=t&&n<t;case 0:return n<t&&i>t;case 1:return n<=t&&i>t;case 2:return i>t;case 4:return!0}}function y(e,t,n,r){for(var s;e.from==e.to||(n<1?e.from>=t:e.from>t)||(n>-1?e.to<=t:e.to<t);){let t=!r&&e instanceof b&&e.index<0?null:e.parent;if(!t)return e;e=t}let o=r?0:i.IgnoreOverlays;if(r)for(let i=e,r=i.parent;r;r=(i=r).parent)i instanceof b&&i.index<0&&(null===(s=r.enter(t,n,o))||void 0===s?void 0:s.from)!=i.from&&(e=r);for(;;){let i=e.enter(t,n,o);if(!i)return e;e=i}}class x{cursor(e=0){return new T(this,e)}getChild(e,t=null,n=null){let i=O(this,e,t,n);return i.length?i[0]:null}getChildren(e,t=null,n=null){return O(this,e,t,n)}resolve(e,t=0){return y(this,e,t,!1)}resolveInner(e,t=0){return y(this,e,t,!0)}matchContext(e){return w(this.parent,e)}enterUnfinishedNodesBefore(e){let t=this.childBefore(e),n=this;for(;t;){let e=t.lastChild;if(!e||e.to!=t.to)break;e.type.isError&&e.from==e.to?(n=t,t=e.prevSibling):t=e}return n}get node(){return this}get next(){return this.parent}}class b extends x{constructor(e,t,n,i){super(),this._tree=e,this.from=t,this.index=n,this._parent=i}get type(){return this._tree.type}get name(){return this._tree.type.name}get to(){return this.from+this._tree.length}nextChild(e,t,n,r,s=0){var o;for(let l=this;;){for(let{children:a,positions:c}=l._tree,u=t>0?a.length:-1;e!=u;e+=t){let u=a[e],f=c[e]+l.from;if(s&i.EnterBracketed&&u instanceof m&&(null===(o=h.get(u))||void 0===o?void 0:o.overlay)===null&&(f>=n||f+u.length<=n)||v(r,n,f,f+u.length)){if(u instanceof k){if(s&i.ExcludeBuffers)continue;let o=u.findChild(0,u.buffer.length,t,n-f,r);if(o>-1)return new P(new S(l,u,e,f),null,o)}else if(s&i.IncludeAnonymous||!u.type.isAnonymous||R(u)){let o;if(!(s&i.IgnoreMounts)&&(o=h.get(u))&&!o.overlay)return new b(o.tree,f,e,l);let a=new b(u,f,e,l);return s&i.IncludeAnonymous||!a.type.isAnonymous?a:a.nextChild(t<0?u.children.length-1:0,t,n,r,s)}}}if(s&i.IncludeAnonymous||!l.type.isAnonymous||(e=l.index>=0?l.index+t:t<0?-1:l._parent._tree.children.length,!(l=l._parent)))return null}}get firstChild(){return this.nextChild(0,1,0,4)}get lastChild(){return this.nextChild(this._tree.children.length-1,-1,0,4)}childAfter(e){return this.nextChild(0,1,e,2)}childBefore(e){return this.nextChild(this._tree.children.length-1,-1,e,-2)}prop(e){return this._tree.prop(e)}enter(e,t,n=0){let r;if(!(n&i.IgnoreOverlays)&&(r=h.get(this._tree))&&r.overlay){let s=e-this.from,o=n&i.EnterBracketed&&r.bracketed;for(let{from:e,to:n}of r.overlay)if((t>0||o?e<=s:e<s)&&(t<0||o?n>=s:n>s))return new b(r.tree,r.overlay[0].from+this.from,-1,this)}return this.nextChild(0,1,e,t,n)}nextSignificantParent(){let e=this;for(;e.type.isAnonymous&&e._parent;)e=e._parent;return e}get parent(){return this._parent?this._parent.nextSignificantParent():null}get nextSibling(){return this._parent&&this.index>=0?this._parent.nextChild(this.index+1,1,0,4):null}get prevSibling(){return this._parent&&this.index>=0?this._parent.nextChild(this.index-1,-1,0,4):null}get tree(){return this._tree}toTree(){return this._tree}toString(){return this._tree.toString()}}function O(e,t,n,i){let r=e.cursor(),s=[];if(!r.firstChild())return s;if(null!=n){for(let e=!1;!e;)if(e=r.type.is(n),!r.nextSibling())return s}for(;;){if(null!=i&&r.type.is(i))return s;if(r.type.is(t)&&s.push(r.node),!r.nextSibling())return null==i?s:[]}}function w(e,t,n=t.length-1){for(let i=e;n>=0;i=i.parent){if(!i)return!1;if(!i.type.isAnonymous){if(t[n]&&t[n]!=i.name)return!1;n--}}return!0}class S{constructor(e,t,n,i){this.parent=e,this.buffer=t,this.index=n,this.start=i}}class P extends x{get name(){return this.type.name}get from(){return this.context.start+this.context.buffer.buffer[this.index+1]}get to(){return this.context.start+this.context.buffer.buffer[this.index+2]}constructor(e,t,n){super(),this.context=e,this._parent=t,this.index=n,this.type=e.buffer.set.types[e.buffer.buffer[n]]}child(e,t,n){let{buffer:i}=this.context,r=i.findChild(this.index+4,i.buffer[this.index+3],e,t-this.context.start,n);return r<0?null:new P(this.context,this,r)}get firstChild(){return this.child(1,0,4)}get lastChild(){return this.child(-1,0,4)}childAfter(e){return this.child(1,e,2)}childBefore(e){return this.child(-1,e,-2)}prop(e){return this.type.prop(e)}enter(e,t,n=0){if(n&i.ExcludeBuffers)return null;let{buffer:r}=this.context,s=r.findChild(this.index+4,r.buffer[this.index+3],t>0?1:-1,e-this.context.start,t);return s<0?null:new P(this.context,this,s)}get parent(){return this._parent||this.context.parent.nextSignificantParent()}externalSibling(e){return this._parent?null:this.context.parent.nextChild(this.context.index+e,e,0,4)}get nextSibling(){let{buffer:e}=this.context,t=e.buffer[this.index+3];return t<(this._parent?e.buffer[this._parent.index+3]:e.buffer.length)?new P(this.context,this._parent,t):this.externalSibling(1)}get prevSibling(){let{buffer:e}=this.context,t=this._parent?this._parent.index+4:0;return this.index==t?this.externalSibling(-1):new P(this.context,this._parent,e.findChild(t,this.index,-1,0,4))}get tree(){return null}toTree(){let e=[],t=[],{buffer:n}=this.context,i=this.index+4,r=n.buffer[this.index+3];if(r>i){let s=n.buffer[this.index+1];e.push(n.slice(i,r,s)),t.push(0)}return new m(this.type,e,t,this.to-this.from)}toString(){return this.context.buffer.childString(this.index)}}function C(e){if(!e.length)return null;let t=0,n=e[0];for(let i=1;i<e.length;i++){let r=e[i];(r.from>n.from||r.to<n.to)&&(n=r,t=i)}let i=n instanceof b&&n.index<0?null:n.parent,r=e.slice();return i?r[t]=i:r.splice(t,1),new A(r,n)}class A{constructor(e,t){this.heads=e,this.node=t}get next(){return C(this.heads)}}class T{get name(){return this.type.name}constructor(e,t=0){if(this.buffer=null,this.stack=[],this.index=0,this.bufferNode=null,this.mode=t&~i.EnterBracketed,e instanceof b)this.yieldNode(e);else{this._tree=e.context.parent,this.buffer=e.context;for(let t=e._parent;t;t=t._parent)this.stack.unshift(t.index);this.bufferNode=e,this.yieldBuf(e.index)}}yieldNode(e){return!!e&&(this._tree=e,this.type=e.type,this.from=e.from,this.to=e.to,!0)}yieldBuf(e,t){this.index=e;let{start:n,buffer:i}=this.buffer;return this.type=t||i.set.types[i.buffer[e]],this.from=n+i.buffer[e+1],this.to=n+i.buffer[e+2],!0}yield(e){return!!e&&(e instanceof b?(this.buffer=null,this.yieldNode(e)):(this.buffer=e.context,this.yieldBuf(e.index,e.type)))}toString(){return this.buffer?this.buffer.buffer.childString(this.index):this._tree.toString()}enterChild(e,t,n){if(!this.buffer)return this.yield(this._tree.nextChild(e<0?this._tree._tree.children.length-1:0,e,t,n,this.mode));let{buffer:i}=this.buffer,r=i.findChild(this.index+4,i.buffer[this.index+3],e,t-this.buffer.start,n);return!(r<0)&&(this.stack.push(this.index),this.yieldBuf(r))}firstChild(){return this.enterChild(1,0,4)}lastChild(){return this.enterChild(-1,0,4)}childAfter(e){return this.enterChild(1,e,2)}childBefore(e){return this.enterChild(-1,e,-2)}enter(e,t,n=this.mode){return this.buffer?!(n&i.ExcludeBuffers)&&this.enterChild(1,e,t):this.yield(this._tree.enter(e,t,n))}parent(){if(!this.buffer)return this.yieldNode(this.mode&i.IncludeAnonymous?this._tree._parent:this._tree.parent);if(this.stack.length)return this.yieldBuf(this.stack.pop());let e=this.mode&i.IncludeAnonymous?this.buffer.parent:this.buffer.parent.nextSignificantParent();return this.buffer=null,this.yieldNode(e)}sibling(e){if(!this.buffer)return!!this._tree._parent&&this.yield(this._tree.index<0?null:this._tree._parent.nextChild(this._tree.index+e,e,0,4,this.mode));let{buffer:t}=this.buffer,n=this.stack.length-1;if(e<0){let e=n<0?0:this.stack[n]+4;if(this.index!=e)return this.yieldBuf(t.findChild(e,this.index,-1,0,4))}else{let e=t.buffer[this.index+3];if(e<(n<0?t.buffer.length:t.buffer[this.stack[n]+3]))return this.yieldBuf(e)}return n<0&&this.yield(this.buffer.parent.nextChild(this.buffer.index+e,e,0,4,this.mode))}nextSibling(){return this.sibling(1)}prevSibling(){return this.sibling(-1)}atLastNode(e){let t,n,{buffer:r}=this;if(r){if(e>0){if(this.index<r.buffer.buffer.length)return!1}else for(let e=0;e<this.index;e++)if(r.buffer.buffer[e+3]<this.index)return!1;({index:t,parent:n}=r)}else({index:t,_parent:n}=this._tree);for(;n;{index:t,_parent:n}=n)if(t>-1)for(let r=t+e,s=e<0?-1:n._tree.children.length;r!=s;r+=e){let e=n._tree.children[r];if(this.mode&i.IncludeAnonymous||e instanceof k||!e.type.isAnonymous||R(e))return!1}return!0}move(e,t){if(t&&this.enterChild(e,0,4))return!0;for(;;){if(this.sibling(e))return!0;if(this.atLastNode(e)||!this.parent())return!1}}next(e=!0){return this.move(1,e)}prev(e=!0){return this.move(-1,e)}moveTo(e,t=0){for(;(this.from==this.to||(t<1?this.from>=e:this.from>e)||(t>-1?this.to<=e:this.to<e))&&this.parent(););for(;this.enterChild(1,e,t););return this}get node(){if(!this.buffer)return this._tree;let e=this.bufferNode,t=null,n=0;if(e&&e.context==this.buffer)e:for(let i=this.index,r=this.stack.length;r>=0;){for(let s=e;s;s=s._parent)if(s.index==i){if(i==this.index)return s;t=s,n=r+1;break e}i=this.stack[--r]}for(let e=n;e<this.stack.length;e++)t=new P(this.buffer,t,this.stack[e]);return this.bufferNode=new P(this.buffer,t,this.index)}get tree(){return this.buffer?null:this._tree._tree}iterate(e,t){for(let n=0;;){let i=!1;if(this.type.isAnonymous||!1!==e(this)){if(this.firstChild()){n++;continue}this.type.isAnonymous||(i=!0)}for(;;){if(i&&t&&t(this),i=this.type.isAnonymous,!n)return;if(this.nextSibling())break;this.parent(),n--,i=!0}}}matchContext(e){if(!this.buffer)return w(this.node.parent,e);let{buffer:t}=this.buffer,{types:n}=t.set;for(let i=e.length-1,r=this.stack.length-1;i>=0;r--){if(r<0)return w(this._tree,e,i);let s=n[t.buffer[this.stack[r]]];if(!s.isAnonymous){if(e[i]&&e[i]!=s.name)return!1;i--}}return!0}}function R(e){return e.children.some(e=>e instanceof k||!e.type.isAnonymous||R(e))}let D=new WeakMap;function M(e,t){if(!e.isAnonymous||t instanceof k||t.type!=e)return 1;let n=D.get(t);if(null==n){for(let i of(n=1,t.children)){if(i.type!=e||!(i instanceof m)){n=1;break}n+=M(e,i)}D.set(t,n)}return n}function I(e,t,n,i,r,s,o,l,a){let h=0;for(let n=i;n<r;n++)h+=M(e,t[n]);let c=Math.ceil(1.5*h/8),u=[],f=[];return!function t(n,i,r,o,l){for(let h=r;h<o;){let r=h,d=i[h],p=M(e,n[h]);for(h++;h<o;h++){let t=M(e,n[h]);if(p+t>=c)break;p+=t}if(h==r+1){if(p>c){let e=n[r];t(e.children,e.positions,0,e.children.length,i[r]+l);continue}u.push(n[r])}else{let t=i[h-1]+n[h-1].length-d;u.push(I(e,n,i,r,h,d,t,null,a))}f.push(d+l-s)}}(t,n,i,r,0),(l||a)(u,f,o)}class Q{constructor(e,t,n,i,r=!1,s=!1){this.from=e,this.to=t,this.tree=n,this.offset=i,this.open=(r?1:0)|(s?2:0)}get openStart(){return(1&this.open)>0}get openEnd(){return(2&this.open)>0}static addTree(e,t=[],n=!1){let i=[new Q(0,e.length,e,0,!1,n)];for(let n of t)n.to>e.length&&i.push(n);return i}static applyChanges(e,t,n=128){if(!t.length)return e;let i=[],r=1,s=e.length?e[0]:null;for(let o=0,l=0,a=0;;o++){let h=o<t.length?t[o]:null,c=h?h.fromA:1e9;if(c-l>=n)for(;s&&s.from<c;){let t=s;if(l>=t.from||c<=t.to||a){let e=Math.max(t.from,l)-a,n=Math.min(t.to,c)-a;t=e>=n?null:new Q(e,n,t.tree,t.offset+a,o>0,!!h)}if(t&&i.push(t),s.to>c)break;s=r<e.length?e[r++]:null}if(!h)break;l=h.toA,a=h.toA-h.toB}return i}}class N{startParse(e,t,n){return"string"==typeof e&&(e=new E(e)),n=n?n.length?n.map(e=>new l(e.from,e.to)):[new l(0,0)]:[new l(0,e.length)],this.createParse(e,t||[],n)}parse(e,t,n){let i=this.startParse(e,t,n);for(;;){let e=i.advance();if(e)return e}}}class E{constructor(e){this.string=e}get length(){return this.string.length}chunk(e){return this.string.slice(e)}get lineChunks(){return!1}read(e,t){return this.string.slice(e,t)}}new a({perNode:!0})},35524:function(e,t,n){n.d(t,{Gv:function(){return a},QR:function(){return u},bW:function(){return f},pJ:function(){return R}});var i=n(41113);let r=0;class s{constructor(e,t,n,i){this.name=e,this.set=t,this.base=n,this.modified=i,this.id=r++}toString(){let{name:e}=this;for(let t of this.modified)t.name&&(e=`${t.name}(${e})`);return e}static define(e,t){if(e instanceof s&&(t=e),null==t?void 0:t.base)throw Error("Can not derive from a modified tag");let n=new s("string"==typeof e?e:"?",[],null,[]);if(n.set.push(n),t)for(let e of t.set)n.set.push(e);return n}static defineModifier(e){let t=new l(e);return e=>e.modified.indexOf(t)>-1?e:l.get(e.base||e,e.modified.concat(t).sort((e,t)=>e.id-t.id))}}let o=0;class l{constructor(e){this.name=e,this.instances=[],this.id=o++}static get(e,t){if(!t.length)return e;let n=t[0].instances.find(n=>{var i;return n.base==e&&(i=n.modified,t.length==i.length&&t.every((e,t)=>e==i[t]))});if(n)return n;let i=[],r=new s(e.name,i,e,t);for(let e of t)e.instances.push(r);let o=function(e){let t=[[]];for(let n=0;n<e.length;n++)for(let i=0,r=t.length;i<r;i++)t.push(t[i].concat(e[n]));return t.sort((e,t)=>t.length-e.length)}(t);for(let t of e.set)if(!t.modified.length)for(let e of o)i.push(l.get(t,e));return r}}function a(e){let t=Object.create(null);for(let n in e){let i=e[n];for(let e of(Array.isArray(i)||(i=[i]),n.split(" ")))if(e){let n=[],r=2,s=e;for(let t=0;;){if("..."==s&&t>0&&t+3==e.length){r=1;break}let i=/^"(?:[^"\\]|\\.)*?"|[^\/!]+/.exec(s);if(!i)throw RangeError("Invalid path: "+e);if(n.push("*"==i[0]?"":'"'==i[0][0]?JSON.parse(i[0]):i[0]),(t+=i[0].length)==e.length)break;let o=e[t++];if(t==e.length&&"!"==o){r=0;break}if("/"!=o)throw RangeError("Invalid path: "+e);s=e.slice(t)}let o=n.length-1,l=n[o];if(!l)throw RangeError("Invalid path: "+e);let a=new c(i,r,o>0?n.slice(0,o):null);t[l]=a.sort(t[l])}}return h.add(t)}let h=new i.md({combine(e,t){let n,i,r;for(;e||t;){if(!e||t&&e.depth>=t.depth?(r=t,t=t.next):(r=e,e=e.next),n&&n.mode==r.mode&&!r.context&&!n.context)continue;let s=new c(r.tags,r.mode,r.context);n?n.next=s:i=s,n=s}return i}});class c{constructor(e,t,n,i){this.tags=e,this.mode=t,this.context=n,this.next=i}get opaque(){return 0==this.mode}get inherit(){return 1==this.mode}sort(e){return!e||e.depth<this.depth?(this.next=e,this):(e.next=this.sort(e.next),e)}get depth(){return this.context?this.context.length:0}}function u(e,t){let n=Object.create(null);for(let t of e)if(Array.isArray(t.tag))for(let e of t.tag)n[e.id]=t.class;else n[t.tag.id]=t.class;let{scope:i,all:r=null}=t||{};return{style:e=>{let t=r;for(let i of e)for(let e of i.set){let i=n[e.id];if(i){t=t?t+" "+i:i;break}}return t},scope:i}}function f(e,t,n,i=0,r=e.length){let s=new d(i,Array.isArray(t)?t:[t],n);s.highlightRange(e.cursor(),i,r,"",s.highlighters),s.flush(r)}c.empty=new c([],2,null);class d{constructor(e,t,n){this.at=e,this.highlighters=t,this.span=n,this.class=""}startSpan(e,t){t!=this.class&&(this.flush(e),e>this.at&&(this.at=e),this.class=t)}flush(e){e>this.at&&this.class&&this.span(this.at,e,this.class)}highlightRange(e,t,n,r,s){let{type:o,from:l,to:a}=e;if(l>=n||a<=t)return;o.isTop&&(s=this.highlighters.filter(e=>!e.scope||e.scope(o)));let u=r,f=function(e){let t=e.type.prop(h);for(;t&&t.context&&!e.matchContext(t.context);)t=t.next;return t||null}(e)||c.empty,d=function(e,t){let n=null;for(let i of e){let e=i.style(t);e&&(n=n?n+" "+e:e)}return n}(s,f.tags);if(d&&(u&&(u+=" "),u+=d,1==f.mode&&(r+=(r?" ":"")+d)),this.startSpan(Math.max(t,l),u),f.opaque)return;let p=e.tree&&e.tree.prop(i.md.mounted);if(p&&p.overlay){let i=e.node.enter(p.overlay[0].from+l,1),o=this.highlighters.filter(e=>!e.scope||e.scope(p.tree.type)),h=e.firstChild();for(let c=0,f=l;;c++){let d=c<p.overlay.length?p.overlay[c]:null,m=d?d.from+l:a,g=Math.max(t,f),k=Math.min(n,m);if(g<k&&h)for(;e.from<k&&(this.highlightRange(e,g,k,r,s),this.startSpan(Math.min(k,e.to),u),!(e.to>=m)&&e.nextSibling()););if(!d||m>n)break;(f=d.to+l)>t&&(this.highlightRange(i.cursor(),Math.max(t,d.from+l),Math.min(n,f),"",o),this.startSpan(Math.min(n,f),u))}h&&e.parent()}else if(e.firstChild()){p&&(r="");do{if(e.to<=t)continue;if(e.from>=n)break;this.highlightRange(e,t,n,r,s),this.startSpan(Math.min(n,e.to),u)}while(e.nextSibling());e.parent()}}}let p=s.define,m=p(),g=p(),k=p(g),v=p(g),y=p(),x=p(y),b=p(y),O=p(),w=p(O),S=p(),P=p(),C=p(),A=p(C),T=p(),R={comment:m,lineComment:p(m),blockComment:p(m),docComment:p(m),name:g,variableName:p(g),typeName:k,tagName:p(k),propertyName:v,attributeName:p(v),className:p(g),labelName:p(g),namespace:p(g),macroName:p(g),literal:y,string:x,docString:p(x),character:p(x),attributeValue:p(x),number:b,integer:p(b),float:p(b),bool:p(y),regexp:p(y),escape:p(y),color:p(y),url:p(y),keyword:S,self:p(S),null:p(S),atom:p(S),unit:p(S),modifier:p(S),operatorKeyword:p(S),controlKeyword:p(S),definitionKeyword:p(S),moduleKeyword:p(S),operator:P,derefOperator:p(P),arithmeticOperator:p(P),logicOperator:p(P),bitwiseOperator:p(P),compareOperator:p(P),updateOperator:p(P),definitionOperator:p(P),typeOperator:p(P),controlOperator:p(P),punctuation:C,separator:p(C),bracket:A,angleBracket:p(A),squareBracket:p(A),paren:p(A),brace:p(A),content:O,heading:w,heading1:p(w),heading2:p(w),heading3:p(w),heading4:p(w),heading5:p(w),heading6:p(w),contentSeparator:p(O),list:p(O),quote:p(O),emphasis:p(O),strong:p(O),link:p(O),monospace:p(O),strikethrough:p(O),inserted:p(),deleted:p(),changed:p(),invalid:p(),meta:T,documentMeta:p(T),annotation:p(T),processingInstruction:p(T),definition:s.defineModifier("definition"),constant:s.defineModifier("constant"),function:s.defineModifier("function"),standard:s.defineModifier("standard"),local:s.defineModifier("local"),special:s.defineModifier("special")};for(let e in R){let t=R[e];t instanceof s&&(t.name=e)}u([{tag:R.link,class:"tok-link"},{tag:R.heading,class:"tok-heading"},{tag:R.emphasis,class:"tok-emphasis"},{tag:R.strong,class:"tok-strong"},{tag:R.keyword,class:"tok-keyword"},{tag:R.atom,class:"tok-atom"},{tag:R.bool,class:"tok-bool"},{tag:R.url,class:"tok-url"},{tag:R.labelName,class:"tok-labelName"},{tag:R.inserted,class:"tok-inserted"},{tag:R.deleted,class:"tok-deleted"},{tag:R.literal,class:"tok-literal"},{tag:R.string,class:"tok-string"},{tag:R.number,class:"tok-number"},{tag:[R.regexp,R.escape,R.special(R.string)],class:"tok-string2"},{tag:R.variableName,class:"tok-variableName"},{tag:R.local(R.variableName),class:"tok-variableName tok-local"},{tag:R.definition(R.variableName),class:"tok-variableName tok-definition"},{tag:R.special(R.variableName),class:"tok-variableName2"},{tag:R.definition(R.propertyName),class:"tok-propertyName tok-definition"},{tag:R.typeName,class:"tok-typeName"},{tag:R.namespace,class:"tok-namespace"},{tag:R.className,class:"tok-className"},{tag:R.macroName,class:"tok-macroName"},{tag:R.propertyName,class:"tok-propertyName"},{tag:R.operator,class:"tok-operator"},{tag:R.comment,class:"tok-comment"},{tag:R.meta,class:"tok-meta"},{tag:R.invalid,class:"tok-invalid"},{tag:R.punctuation,class:"tok-punctuation"}])},6327:function(e,t,n){n.d(t,{x8:function(){return el},VY:function(){return er},dk:function(){return eo},aV:function(){return ei},h_:function(){return en},fC:function(){return ee},Dx:function(){return es},xz:function(){return et}});var i=n(67294),r=n(36206),s=n(28771),o=n(25360),l=n(91276),a=n(77342),h=n(46063),c=n(95420),u=n(42651),f=n(9981),d=e=>{let t,n;let{present:r,children:o}=e,l=function(e){var t,n;let[r,s]=i.useState(),o=i.useRef(null),l=i.useRef(e),a=i.useRef("none"),[h,c]=(t=e?"mounted":"unmounted",n={mounted:{UNMOUNT:"unmounted",ANIMATION_OUT:"unmountSuspended"},unmountSuspended:{MOUNT:"mounted",ANIMATION_END:"unmounted"},unmounted:{MOUNT:"mounted"}},i.useReducer((e,t)=>n[e][t]??e,t));return i.useEffect(()=>{let e=p(o.current);a.current="mounted"===h?e:"none"},[h]),(0,f.b)(()=>{let t=o.current,n=l.current;if(n!==e){let i=a.current,r=p(t);e?c("MOUNT"):"none"===r||t?.display==="none"?c("UNMOUNT"):n&&i!==r?c("ANIMATION_OUT"):c("UNMOUNT"),l.current=e}},[e,c]),(0,f.b)(()=>{if(r){let e;let t=r.ownerDocument.defaultView??window,n=n=>{let i=p(o.current).includes(n.animationName);if(n.target===r&&i&&(c("ANIMATION_END"),!l.current)){let n=r.style.animationFillMode;r.style.animationFillMode="forwards",e=t.setTimeout(()=>{"forwards"===r.style.animationFillMode&&(r.style.animationFillMode=n)})}},i=e=>{e.target===r&&(a.current=p(o.current))};return r.addEventListener("animationstart",i),r.addEventListener("animationcancel",n),r.addEventListener("animationend",n),()=>{t.clearTimeout(e),r.removeEventListener("animationstart",i),r.removeEventListener("animationcancel",n),r.removeEventListener("animationend",n)}}c("ANIMATION_END")},[r,c]),{isPresent:["mounted","unmountSuspended"].includes(h),ref:i.useCallback(e=>{o.current=e?getComputedStyle(e):null,s(e)},[])}}(r),a="function"==typeof o?o({present:l.isPresent}):i.Children.only(o),h=(0,s.e)(l.ref,(t=Object.getOwnPropertyDescriptor(a.props,"ref")?.get)&&"isReactWarning"in t&&t.isReactWarning?a.ref:(t=Object.getOwnPropertyDescriptor(a,"ref")?.get)&&"isReactWarning"in t&&t.isReactWarning?a.props.ref:a.props.ref||a.ref);return"function"==typeof o||l.isPresent?i.cloneElement(a,{ref:h}):null};function p(e){return e?.animationName||"none"}d.displayName="Presence";var m=n(75320),g=n(27552),k=n(6223),v=n(23541),y=n(88426),x=n(85893),b="Dialog",[O,w]=(0,o.b)(b),[S,P]=O(b),C=e=>{let{__scopeDialog:t,children:n,open:r,defaultOpen:s,onOpenChange:o,modal:h=!0}=e,c=i.useRef(null),u=i.useRef(null),[f,d]=(0,a.T)({prop:r,defaultProp:s??!1,onChange:o,caller:b});return(0,x.jsx)(S,{scope:t,triggerRef:c,contentRef:u,contentId:(0,l.M)(),titleId:(0,l.M)(),descriptionId:(0,l.M)(),open:f,onOpenChange:d,onOpenToggle:i.useCallback(()=>d(e=>!e),[d]),modal:h,children:n})};C.displayName=b;var A="DialogTrigger",T=i.forwardRef((e,t)=>{let{__scopeDialog:n,...i}=e,o=P(A,n),l=(0,s.e)(t,o.triggerRef);return(0,x.jsx)(m.WV.button,{type:"button","aria-haspopup":"dialog","aria-expanded":o.open,"aria-controls":o.contentId,"data-state":V(o.open),...i,ref:l,onClick:(0,r.M)(e.onClick,o.onOpenToggle)})});T.displayName=A;var R="DialogPortal",[D,M]=O(R,{forceMount:void 0}),I=e=>{let{__scopeDialog:t,forceMount:n,children:r,container:s}=e,o=P(R,t);return(0,x.jsx)(D,{scope:t,forceMount:n,children:i.Children.map(r,e=>(0,x.jsx)(d,{present:n||o.open,children:(0,x.jsx)(u.h,{asChild:!0,container:s,children:e})}))})};I.displayName=R;var Q="DialogOverlay",N=i.forwardRef((e,t)=>{let n=M(Q,e.__scopeDialog),{forceMount:i=n.forceMount,...r}=e,s=P(Q,e.__scopeDialog);return s.modal?(0,x.jsx)(d,{present:i||s.open,children:(0,x.jsx)(B,{...r,ref:t})}):null});N.displayName=Q;var E=(0,y.Z8)("DialogOverlay.RemoveScroll"),B=i.forwardRef((e,t)=>{let{__scopeDialog:n,...i}=e,r=P(Q,n);return(0,x.jsx)(k.Z,{as:E,allowPinchZoom:!0,shards:[r.contentRef],children:(0,x.jsx)(m.WV.div,{"data-state":V(r.open),...i,ref:t,style:{pointerEvents:"auto",...i.style}})})}),L="DialogContent",$=i.forwardRef((e,t)=>{let n=M(L,e.__scopeDialog),{forceMount:i=n.forceMount,...r}=e,s=P(L,e.__scopeDialog);return(0,x.jsx)(d,{present:i||s.open,children:s.modal?(0,x.jsx)(j,{...r,ref:t}):(0,x.jsx)(z,{...r,ref:t})})});$.displayName=L;var j=i.forwardRef((e,t)=>{let n=P(L,e.__scopeDialog),o=i.useRef(null),l=(0,s.e)(t,n.contentRef,o);return i.useEffect(()=>{let e=o.current;if(e)return(0,v.Ry)(e)},[]),(0,x.jsx)(X,{...e,ref:l,trapFocus:n.open,disableOutsidePointerEvents:!0,onCloseAutoFocus:(0,r.M)(e.onCloseAutoFocus,e=>{e.preventDefault(),n.triggerRef.current?.focus()}),onPointerDownOutside:(0,r.M)(e.onPointerDownOutside,e=>{let t=e.detail.originalEvent,n=0===t.button&&!0===t.ctrlKey;(2===t.button||n)&&e.preventDefault()}),onFocusOutside:(0,r.M)(e.onFocusOutside,e=>e.preventDefault())})}),z=i.forwardRef((e,t)=>{let n=P(L,e.__scopeDialog),r=i.useRef(!1),s=i.useRef(!1);return(0,x.jsx)(X,{...e,ref:t,trapFocus:!1,disableOutsidePointerEvents:!1,onCloseAutoFocus:t=>{e.onCloseAutoFocus?.(t),t.defaultPrevented||(r.current||n.triggerRef.current?.focus(),t.preventDefault()),r.current=!1,s.current=!1},onInteractOutside:t=>{e.onInteractOutside?.(t),t.defaultPrevented||(r.current=!0,"pointerdown"!==t.detail.originalEvent.type||(s.current=!0));let i=t.target;n.triggerRef.current?.contains(i)&&t.preventDefault(),"focusin"===t.detail.originalEvent.type&&s.current&&t.preventDefault()}})}),X=i.forwardRef((e,t)=>{let{__scopeDialog:n,trapFocus:r,onOpenAutoFocus:o,onCloseAutoFocus:l,...a}=e,u=P(L,n),f=i.useRef(null),d=(0,s.e)(t,f);return(0,g.EW)(),(0,x.jsxs)(x.Fragment,{children:[(0,x.jsx)(c.M,{asChild:!0,loop:!0,trapped:r,onMountAutoFocus:o,onUnmountAutoFocus:l,children:(0,x.jsx)(h.XB,{role:"dialog",id:u.contentId,"aria-describedby":u.descriptionId,"aria-labelledby":u.titleId,"data-state":V(u.open),...a,ref:d,onDismiss:()=>u.onOpenChange(!1)})}),(0,x.jsxs)(x.Fragment,{children:[(0,x.jsx)(H,{titleId:u.titleId}),(0,x.jsx)(K,{contentRef:f,descriptionId:u.descriptionId})]})]})}),q="DialogTitle",J=i.forwardRef((e,t)=>{let{__scopeDialog:n,...i}=e,r=P(q,n);return(0,x.jsx)(m.WV.h2,{id:r.titleId,...i,ref:t})});J.displayName=q;var W="DialogDescription",_=i.forwardRef((e,t)=>{let{__scopeDialog:n,...i}=e,r=P(W,n);return(0,x.jsx)(m.WV.p,{id:r.descriptionId,...i,ref:t})});_.displayName=W;var F="DialogClose",U=i.forwardRef((e,t)=>{let{__scopeDialog:n,...i}=e,s=P(F,n);return(0,x.jsx)(m.WV.button,{type:"button",...i,ref:t,onClick:(0,r.M)(e.onClick,()=>s.onOpenChange(!1))})});function V(e){return e?"open":"closed"}U.displayName=F;var Z="DialogTitleWarning",[G,Y]=(0,o.k)(Z,{contentName:L,titleName:q,docsSlug:"dialog"}),H=({titleId:e})=>{let t=Y(Z),n=`\`${t.contentName}\` requires a \`${t.titleName}\` for the component to be accessible for screen reader users.

If you want to hide the \`${t.titleName}\`, you can wrap it with our VisuallyHidden component.

For more information, see https://radix-ui.com/primitives/docs/components/${t.docsSlug}`;return i.useEffect(()=>{e&&!document.getElementById(e)&&console.error(n)},[n,e]),null},K=({contentRef:e,descriptionId:t})=>{let n=Y("DialogDescriptionWarning"),r=`Warning: Missing \`Description\` or \`aria-describedby={undefined}\` for {${n.contentName}}.`;return i.useEffect(()=>{let n=e.current?.getAttribute("aria-describedby");t&&n&&!document.getElementById(t)&&console.warn(r)},[r,e,t]),null},ee=C,et=T,en=I,ei=N,er=$,es=J,eo=_,el=U},49102:function(e,t,n){n.d(t,{f:function(){return l}});var i=n(67294),r=n(75320),s=n(85893),o=i.forwardRef((e,t)=>(0,s.jsx)(r.WV.label,{...e,ref:t,onMouseDown:t=>{t.target.closest("button, input, select, textarea")||(e.onMouseDown?.(t),!t.defaultPrevented&&t.detail>1&&t.preventDefault())}}));o.displayName="Label";var l=o},99094:function(e,t,n){n.d(t,{ZP:function(){return is}});var i=n(87462),r=n(63366),s=n(67294),o=n(37205),l=n(47421),a=n(59119),h=n(41113);function c(e,t){return({state:n,dispatch:i})=>{if(n.readOnly)return!1;let r=e(t,n);return!!r&&(i(n.update(r)),!0)}}let u=c(function(e,t,n=t.selection.ranges){let i=[],r=-1;for(let{from:e,to:s}of n){let n=i.length,o=1e9,l=p(t,e).line;if(l){for(let n=e;n<=s;){let a=t.doc.lineAt(n);if(a.from>r&&(e==s||s>a.from)){r=a.from;let e=/^\s*/.exec(a.text)[0].length,t=e==a.length,n=a.text.slice(e,e+l.length)==l?e:-1;e<a.text.length&&e<o&&(o=e),i.push({line:a,comment:n,token:l,indent:e,empty:t,single:!1})}n=a.to+1}if(o<1e9)for(let e=n;e<i.length;e++)i[e].indent<i[e].line.text.length&&(i[e].indent=o);i.length==n+1&&(i[n].single=!0)}}if(2!=e&&i.some(e=>e.comment<0&&(!e.empty||e.single))){let e=[];for(let{line:t,token:n,indent:r,empty:s,single:o}of i)(o||!s)&&e.push({from:t.from+r,insert:n+" "});let n=t.changes(e);return{changes:n,selection:t.selection.map(n,1)}}if(1!=e&&i.some(e=>e.comment>=0)){let e=[];for(let{line:t,comment:n,token:r}of i)if(n>=0){let i=t.from+n,s=i+r.length;" "==t.text[s-t.from]&&s++,e.push({from:i,to:s})}return{changes:e}}return null},0),f=c(m,0),d=c((e,t)=>m(e,t,function(e){let t=[];for(let n of e.selection.ranges){let i=e.doc.lineAt(n.from),r=n.to<=i.to?i:e.doc.lineAt(n.to);r.from>i.from&&r.from==n.to&&(r=n.to==i.to+1?i:e.doc.lineAt(n.to-1));let s=t.length-1;s>=0&&t[s].to>i.from?t[s].to=r.to:t.push({from:i.from+/^\s*/.exec(i.text)[0].length,to:r.to})}return t}(t)),0);function p(e,t){let n=e.languageDataAt("commentTokens",t,1);return n.length?n[0]:{}}function m(e,t,n=t.selection.ranges){let i=n.map(e=>p(t,e.from).block);if(!i.every(e=>e))return null;let r=n.map((e,n)=>(function(e,{open:t,close:n},i,r){let s,o,l=e.sliceDoc(i-50,i),a=e.sliceDoc(r,r+50),h=/\s*$/.exec(l)[0].length,c=/^\s*/.exec(a)[0].length,u=l.length-h;if(l.slice(u-t.length,u)==t&&a.slice(c,c+n.length)==n)return{open:{pos:i-h,margin:h&&1},close:{pos:r+c,margin:c&&1}};r-i<=100?s=o=e.sliceDoc(i,r):(s=e.sliceDoc(i,i+50),o=e.sliceDoc(r-50,r));let f=/^\s*/.exec(s)[0].length,d=/\s*$/.exec(o)[0].length,p=o.length-d-n.length;return s.slice(f,f+t.length)==t&&o.slice(p,p+n.length)==n?{open:{pos:i+f+t.length,margin:/\s/.test(s.charAt(f+t.length))?1:0},close:{pos:r-d-n.length,margin:/\s/.test(o.charAt(p-1))?1:0}}:null})(t,i[n],e.from,e.to));if(2!=e&&!r.every(e=>e))return{changes:t.changes(n.map((e,t)=>r[t]?[]:[{from:e.from,insert:i[t].open+" "},{from:e.to,insert:" "+i[t].close}]))};if(1!=e&&r.some(e=>e)){let e=[];for(let t=0,n;t<r.length;t++)if(n=r[t]){let r=i[t],{open:s,close:o}=n;e.push({from:s.pos-r.open.length,to:s.pos+s.margin},{from:o.pos-o.margin,to:o.pos+r.close.length})}return{changes:e}}return null}let g=o.q6.define(),k=o.q6.define(),v=o.r$.define(),y=o.r$.define({combine:e=>(0,o.BO)(e,{minDepth:100,newGroupDelay:500,joinToEvent:(e,t)=>t},{minDepth:Math.max,newGroupDelay:Math.min,joinToEvent:(e,t)=>(n,i)=>e(n,i)||t(n,i)})}),x=o.QQ.define({create:()=>Q.empty,update(e,t){let n=t.state.facet(y),i=t.annotation(g);if(i){let r=C.fromTransaction(t,i.selection),s=i.side,o=0==s?e.undone:e.done;return o=r?A(o,o.length,n.minDepth,r):D(o,t.startState.selection),new Q(0==s?i.rest:o,0==s?o:i.rest)}let r=t.annotation(k);if(("full"==r||"before"==r)&&(e=e.isolate()),!1===t.annotation(o.YW.addToHistory))return t.changes.empty?e:e.addMapping(t.changes.desc);let s=C.fromTransaction(t),l=t.annotation(o.YW.time),a=t.annotation(o.YW.userEvent);return s?e=e.addChanges(s,l,a,n,t):t.selection&&(e=e.addSelection(t.startState.selection,l,a,n.newGroupDelay)),("full"==r||"after"==r)&&(e=e.isolate()),e},toJSON:e=>({done:e.done.map(e=>e.toJSON()),undone:e.undone.map(e=>e.toJSON())}),fromJSON:e=>new Q(e.done.map(C.fromJSON),e.undone.map(C.fromJSON))});function b(e,t){return function({state:n,dispatch:i}){if(!t&&n.readOnly)return!1;let r=n.field(x,!1);if(!r)return!1;let s=r.pop(e,n,t);return!!s&&(i(s),!0)}}let O=b(0,!1),w=b(1,!1),S=b(0,!0),P=b(1,!0);class C{constructor(e,t,n,i,r){this.changes=e,this.effects=t,this.mapped=n,this.startSelection=i,this.selectionsAfter=r}setSelAfter(e){return new C(this.changes,this.effects,this.mapped,this.startSelection,e)}toJSON(){var e,t,n;return{changes:null===(e=this.changes)||void 0===e?void 0:e.toJSON(),mapped:null===(t=this.mapped)||void 0===t?void 0:t.toJSON(),startSelection:null===(n=this.startSelection)||void 0===n?void 0:n.toJSON(),selectionsAfter:this.selectionsAfter.map(e=>e.toJSON())}}static fromJSON(e){return new C(e.changes&&o.as.fromJSON(e.changes),[],e.mapped&&o.n0.fromJSON(e.mapped),e.startSelection&&o.jT.fromJSON(e.startSelection),e.selectionsAfter.map(o.jT.fromJSON))}static fromTransaction(e,t){let n=R;for(let t of e.startState.facet(v)){let i=t(e);i.length&&(n=n.concat(i))}return!n.length&&e.changes.empty?null:new C(e.changes.invert(e.startState.doc),n,void 0,t||e.startState.selection,R)}static selection(e){return new C(void 0,R,void 0,void 0,e)}}function A(e,t,n,i){let r=e.slice(t+1>n+20?t-n-1:0,t);return r.push(i),r}function T(e,t){return e.length?t.length?e.concat(t):e:t}let R=[];function D(e,t){if(!e.length)return[C.selection([t])];{let n=e[e.length-1],i=n.selectionsAfter.slice(Math.max(0,n.selectionsAfter.length-200));return i.length&&i[i.length-1].eq(t)?e:(i.push(t),A(e,e.length-1,1e9,n.setSelAfter(i)))}}function M(e,t){if(!e.length)return e;let n=e.length,i=R;for(;n;){let r=function(e,t,n){let i=T(e.selectionsAfter.length?e.selectionsAfter.map(e=>e.map(t)):R,n);if(!e.changes)return C.selection(i);let r=e.changes.map(t),s=t.mapDesc(e.changes,!0),l=e.mapped?e.mapped.composeDesc(s):s;return new C(r,o.Py.mapEffects(e.effects,t),l,e.startSelection.map(s),i)}(e[n-1],t,i);if(r.changes&&!r.changes.empty||r.effects.length){let t=e.slice(0,n);return t[n-1]=r,t}t=r.mapped,n--,i=r.selectionsAfter}return i.length?[C.selection(i)]:R}let I=/^(input\.type|delete)($|\.)/;class Q{constructor(e,t,n=0,i){this.done=e,this.undone=t,this.prevTime=n,this.prevUserEvent=i}isolate(){return this.prevTime?new Q(this.done,this.undone):this}addChanges(e,t,n,i,r){var s,l;let a,h,c=this.done,u=c[c.length-1];return new Q(c=u&&u.changes&&!u.changes.empty&&e.changes&&(!n||I.test(n))&&(!u.selectionsAfter.length&&t-this.prevTime<i.newGroupDelay&&i.joinToEvent(r,(s=u.changes,l=e.changes,a=[],h=!1,s.iterChangedRanges((e,t)=>a.push(e,t)),l.iterChangedRanges((e,t,n,i)=>{for(let e=0;e<a.length;){let t=a[e++],r=a[e++];i>=t&&n<=r&&(h=!0)}}),h))||"input.type.compose"==n)?A(c,c.length-1,i.minDepth,new C(e.changes.compose(u.changes),T(o.Py.mapEffects(e.effects,u.changes),u.effects),u.mapped,u.startSelection,R)):A(c,c.length,i.minDepth,e),R,t,n)}addSelection(e,t,n,i){var r;let s=this.done.length?this.done[this.done.length-1].selectionsAfter:R;return s.length>0&&t-this.prevTime<i&&n==this.prevUserEvent&&n&&/^select($|\.)/.test(n)&&(r=s[s.length-1]).ranges.length==e.ranges.length&&0===r.ranges.filter((t,n)=>t.empty!=e.ranges[n].empty).length?this:new Q(D(this.done,e),this.undone,t,n)}addMapping(e){return new Q(M(this.done,e),M(this.undone,e),this.prevTime,this.prevUserEvent)}pop(e,t,n){let i=0==e?this.done:this.undone;if(0==i.length)return null;let r=i[i.length-1],s=r.selectionsAfter[0]||t.selection;if(n&&r.selectionsAfter.length){let n,o;return t.update({selection:r.selectionsAfter[r.selectionsAfter.length-1],annotations:g.of({side:e,rest:(n=i[i.length-1],(o=i.slice())[i.length-1]=n.setSelAfter(n.selectionsAfter.slice(0,n.selectionsAfter.length-1)),o),selection:s}),userEvent:0==e?"select.undo":"select.redo",scrollIntoView:!0})}if(!r.changes)return null;{let n=1==i.length?R:i.slice(0,i.length-1);return r.mapped&&(n=M(n,r.mapped)),t.update({changes:r.changes,selection:r.startSelection,effects:r.effects,annotations:g.of({side:e,rest:n,selection:s}),filter:!1,userEvent:0==e?"undo":"redo",scrollIntoView:!0})}}}Q.empty=new Q(R,R);let N=[{key:"Mod-z",run:O,preventDefault:!0},{key:"Mod-y",mac:"Mod-Shift-z",run:w,preventDefault:!0},{linux:"Ctrl-Shift-z",run:w,preventDefault:!0},{key:"Mod-u",run:S,preventDefault:!0},{key:"Alt-u",mac:"Mod-Shift-u",run:P,preventDefault:!0}];function E(e,t){return o.jT.create(e.ranges.map(t),e.mainIndex)}function B(e,t){return e.update({selection:t,scrollIntoView:!0,userEvent:"select"})}function L({state:e,dispatch:t},n){let i=E(e.selection,n);return!i.eq(e.selection,!0)&&(t(B(e,i)),!0)}function $(e,t){return o.jT.cursor(t?e.to:e.from)}function j(e,t){return L(e,n=>n.empty?e.moveByChar(n,t):$(n,t))}function z(e){return e.textDirectionAt(e.state.selection.main.head)==l.Nm.LTR}let X=e=>j(e,!z(e)),q=e=>j(e,z(e));function J(e,t){return L(e,n=>n.empty?e.moveByGroup(n,t):$(n,t))}function W(e,t,n){let i,r,s=(0,a.qz)(e).resolveInner(t.head),l=n?h.md.closedBy:h.md.openedBy;for(let i=t.head;;){let t=n?s.childAfter(i):s.childBefore(i);if(!t)break;!function(e,t,n){if(t.type.prop(n))return!0;let i=t.to-t.from;return i&&(i>2||/[^\s,.;:]/.test(e.sliceDoc(t.from,t.to)))||t.firstChild}(e,t,l)?i=n?t.to:t.from:s=t}return r=s.type.prop(l)&&(i=n?(0,a.Um)(e,s.from,1):(0,a.Um)(e,s.to,-1))&&i.matched?n?i.end.to:i.end.from:n?s.to:s.from,o.jT.cursor(r,n?-1:1)}function _(e,t){return L(e,n=>{if(!n.empty)return $(n,t);let i=e.moveVertically(n,t);return i.head!=n.head?i:e.moveToLineBoundary(n,t)})}"undefined"!=typeof Intl&&Intl.Segmenter;let F=e=>_(e,!1),U=e=>_(e,!0);function V(e){let t=e.scrollDOM.clientHeight<e.scrollDOM.scrollHeight-2,n=0,i=0,r;if(t){for(let t of e.state.facet(l.tk.scrollMargins)){let r=t(e);(null==r?void 0:r.top)&&(n=Math.max(null==r?void 0:r.top,n)),(null==r?void 0:r.bottom)&&(i=Math.max(null==r?void 0:r.bottom,i))}r=e.scrollDOM.clientHeight-n-i}else r=(e.dom.ownerDocument.defaultView||window).innerHeight;return{marginTop:n,marginBottom:i,selfScroll:t,height:Math.max(e.defaultLineHeight,r-5)}}function Z(e,t){let n,i=V(e),{state:r}=e,s=E(r.selection,n=>n.empty?e.moveVertically(n,t,i.height):$(n,t));if(s.eq(r.selection))return!1;if(i.selfScroll){let t=e.coordsAtPos(r.selection.main.head),o=e.scrollDOM.getBoundingClientRect(),a=o.top+i.marginTop,h=o.bottom-i.marginBottom;t&&t.top>a&&t.bottom<h&&(n=l.tk.scrollIntoView(s.main.head,{y:"start",yMargin:t.top-a}))}return e.dispatch(B(r,s),{effects:n}),!0}let G=e=>Z(e,!1),Y=e=>Z(e,!0);function H(e,t,n){let i=e.lineBlockAt(t.head),r=e.moveToLineBoundary(t,n);if(r.head==t.head&&r.head!=(n?i.to:i.from)&&(r=e.moveToLineBoundary(t,n,!1)),!n&&r.head==i.from&&i.length){let n=/^\s*/.exec(e.state.sliceDoc(i.from,Math.min(i.from+100,i.to)))[0].length;n&&t.head!=i.from+n&&(r=o.jT.cursor(i.from+n))}return r}function K(e,t){let n=E(e.state.selection,e=>{let n=t(e);return o.jT.range(e.anchor,n.head,n.goalColumn,n.bidiLevel||void 0)});return!n.eq(e.state.selection)&&(e.dispatch(B(e.state,n)),!0)}function ee(e,t){return K(e,n=>e.moveByChar(n,t))}let et=e=>ee(e,!z(e)),en=e=>ee(e,z(e));function ei(e,t){return K(e,n=>e.moveByGroup(n,t))}function er(e,t){return K(e,n=>e.moveVertically(n,t))}let es=e=>er(e,!1),eo=e=>er(e,!0);function el(e,t){return K(e,n=>e.moveVertically(n,t,V(e).height))}let ea=e=>el(e,!1),eh=e=>el(e,!0),ec=({state:e,dispatch:t})=>(t(B(e,{anchor:0})),!0),eu=({state:e,dispatch:t})=>(t(B(e,{anchor:e.doc.length})),!0),ef=({state:e,dispatch:t})=>(t(B(e,{anchor:e.selection.main.anchor,head:0})),!0),ed=({state:e,dispatch:t})=>(t(B(e,{anchor:e.selection.main.anchor,head:e.doc.length})),!0);function ep(e,t){let{state:n}=e,i=n.selection,r=n.selection.ranges.slice();for(let i of n.selection.ranges){let s=n.doc.lineAt(i.head);if(t?s.to<e.state.doc.length:s.from>0)for(let n=i;;){let i=e.moveVertically(n,t);if(i.head<s.from||i.head>s.to){r.some(e=>e.head==i.head)||r.push(i);break}if(i.head==n.head)break;n=i}}return r.length!=i.ranges.length&&(e.dispatch(B(n,o.jT.create(r,r.length-1))),!0)}function em(e,t){if(e.state.readOnly)return!1;let n="delete.selection",{state:i}=e,r=i.changeByRange(i=>{let{from:r,to:s}=i;if(r==s){let o=t(i);o<r?(n="delete.backward",o=eg(e,o,!1)):o>r&&(n="delete.forward",o=eg(e,o,!0)),r=Math.min(r,o),s=Math.max(s,o)}else r=eg(e,r,!1),s=eg(e,s,!0);return r==s?{range:i}:{changes:{from:r,to:s},range:o.jT.cursor(r,r<i.head?-1:1)}});return!r.changes.empty&&(e.dispatch(i.update(r,{scrollIntoView:!0,userEvent:n,effects:"delete.selection"==n?l.tk.announce.of(i.phrase("Selection deleted")):void 0})),!0)}function eg(e,t,n){if(e instanceof l.tk)for(let i of e.state.facet(l.tk.atomicRanges).map(t=>t(e)))i.between(t,t,(e,i)=>{e<t&&i>t&&(t=n?i:e)});return t}let ek=(e,t,n)=>em(e,i=>{let r=i.from,{state:s}=e,l=s.doc.lineAt(r),h,c;if(n&&!t&&r>l.from&&r<l.from+200&&!/[^ \t]/.test(h=l.text.slice(0,r-l.from))){if("	"==h[h.length-1])return r-1;let e=(0,o.IS)(h,s.tabSize)%(0,a.y1)(s)||(0,a.y1)(s);for(let t=0;t<e&&" "==h[h.length-1-t];t++)r--;c=r}else(c=(0,o.cp)(l.text,r-l.from,t,t)+l.from)==r&&l.number!=(t?s.doc.lines:1)?c+=t?1:-1:!t&&/[\ufe00-\ufe0f]/.test(l.text.slice(c-l.from,r-l.from))&&(c=(0,o.cp)(l.text,c-l.from,!1,!1)+l.from);return c}),ev=e=>ek(e,!1,!0),ey=e=>ek(e,!0,!1),ex=(e,t)=>em(e,n=>{let i=n.head,{state:r}=e,s=r.doc.lineAt(i),l=r.charCategorizer(i);for(let e=null;;){if(i==(t?s.to:s.from)){i==n.head&&s.number!=(t?r.doc.lines:1)&&(i+=t?1:-1);break}let a=(0,o.cp)(s.text,i-s.from,t)+s.from,h=s.text.slice(Math.min(i,a)-s.from,Math.max(i,a)-s.from),c=l(h);if(null!=e&&c!=e)break;(" "!=h||i!=n.head)&&(e=c),i=a}return i}),eb=e=>ex(e,!1);function eO(e){let t=[],n=-1;for(let i of e.selection.ranges){let r=e.doc.lineAt(i.from),s=e.doc.lineAt(i.to);if(i.empty||i.to!=s.from||(s=e.doc.lineAt(i.to-1)),n>=r.number){let e=t[t.length-1];e.to=s.to,e.ranges.push(i)}else t.push({from:r.from,to:s.to,ranges:[i]});n=s.number+1}return t}function ew(e,t,n){if(e.readOnly)return!1;let i=[],r=[];for(let t of eO(e)){if(n?t.to==e.doc.length:0==t.from)continue;let s=e.doc.lineAt(n?t.to+1:t.from-1),l=s.length+1;if(n)for(let n of(i.push({from:t.to,to:s.to},{from:t.from,insert:s.text+e.lineBreak}),t.ranges))r.push(o.jT.range(Math.min(e.doc.length,n.anchor+l),Math.min(e.doc.length,n.head+l)));else for(let n of(i.push({from:s.from,to:t.from},{from:t.to,insert:e.lineBreak+s.text}),t.ranges))r.push(o.jT.range(n.anchor-l,n.head-l))}return!!i.length&&(t(e.update({changes:i,scrollIntoView:!0,selection:o.jT.create(r,e.selection.mainIndex),userEvent:"move.line"})),!0)}function eS(e,t,n){if(e.readOnly)return!1;let i=[];for(let t of eO(e))n?i.push({from:t.from,insert:e.doc.slice(t.from,t.to)+e.lineBreak}):i.push({from:t.to,insert:e.lineBreak+e.doc.slice(t.from,t.to)});let r=e.changes(i);return t(e.update({changes:r,selection:e.selection.map(r,n?1:-1),scrollIntoView:!0,userEvent:"input.copyline"})),!0}let eP=eA(!1),eC=eA(!0);function eA(e){return({state:t,dispatch:n})=>{if(t.readOnly)return!1;let i=t.changeByRange(n=>{let{from:i,to:r}=n,s=t.doc.lineAt(i),l=!e&&i==r&&function(e,t){if(/\(\)|\[\]|\{\}/.test(e.sliceDoc(t-1,t+1)))return{from:t,to:t};let n=(0,a.qz)(e).resolveInner(t),i=n.childBefore(t),r=n.childAfter(t),s;return i&&r&&i.to<=t&&r.from>=t&&(s=i.type.prop(h.md.closedBy))&&s.indexOf(r.name)>-1&&e.doc.lineAt(i.to).from==e.doc.lineAt(r.from).from&&!/\S/.test(e.sliceDoc(i.to,r.from))?{from:i.to,to:r.from}:null}(t,i);e&&(i=r=(r<=s.to?s:t.doc.lineAt(r)).to);let c=new a.Gn(t,{simulateBreak:i,simulateDoubleBreak:!!l}),u=(0,a.K0)(c,i);for(null==u&&(u=(0,o.IS)(/^\s*/.exec(t.doc.lineAt(i).text)[0],t.tabSize));r<s.to&&/\s/.test(s.text[r-s.from]);)r++;l?{from:i,to:r}=l:i>s.from&&i<s.from+100&&!/\S/.test(s.text.slice(0,i))&&(i=s.from);let f=["",(0,a.SS)(t,u)];return l&&f.push((0,a.SS)(t,c.lineIndent(s.from,-1))),{changes:{from:i,to:r,insert:o.xv.of(f)},range:o.jT.cursor(i+1+f[1].length)}});return n(t.update(i,{scrollIntoView:!0,userEvent:"input"})),!0}}function eT(e,t){let n=-1;return e.changeByRange(i=>{let r=[];for(let s=i.from;s<=i.to;){let o=e.doc.lineAt(s);o.number>n&&(i.empty||i.to>o.from)&&(t(o,r,i),n=o.number),s=o.to+1}let s=e.changes(r);return{changes:r,range:o.jT.range(s.mapPos(i.anchor,1),s.mapPos(i.head,1))}})}let eR=({state:e,dispatch:t})=>!e.readOnly&&(t(e.update(eT(e,(t,n)=>{n.push({from:t.from,insert:e.facet(a.c)})}),{userEvent:"input.indent"})),!0),eD=({state:e,dispatch:t})=>!e.readOnly&&(t(e.update(eT(e,(t,n)=>{let i=/^\s*/.exec(t.text)[0];if(!i)return;let r=(0,o.IS)(i,e.tabSize),s=0,l=(0,a.SS)(e,Math.max(0,r-(0,a.y1)(e)));for(;s<i.length&&s<l.length&&i.charCodeAt(s)==l.charCodeAt(s);)s++;n.push({from:t.from+s,to:t.from+i.length,insert:l.slice(s)})}),{userEvent:"delete.dedent"})),!0),eM=[{key:"Ctrl-b",run:X,shift:et,preventDefault:!0},{key:"Ctrl-f",run:q,shift:en},{key:"Ctrl-p",run:F,shift:es},{key:"Ctrl-n",run:U,shift:eo},{key:"Ctrl-a",run:e=>L(e,t=>o.jT.cursor(e.lineBlockAt(t.head).from,1)),shift:e=>K(e,t=>o.jT.cursor(e.lineBlockAt(t.head).from))},{key:"Ctrl-e",run:e=>L(e,t=>o.jT.cursor(e.lineBlockAt(t.head).to,-1)),shift:e=>K(e,t=>o.jT.cursor(e.lineBlockAt(t.head).to))},{key:"Ctrl-d",run:ey},{key:"Ctrl-h",run:ev},{key:"Ctrl-k",run:e=>em(e,t=>{let n=e.lineBlockAt(t.head).to;return t.head<n?n:Math.min(e.state.doc.length,t.head+1)})},{key:"Ctrl-Alt-h",run:eb},{key:"Ctrl-o",run:({state:e,dispatch:t})=>{if(e.readOnly)return!1;let n=e.changeByRange(e=>({changes:{from:e.from,to:e.to,insert:o.xv.of(["",""])},range:o.jT.cursor(e.from)}));return t(e.update(n,{scrollIntoView:!0,userEvent:"input"})),!0}},{key:"Ctrl-t",run:({state:e,dispatch:t})=>{if(e.readOnly)return!1;let n=e.changeByRange(t=>{if(!t.empty||0==t.from||t.from==e.doc.length)return{range:t};let n=t.from,i=e.doc.lineAt(n),r=n==i.from?n-1:(0,o.cp)(i.text,n-i.from,!1)+i.from,s=n==i.to?n+1:(0,o.cp)(i.text,n-i.from,!0)+i.from;return{changes:{from:r,to:s,insert:e.doc.slice(n,s).append(e.doc.slice(r,n))},range:o.jT.cursor(s)}});return!n.changes.empty&&(t(e.update(n,{scrollIntoView:!0,userEvent:"move.character"})),!0)}},{key:"Ctrl-v",run:Y}],eI=[{key:"Alt-ArrowLeft",mac:"Ctrl-ArrowLeft",run:e=>L(e,t=>W(e.state,t,!z(e))),shift:e=>K(e,t=>W(e.state,t,!z(e)))},{key:"Alt-ArrowRight",mac:"Ctrl-ArrowRight",run:e=>L(e,t=>W(e.state,t,z(e))),shift:e=>K(e,t=>W(e.state,t,z(e)))},{key:"Alt-ArrowUp",run:({state:e,dispatch:t})=>ew(e,t,!1)},{key:"Shift-Alt-ArrowUp",run:({state:e,dispatch:t})=>eS(e,t,!1)},{key:"Alt-ArrowDown",run:({state:e,dispatch:t})=>ew(e,t,!0)},{key:"Shift-Alt-ArrowDown",run:({state:e,dispatch:t})=>eS(e,t,!0)},{key:"Mod-Alt-ArrowUp",run:e=>ep(e,!1)},{key:"Mod-Alt-ArrowDown",run:e=>ep(e,!0)},{key:"Escape",run:({state:e,dispatch:t})=>{let n=e.selection,i=null;return n.ranges.length>1?i=o.jT.create([n.main]):n.main.empty||(i=o.jT.create([o.jT.cursor(n.main.head)])),!!i&&(t(B(e,i)),!0)}},{key:"Mod-Enter",run:eC},{key:"Alt-l",mac:"Ctrl-l",run:({state:e,dispatch:t})=>{let n=eO(e).map(({from:t,to:n})=>o.jT.range(t,Math.min(n+1,e.doc.length)));return t(e.update({selection:o.jT.create(n),userEvent:"select"})),!0}},{key:"Mod-i",run:({state:e,dispatch:t})=>{let n=E(e.selection,t=>{let n=(0,a.qz)(e),i=n.resolveStack(t.from,1);if(t.empty){let e=n.resolveStack(t.from,-1);e.node.from>=i.node.from&&e.node.to<=i.node.to&&(i=e)}for(let e=i;e;e=e.next){let{node:n}=e;if((n.from<t.from&&n.to>=t.to||n.to>t.to&&n.from<=t.from)&&e.next)return o.jT.range(n.to,n.from)}return t});return!n.eq(e.selection)&&(t(B(e,n)),!0)},preventDefault:!0},{key:"Mod-[",run:eD},{key:"Mod-]",run:eR},{key:"Mod-Alt-\\",run:({state:e,dispatch:t})=>{if(e.readOnly)return!1;let n=Object.create(null),i=new a.Gn(e,{overrideIndentation:e=>{let t=n[e];return null==t?-1:t}}),r=eT(e,(t,r,s)=>{let o=(0,a.K0)(i,t.from);if(null==o)return;/\S/.test(t.text)||(o=0);let l=/^\s*/.exec(t.text)[0],h=(0,a.SS)(e,o);(l!=h||s.from<t.from+l.length)&&(n[t.from]=o,r.push({from:t.from,to:t.from+l.length,insert:h}))});return r.changes.empty||t(e.update(r,{userEvent:"indent"})),!0}},{key:"Shift-Mod-k",run:e=>{if(e.state.readOnly)return!1;let{state:t}=e,n=t.changes(eO(t).map(({from:e,to:n})=>(e>0?e--:n<t.doc.length&&n++,{from:e,to:n}))),i=E(t.selection,t=>{let n;if(e.lineWrapping){let i=e.lineBlockAt(t.head),r=e.coordsAtPos(t.head,t.assoc||1);r&&(n=i.bottom+e.documentTop-r.bottom+e.defaultLineHeight/2)}return e.moveVertically(t,!0,n)}).map(n);return e.dispatch({changes:n,selection:i,scrollIntoView:!0,userEvent:"delete.line"}),!0}},{key:"Shift-Mod-\\",run:({state:e,dispatch:t})=>{let n,i;return n=!1,i=E(e.selection,t=>{let i=(0,a.Um)(e,t.head,-1)||(0,a.Um)(e,t.head,1)||t.head>0&&(0,a.Um)(e,t.head-1,1)||t.head<e.doc.length&&(0,a.Um)(e,t.head+1,-1);if(!i||!i.end)return t;n=!0;let r=i.start.from==t.head?i.end.to:i.end.from;return o.jT.cursor(r)}),!!n&&(t(B(e,i)),!0)}},{key:"Mod-/",run:e=>{let{state:t}=e,n=t.doc.lineAt(t.selection.main.from),i=p(e.state,n.from);return i.line?u(e):!!i.block&&d(e)}},{key:"Alt-A",run:f},{key:"Ctrl-m",mac:"Shift-Alt-m",run:e=>(e.setTabFocusMode(),!0)}].concat([{key:"ArrowLeft",run:X,shift:et,preventDefault:!0},{key:"Mod-ArrowLeft",mac:"Alt-ArrowLeft",run:e=>J(e,!z(e)),shift:e=>ei(e,!z(e)),preventDefault:!0},{mac:"Cmd-ArrowLeft",run:e=>L(e,t=>H(e,t,!z(e))),shift:e=>K(e,t=>H(e,t,!z(e))),preventDefault:!0},{key:"ArrowRight",run:q,shift:en,preventDefault:!0},{key:"Mod-ArrowRight",mac:"Alt-ArrowRight",run:e=>J(e,z(e)),shift:e=>ei(e,z(e)),preventDefault:!0},{mac:"Cmd-ArrowRight",run:e=>L(e,t=>H(e,t,z(e))),shift:e=>K(e,t=>H(e,t,z(e))),preventDefault:!0},{key:"ArrowUp",run:F,shift:es,preventDefault:!0},{mac:"Cmd-ArrowUp",run:ec,shift:ef},{mac:"Ctrl-ArrowUp",run:G,shift:ea},{key:"ArrowDown",run:U,shift:eo,preventDefault:!0},{mac:"Cmd-ArrowDown",run:eu,shift:ed},{mac:"Ctrl-ArrowDown",run:Y,shift:eh},{key:"PageUp",run:G,shift:ea},{key:"PageDown",run:Y,shift:eh},{key:"Home",run:e=>L(e,t=>H(e,t,!1)),shift:e=>K(e,t=>H(e,t,!1)),preventDefault:!0},{key:"Mod-Home",run:ec,shift:ef},{key:"End",run:e=>L(e,t=>H(e,t,!0)),shift:e=>K(e,t=>H(e,t,!0)),preventDefault:!0},{key:"Mod-End",run:eu,shift:ed},{key:"Enter",run:eP,shift:eP},{key:"Mod-a",run:({state:e,dispatch:t})=>(t(e.update({selection:{anchor:0,head:e.doc.length},userEvent:"select"})),!0)},{key:"Backspace",run:ev,shift:ev,preventDefault:!0},{key:"Delete",run:ey,preventDefault:!0},{key:"Mod-Backspace",mac:"Alt-Backspace",run:eb,preventDefault:!0},{key:"Mod-Delete",mac:"Alt-Delete",run:e=>ex(e,!0),preventDefault:!0},{mac:"Mod-Backspace",run:e=>em(e,t=>{let n=e.moveToLineBoundary(t,!1).head;return t.head>n?n:Math.max(0,t.head-1)}),preventDefault:!0},{mac:"Mod-Delete",run:e=>em(e,t=>{let n=e.moveToLineBoundary(t,!0).head;return t.head<n?n:Math.min(e.state.doc.length,t.head+1)}),preventDefault:!0}].concat(eM.map(e=>({mac:e.key,run:e.run,shift:e.shift})))),eQ={key:"Tab",run:eR,shift:eD};var eN=n(18580);let eE="function"==typeof String.prototype.normalize?e=>e.normalize("NFKD"):e=>e;class eB{constructor(e,t,n=0,i=e.length,r,s){this.test=s,this.value={from:0,to:0},this.done=!1,this.matches=[],this.buffer="",this.bufferPos=0,this.iter=e.iterRange(n,i),this.bufferStart=n,this.normalize=r?e=>r(eE(e)):eE,this.query=this.normalize(t)}peek(){if(this.bufferPos==this.buffer.length){if(this.bufferStart+=this.buffer.length,this.iter.next(),this.iter.done)return -1;this.bufferPos=0,this.buffer=this.iter.value}return(0,o.gm)(this.buffer,this.bufferPos)}next(){for(;this.matches.length;)this.matches.pop();return this.nextOverlapping()}nextOverlapping(){for(;;){let e=this.peek();if(e<0)return this.done=!0,this;let t=(0,o.bg)(e),n=this.bufferStart+this.bufferPos;this.bufferPos+=(0,o.nZ)(e);let i=this.normalize(t);if(i.length)for(let e=0,r=n;;e++){let s=i.charCodeAt(e),o=this.match(s,r,this.bufferPos+this.bufferStart);if(e==i.length-1){if(o)return this.value=o,this;break}r==n&&e<t.length&&t.charCodeAt(e)==s&&r++}}}match(e,t,n){let i=null;for(let t=0;t<this.matches.length;t+=2){let r=this.matches[t],s=!1;this.query.charCodeAt(r)==e&&(r==this.query.length-1?i={from:this.matches[t+1],to:n}:(this.matches[t]++,s=!0)),s||(this.matches.splice(t,2),t-=2)}return this.query.charCodeAt(0)==e&&(1==this.query.length?i={from:t,to:n}:this.matches.push(1,t)),i&&this.test&&!this.test(i.from,i.to,this.buffer,this.bufferStart)&&(i=null),i}}"undefined"!=typeof Symbol&&(eB.prototype[Symbol.iterator]=function(){return this});let eL={from:-1,to:-1,match:/.*/.exec("")},e$="gm"+(null==/x/.unicode?"":"u");class ej{constructor(e,t,n,i=0,r=e.length){if(this.text=e,this.to=r,this.curLine="",this.done=!1,this.value=eL,/\\[sWDnr]|\n|\r|\[\^/.test(t))return new eq(e,t,n,i,r);this.re=new RegExp(t,e$+((null==n?void 0:n.ignoreCase)?"i":"")),this.test=null==n?void 0:n.test,this.iter=e.iter();let s=e.lineAt(i);this.curLineStart=s.from,this.matchPos=eJ(e,i),this.getLine(this.curLineStart)}getLine(e){this.iter.next(e),this.iter.lineBreak?this.curLine="":(this.curLine=this.iter.value,this.curLineStart+this.curLine.length>this.to&&(this.curLine=this.curLine.slice(0,this.to-this.curLineStart)),this.iter.next())}nextLine(){this.curLineStart=this.curLineStart+this.curLine.length+1,this.curLineStart>this.to?this.curLine="":this.getLine(0)}next(){for(let e=this.matchPos-this.curLineStart;;){this.re.lastIndex=e;let t=this.matchPos<=this.to&&this.re.exec(this.curLine);if(t){let n=this.curLineStart+t.index,i=n+t[0].length;if(this.matchPos=eJ(this.text,i+(n==i?1:0)),n==this.curLineStart+this.curLine.length&&this.nextLine(),(n<i||n>this.value.to)&&(!this.test||this.test(n,i,t)))return this.value={from:n,to:i,match:t},this;e=this.matchPos-this.curLineStart}else{if(!(this.curLineStart+this.curLine.length<this.to))return this.done=!0,this;this.nextLine(),e=0}}}}let ez=new WeakMap;class eX{constructor(e,t){this.from=e,this.text=t}get to(){return this.from+this.text.length}static get(e,t,n){let i=ez.get(e);if(!i||i.from>=n||i.to<=t){let i=new eX(t,e.sliceString(t,n));return ez.set(e,i),i}if(i.from==t&&i.to==n)return i;let{text:r,from:s}=i;return s>t&&(r=e.sliceString(t,s)+r,s=t),i.to<n&&(r+=e.sliceString(i.to,n)),ez.set(e,new eX(s,r)),new eX(t,r.slice(t-s,n-s))}}class eq{constructor(e,t,n,i,r){this.text=e,this.to=r,this.done=!1,this.value=eL,this.matchPos=eJ(e,i),this.re=new RegExp(t,e$+((null==n?void 0:n.ignoreCase)?"i":"")),this.test=null==n?void 0:n.test,this.flat=eX.get(e,i,this.chunkEnd(i+5e3))}chunkEnd(e){return e>=this.to?this.to:this.text.lineAt(e).to}next(){for(;;){let e=this.re.lastIndex=this.matchPos-this.flat.from,t=this.re.exec(this.flat.text);if(t&&!t[0]&&t.index==e&&(this.re.lastIndex=e+1,t=this.re.exec(this.flat.text)),t){let e=this.flat.from+t.index,n=e+t[0].length;if((this.flat.to>=this.to||t.index+t[0].length<=this.flat.text.length-10)&&(!this.test||this.test(e,n,t)))return this.value={from:e,to:n,match:t},this.matchPos=eJ(this.text,n+(e==n?1:0)),this}if(this.flat.to==this.to)return this.done=!0,this;this.flat=eX.get(this.text,this.flat.from,this.chunkEnd(this.flat.from+2*this.flat.text.length))}}}function eJ(e,t){if(t>=e.length)return t;let n=e.lineAt(t),i;for(;t<n.to&&(i=n.text.charCodeAt(t-n.from))>=56320&&i<57344;)t++;return t}"undefined"!=typeof Symbol&&(ej.prototype[Symbol.iterator]=eq.prototype[Symbol.iterator]=function(){return this});let eW={highlightWordAroundCursor:!1,minSelectionLength:1,maxMatches:100,wholeWords:!1},e_=o.r$.define({combine:e=>(0,o.BO)(e,eW,{highlightWordAroundCursor:(e,t)=>e||t,minSelectionLength:Math.min,maxMatches:Math.min})}),eF=l.p.mark({class:"cm-selectionMatch"}),eU=l.p.mark({class:"cm-selectionMatch cm-selectionMatch-main"});function eV(e,t,n,i){return(0==n||e(t.sliceDoc(n-1,n))!=o.D0.Word)&&(i==t.doc.length||e(t.sliceDoc(i,i+1))!=o.D0.Word)}let eZ=l.lg.fromClass(class{constructor(e){this.decorations=this.getDeco(e)}update(e){(e.selectionSet||e.docChanged||e.viewportChanged)&&(this.decorations=this.getDeco(e.view))}getDeco(e){let t=e.state.facet(e_),{state:n}=e,i=n.selection;if(i.ranges.length>1)return l.p.none;let r=i.main,s,a=null;if(r.empty){if(!t.highlightWordAroundCursor)return l.p.none;let e=n.wordAt(r.head);if(!e)return l.p.none;a=n.charCategorizer(r.head),s=n.sliceDoc(e.from,e.to)}else{let e=r.to-r.from;if(e<t.minSelectionLength||e>200)return l.p.none;if(t.wholeWords){var h,c,u;if(s=n.sliceDoc(r.from,r.to),!(eV(a=n.charCategorizer(r.head),n,r.from,r.to)&&(h=a,c=r.from,u=r.to,h(n.sliceDoc(c,c+1))==o.D0.Word&&h(n.sliceDoc(u-1,u))==o.D0.Word)))return l.p.none}else if(!(s=n.sliceDoc(r.from,r.to)))return l.p.none}let f=[];for(let i of e.visibleRanges){let e=new eB(n.doc,s,i.from,i.to);for(;!e.next().done;){let{from:i,to:s}=e.value;if((!a||eV(a,n,i,s))&&(r.empty&&i<=r.from&&s>=r.to?f.push(eU.range(i,s)):(i>=r.to||s<=r.from)&&f.push(eF.range(i,s)),f.length>t.maxMatches))return l.p.none}}return l.p.set(f)}},{decorations:e=>e.decorations}),eG=l.tk.baseTheme({".cm-selectionMatch":{backgroundColor:"#99ff7780"},".cm-searchMatch .cm-selectionMatch":{backgroundColor:"transparent"}}),eY=({state:e,dispatch:t})=>{let{selection:n}=e,i=o.jT.create(n.ranges.map(t=>e.wordAt(t.head)||o.jT.cursor(t.head)),n.mainIndex);return!i.eq(n)&&(t(e.update({selection:i})),!0)},eH=o.r$.define({combine:e=>(0,o.BO)(e,{top:!1,caseSensitive:!1,literal:!1,regexp:!1,wholeWord:!1,createPanel:e=>new tk(e),scrollToMatch:e=>l.tk.scrollIntoView(e)})});class eK{constructor(e){this.search=e.search,this.caseSensitive=!!e.caseSensitive,this.literal=!!e.literal,this.regexp=!!e.regexp,this.replace=e.replace||"",this.valid=!!this.search&&(!this.regexp||function(e){try{return new RegExp(e,e$),!0}catch(e){return!1}}(this.search)),this.unquoted=this.unquote(this.search),this.wholeWord=!!e.wholeWord,this.test=e.test}unquote(e){return this.literal?e:e.replace(/\\([nrt\\])/g,(e,t)=>"n"==t?"\n":"r"==t?"\r":"t"==t?"	":"\\")}eq(e){return this.search==e.search&&this.replace==e.replace&&this.caseSensitive==e.caseSensitive&&this.regexp==e.regexp&&this.wholeWord==e.wholeWord&&this.test==e.test}create(){return this.regexp?new e9(this):new e2(this)}getCursor(e,t=0,n){let i=e.doc?e:o.yy.create({doc:e});return null==n&&(n=i.doc.length),this.regexp?e5(this,i,t,n):e1(this,i,t,n)}}class e0{constructor(e){this.spec=e}}function e1(e,t,n,i){var r,s,l,a;let h;return e.wholeWord&&(r=t.doc,s=t.charCategorizer(t.selection.main.head),h=(e,t,n,i)=>((i>e||i+n.length<t)&&(i=Math.max(0,e-2),n=r.sliceString(i,Math.min(r.length,t+2))),(s(e3(n,e-i))!=o.D0.Word||s(e4(n,e-i))!=o.D0.Word)&&(s(e4(n,t-i))!=o.D0.Word||s(e3(n,t-i))!=o.D0.Word))),e.test&&(l=e.test,a=h,h=(e,n,i,r)=>(!a||!!a(e,n,i,r))&&l(e>=r&&n<=r+i.length?i.slice(e-r,n-r):t.doc.sliceString(e,n),t,e,n)),new eB(t.doc,e.unquoted,n,i,e.caseSensitive?void 0:e=>e.toLowerCase(),h)}class e2 extends e0{constructor(e){super(e)}nextMatch(e,t,n){let i=e1(this.spec,e,n,e.doc.length).nextOverlapping();if(i.done){let n=Math.min(e.doc.length,t+this.spec.unquoted.length);i=e1(this.spec,e,0,n).nextOverlapping()}return i.done||i.value.from==t&&i.value.to==n?null:i.value}prevMatchInRange(e,t,n){for(let i=n;;){let n=Math.max(t,i-1e4-this.spec.unquoted.length),r=e1(this.spec,e,n,i),s=null;for(;!r.nextOverlapping().done;)s=r.value;if(s)return s;if(n==t)return null;i-=1e4}}prevMatch(e,t,n){let i=this.prevMatchInRange(e,0,t);return i||(i=this.prevMatchInRange(e,Math.max(0,n-this.spec.unquoted.length),e.doc.length)),i&&(i.from!=t||i.to!=n)?i:null}getReplacement(e){return this.spec.unquote(this.spec.replace)}matchAll(e,t){let n=e1(this.spec,e,0,e.doc.length),i=[];for(;!n.next().done;){if(i.length>=t)return null;i.push(n.value)}return i}highlight(e,t,n,i){let r=e1(this.spec,e,Math.max(0,t-this.spec.unquoted.length),Math.min(n+this.spec.unquoted.length,e.doc.length));for(;!r.next().done;)i(r.value.from,r.value.to)}}function e5(e,t,n,i){var r,s,l;let a;return e.wholeWord&&(r=t.charCategorizer(t.selection.main.head),a=(e,t,n)=>!n[0].length||(r(e3(n.input,n.index))!=o.D0.Word||r(e4(n.input,n.index))!=o.D0.Word)&&(r(e4(n.input,n.index+n[0].length))!=o.D0.Word||r(e3(n.input,n.index+n[0].length))!=o.D0.Word)),e.test&&(s=e.test,l=a,a=(e,n,i)=>(!l||l(e,n,i))&&s(i[0],t,e,n)),new ej(t.doc,e.search,{ignoreCase:!e.caseSensitive,test:a},n,i)}function e3(e,t){return e.slice((0,o.cp)(e,t,!1),t)}function e4(e,t){return e.slice(t,(0,o.cp)(e,t))}class e9 extends e0{nextMatch(e,t,n){let i=e5(this.spec,e,n,e.doc.length).next();return i.done&&(i=e5(this.spec,e,0,t).next()),i.done?null:i.value}prevMatchInRange(e,t,n){for(let i=1;;i++){let r=Math.max(t,n-1e4*i),s=e5(this.spec,e,r,n),o=null;for(;!s.next().done;)o=s.value;if(o&&(r==t||o.from>r+10))return o;if(r==t)return null}}prevMatch(e,t,n){return this.prevMatchInRange(e,0,t)||this.prevMatchInRange(e,n,e.doc.length)}getReplacement(e){return this.spec.unquote(this.spec.replace).replace(/\$([$&]|\d+)/g,(t,n)=>{if("&"==n)return e.match[0];if("$"==n)return"$";for(let t=n.length;t>0;t--){let i=+n.slice(0,t);if(i>0&&i<e.match.length)return e.match[i]+n.slice(t)}return t})}matchAll(e,t){let n=e5(this.spec,e,0,e.doc.length),i=[];for(;!n.next().done;){if(i.length>=t)return null;i.push(n.value)}return i}highlight(e,t,n,i){let r=e5(this.spec,e,Math.max(0,t-250),Math.min(n+250,e.doc.length));for(;!r.next().done;)i(r.value.from,r.value.to)}}let e6=o.Py.define(),e7=o.Py.define(),e8=o.QQ.define({create:e=>new te(tu(e).create(),null),update(e,t){for(let n of t.effects)n.is(e6)?e=new te(n.value.create(),e.panel):n.is(e7)&&(e=new te(e.query,n.value?tc:null));return e},provide:e=>l.mH.from(e,e=>e.panel)});class te{constructor(e,t){this.query=e,this.panel=t}}let tt=l.p.mark({class:"cm-searchMatch"}),tn=l.p.mark({class:"cm-searchMatch cm-searchMatch-selected"}),ti=l.lg.fromClass(class{constructor(e){this.view=e,this.decorations=this.highlight(e.state.field(e8))}update(e){let t=e.state.field(e8);(t!=e.startState.field(e8)||e.docChanged||e.selectionSet||e.viewportChanged)&&(this.decorations=this.highlight(t))}highlight({query:e,panel:t}){if(!t||!e.spec.valid)return l.p.none;let{view:n}=this,i=new o.f_;for(let t=0,r=n.visibleRanges,s=r.length;t<s;t++){let{from:o,to:l}=r[t];for(;t<s-1&&l>r[t+1].from-500;)l=r[++t].to;e.highlight(n.state,o,l,(e,t)=>{let r=n.state.selection.ranges.some(n=>n.from==e&&n.to==t);i.add(e,t,r?tn:tt)})}return i.finish()}},{decorations:e=>e.decorations});function tr(e){return t=>{let n=t.state.field(e8,!1);return n&&n.query.spec.valid?e(t,n):tp(t)}}let ts=tr((e,{query:t})=>{let{to:n}=e.state.selection.main,i=t.nextMatch(e.state,n,n);if(!i)return!1;let r=o.jT.single(i.from,i.to),s=e.state.facet(eH);return e.dispatch({selection:r,effects:[tx(e,i),s.scrollToMatch(r.main,e)],userEvent:"select.search"}),td(e),!0}),to=tr((e,{query:t})=>{let{state:n}=e,{from:i}=n.selection.main,r=t.prevMatch(n,i,i);if(!r)return!1;let s=o.jT.single(r.from,r.to),l=e.state.facet(eH);return e.dispatch({selection:s,effects:[tx(e,r),l.scrollToMatch(s.main,e)],userEvent:"select.search"}),td(e),!0}),tl=tr((e,{query:t})=>{let n=t.matchAll(e.state,1e3);return!!n&&!!n.length&&(e.dispatch({selection:o.jT.create(n.map(e=>o.jT.range(e.from,e.to))),userEvent:"select.search.matches"}),!0)}),ta=tr((e,{query:t})=>{let{state:n}=e,{from:i,to:r}=n.selection.main;if(n.readOnly)return!1;let s=t.nextMatch(n,i,i);if(!s)return!1;let a=s,h=[],c,u,f=[];a.from==i&&a.to==r&&(u=n.toText(t.getReplacement(a)),h.push({from:a.from,to:a.to,insert:u}),a=t.nextMatch(n,a.from,a.to),f.push(l.tk.announce.of(n.phrase("replaced match on line $",n.doc.lineAt(i).number)+".")));let d=e.state.changes(h);return a&&(c=o.jT.single(a.from,a.to).map(d),f.push(tx(e,a)),f.push(n.facet(eH).scrollToMatch(c.main,e))),e.dispatch({changes:d,selection:c,effects:f,userEvent:"input.replace"}),!0}),th=tr((e,{query:t})=>{if(e.state.readOnly)return!1;let n=t.matchAll(e.state,1e9).map(e=>{let{from:n,to:i}=e;return{from:n,to:i,insert:t.getReplacement(e)}});if(!n.length)return!1;let i=e.state.phrase("replaced $ matches",n.length)+".";return e.dispatch({changes:n,effects:l.tk.announce.of(i),userEvent:"input.replace.all"}),!0});function tc(e){return e.state.facet(eH).createPanel(e)}function tu(e,t){var n,i,r,s,o;let l=e.selection.main,a=l.empty||l.to>l.from+100?"":e.sliceDoc(l.from,l.to);if(t&&!a)return t;let h=e.facet(eH);return new eK({search:(null!==(n=null==t?void 0:t.literal)&&void 0!==n?n:h.literal)?a:a.replace(/\n/g,"\\n"),caseSensitive:null!==(i=null==t?void 0:t.caseSensitive)&&void 0!==i?i:h.caseSensitive,literal:null!==(r=null==t?void 0:t.literal)&&void 0!==r?r:h.literal,regexp:null!==(s=null==t?void 0:t.regexp)&&void 0!==s?s:h.regexp,wholeWord:null!==(o=null==t?void 0:t.wholeWord)&&void 0!==o?o:h.wholeWord})}function tf(e){let t=(0,l.Sd)(e,tc);return t&&t.dom.querySelector("[main-field]")}function td(e){let t=tf(e);t&&t==e.root.activeElement&&t.select()}let tp=e=>{let t=e.state.field(e8,!1);if(t&&t.panel){let n=tf(e);if(n&&n!=e.root.activeElement){let i=tu(e.state,t.query.spec);i.valid&&e.dispatch({effects:e6.of(i)}),n.focus(),n.select()}}else e.dispatch({effects:[e7.of(!0),t?e6.of(tu(e.state,t.query.spec)):o.Py.appendConfig.of(tO)]});return!0},tm=e=>{let t=e.state.field(e8,!1);if(!t||!t.panel)return!1;let n=(0,l.Sd)(e,tc);return n&&n.dom.contains(e.root.activeElement)&&e.focus(),e.dispatch({effects:e7.of(!1)}),!0},tg=[{key:"Mod-f",run:tp,scope:"editor search-panel"},{key:"F3",run:ts,shift:to,scope:"editor search-panel",preventDefault:!0},{key:"Mod-g",run:ts,shift:to,scope:"editor search-panel",preventDefault:!0},{key:"Escape",run:tm,scope:"editor search-panel"},{key:"Mod-Shift-l",run:({state:e,dispatch:t})=>{let n=e.selection;if(n.ranges.length>1||n.main.empty)return!1;let{from:i,to:r}=n.main,s=[],l=0;for(let t=new eB(e.doc,e.sliceDoc(i,r));!t.next().done;){if(s.length>1e3)return!1;t.value.from==i&&(l=s.length),s.push(o.jT.range(t.value.from,t.value.to))}return t(e.update({selection:o.jT.create(s,l),userEvent:"select.search.matches"})),!0}},{key:"Mod-Alt-g",run:e=>{let{state:t}=e,n=String(t.doc.lineAt(e.state.selection.main.head).number),{close:i,result:r}=(0,l.vC)(e,{label:t.phrase("Go to line"),input:{type:"text",name:"line",value:n},focus:!0,submitLabel:t.phrase("go")});return r.then(n=>{let r=n&&/^([+-])?(\d+)?(:\d+)?(%)?$/.exec(n.elements.line.value);if(!r){e.dispatch({effects:i});return}let s=t.doc.lineAt(t.selection.main.head),[,a,h,c,u]=r,f=c?+c.slice(1):0,d=h?+h:s.number;if(h&&u){let e=d/100;a&&(e=e*("-"==a?-1:1)+s.number/t.doc.lines),d=Math.round(t.doc.lines*e)}else h&&a&&(d=d*("-"==a?-1:1)+s.number);let p=t.doc.line(Math.max(1,Math.min(t.doc.lines,d))),m=o.jT.cursor(p.from+Math.max(0,Math.min(f,p.length)));e.dispatch({effects:[i,l.tk.scrollIntoView(m.from,{y:"center"})],selection:m})}),!0}},{key:"Mod-d",run:({state:e,dispatch:t})=>{let{ranges:n}=e.selection;if(n.some(e=>e.from===e.to))return eY({state:e,dispatch:t});let i=e.sliceDoc(n[0].from,n[0].to);if(e.selection.ranges.some(t=>e.sliceDoc(t.from,t.to)!=i))return!1;let r=function(e,t){let{main:n,ranges:i}=e.selection,r=e.wordAt(n.head),s=r&&r.from==n.from&&r.to==n.to;for(let n=!1,r=new eB(e.doc,t,i[i.length-1].to);;)if(r.next(),r.done){if(n)return null;r=new eB(e.doc,t,0,Math.max(0,i[i.length-1].from-1)),n=!0}else{if(n&&i.some(e=>e.from==r.value.from))continue;if(s){let t=e.wordAt(r.value.from);if(!t||t.from!=r.value.from||t.to!=r.value.to)continue}return r.value}}(e,i);return!!r&&(t(e.update({selection:e.selection.addRange(o.jT.range(r.from,r.to),!1),effects:l.tk.scrollIntoView(r.to)})),!0)},preventDefault:!0}];class tk{constructor(e){this.view=e;let t=this.query=e.state.field(e8).query.spec;function n(e,t,n){return(0,eN.Z)("button",{class:"cm-button",name:e,onclick:t,type:"button"},n)}this.commit=this.commit.bind(this),this.searchField=(0,eN.Z)("input",{value:t.search,placeholder:tv(e,"Find"),"aria-label":tv(e,"Find"),class:"cm-textfield",name:"search",form:"","main-field":"true",onchange:this.commit,onkeyup:this.commit}),this.replaceField=(0,eN.Z)("input",{value:t.replace,placeholder:tv(e,"Replace"),"aria-label":tv(e,"Replace"),class:"cm-textfield",name:"replace",form:"",onchange:this.commit,onkeyup:this.commit}),this.caseField=(0,eN.Z)("input",{type:"checkbox",name:"case",form:"",checked:t.caseSensitive,onchange:this.commit}),this.reField=(0,eN.Z)("input",{type:"checkbox",name:"re",form:"",checked:t.regexp,onchange:this.commit}),this.wordField=(0,eN.Z)("input",{type:"checkbox",name:"word",form:"",checked:t.wholeWord,onchange:this.commit}),this.dom=(0,eN.Z)("div",{onkeydown:e=>this.keydown(e),class:"cm-search"},[this.searchField,n("next",()=>ts(e),[tv(e,"next")]),n("prev",()=>to(e),[tv(e,"previous")]),n("select",()=>tl(e),[tv(e,"all")]),(0,eN.Z)("label",null,[this.caseField,tv(e,"match case")]),(0,eN.Z)("label",null,[this.reField,tv(e,"regexp")]),(0,eN.Z)("label",null,[this.wordField,tv(e,"by word")]),...e.state.readOnly?[]:[(0,eN.Z)("br"),this.replaceField,n("replace",()=>ta(e),[tv(e,"replace")]),n("replaceAll",()=>th(e),[tv(e,"replace all")])],(0,eN.Z)("button",{name:"close",onclick:()=>tm(e),"aria-label":tv(e,"close"),type:"button"},["\xd7"])])}commit(){let e=new eK({search:this.searchField.value,caseSensitive:this.caseField.checked,regexp:this.reField.checked,wholeWord:this.wordField.checked,replace:this.replaceField.value});e.eq(this.query)||(this.query=e,this.view.dispatch({effects:e6.of(e)}))}keydown(e){(0,l.$1)(this.view,e,"search-panel")?e.preventDefault():13==e.keyCode&&e.target==this.searchField?(e.preventDefault(),(e.shiftKey?to:ts)(this.view)):13==e.keyCode&&e.target==this.replaceField&&(e.preventDefault(),ta(this.view))}update(e){for(let t of e.transactions)for(let e of t.effects)e.is(e6)&&!e.value.eq(this.query)&&this.setQuery(e.value)}setQuery(e){this.query=e,this.searchField.value=e.search,this.replaceField.value=e.replace,this.caseField.checked=e.caseSensitive,this.reField.checked=e.regexp,this.wordField.checked=e.wholeWord}mount(){this.searchField.select()}get pos(){return 80}get top(){return this.view.state.facet(eH).top}}function tv(e,t){return e.state.phrase(t)}let ty=/[\s\.,:;?!]/;function tx(e,{from:t,to:n}){let i=e.state.doc.lineAt(t),r=e.state.doc.lineAt(n).to,s=Math.max(i.from,t-30),o=Math.min(r,n+30),a=e.state.sliceDoc(s,o);if(s!=i.from){for(let e=0;e<30;e++)if(!ty.test(a[e+1])&&ty.test(a[e])){a=a.slice(e);break}}if(o!=r){for(let e=a.length-1;e>a.length-30;e--)if(!ty.test(a[e-1])&&ty.test(a[e])){a=a.slice(0,e);break}}return l.tk.announce.of(`${e.state.phrase("current match")}. ${a} ${e.state.phrase("on line")} ${i.number}.`)}let tb=l.tk.baseTheme({".cm-panel.cm-search":{padding:"2px 6px 4px",position:"relative","& [name=close]":{position:"absolute",top:"0",right:"4px",backgroundColor:"inherit",border:"none",font:"inherit",padding:0,margin:0},"& input, & button, & label":{margin:".2em .6em .2em 0"},"& input[type=checkbox]":{marginRight:".2em"},"& label":{fontSize:"80%",whiteSpace:"pre"}},"&light .cm-searchMatch":{backgroundColor:"#ffff0054"},"&dark .cm-searchMatch":{backgroundColor:"#00ffff8a"},"&light .cm-searchMatch-selected":{backgroundColor:"#ff6a0054"},"&dark .cm-searchMatch-selected":{backgroundColor:"#ff00ff8a"}}),tO=[e8,o.Wl.low(ti),tb];class tw{constructor(e,t,n,i){this.state=e,this.pos=t,this.explicit=n,this.view=i,this.abortListeners=[],this.abortOnDocChange=!1}tokenBefore(e){let t=(0,a.qz)(this.state).resolveInner(this.pos,-1);for(;t&&0>e.indexOf(t.name);)t=t.parent;return t?{from:t.from,to:this.pos,text:this.state.sliceDoc(t.from,this.pos),type:t.type}:null}matchBefore(e){let t=this.state.doc.lineAt(this.pos),n=Math.max(t.from,this.pos-250),i=t.text.slice(n-t.from,this.pos-t.from),r=i.search(tA(e,!1));return r<0?null:{from:n+r,to:this.pos,text:i.slice(r)}}get aborted(){return null==this.abortListeners}addEventListener(e,t,n){"abort"==e&&this.abortListeners&&(this.abortListeners.push(t),n&&n.onDocChange&&(this.abortOnDocChange=!0))}}function tS(e){let t=Object.keys(e).join(""),n=/\w/.test(t);return n&&(t=t.replace(/\w/g,"")),`[${n?"\\w":""}${t.replace(/[^\w\s]/g,"\\$&")}]`}class tP{constructor(e,t,n,i){this.completion=e,this.source=t,this.match=n,this.score=i}}function tC(e){return e.selection.main.from}function tA(e,t){var n;let{source:i}=e,r=t&&"^"!=i[0],s="$"!=i[i.length-1];return r||s?RegExp(`${r?"^":""}(?:${i})${s?"$":""}`,null!==(n=e.flags)&&void 0!==n?n:e.ignoreCase?"i":""):e}let tT=o.q6.define(),tR=new WeakMap;function tD(e){if(!Array.isArray(e))return e;let t=tR.get(e);return t||tR.set(e,t=function(e){let t=e.map(e=>"string"==typeof e?{label:e}:e),[n,i]=t.every(e=>/^\w+$/.test(e.label))?[/\w*$/,/\w+$/]:function(e){let t=Object.create(null),n=Object.create(null);for(let{label:i}of e){t[i[0]]=!0;for(let e=1;e<i.length;e++)n[i[e]]=!0}let i=tS(t)+tS(n)+"*$";return[RegExp("^"+i),new RegExp(i)]}(t);return e=>{let r=e.matchBefore(i);return r||e.explicit?{from:r?r.from:e.pos,options:t,validFor:n}:null}}(e)),t}let tM=o.Py.define(),tI=o.Py.define();class tQ{constructor(e){this.pattern=e,this.chars=[],this.folded=[],this.any=[],this.precise=[],this.byWord=[],this.score=0,this.matched=[];for(let t=0;t<e.length;){let n=(0,o.gm)(e,t),i=(0,o.nZ)(n);this.chars.push(n);let r=e.slice(t,t+i),s=r.toUpperCase();this.folded.push((0,o.gm)(s==r?r.toLowerCase():s,0)),t+=i}this.astral=e.length!=this.chars.length}ret(e,t){return this.score=e,this.matched=t,this}match(e){if(0==this.pattern.length)return this.ret(-100,[]);if(e.length<this.pattern.length)return null;let{chars:t,folded:n,any:i,precise:r,byWord:s}=this;if(1==t.length){let i=(0,o.gm)(e,0),r=(0,o.nZ)(i),s=r==e.length?0:-100;if(i==t[0]);else{if(i!=n[0])return null;s+=-200}return this.ret(s,[0,r])}let l=e.indexOf(this.pattern);if(0==l)return this.ret(e.length==this.pattern.length?0:-100,[0,this.pattern.length]);let a=t.length,h=0;if(l<0){for(let r=0,s=Math.min(e.length,200);r<s&&h<a;){let s=(0,o.gm)(e,r);(s==t[h]||s==n[h])&&(i[h++]=r),r+=(0,o.nZ)(s)}if(h<a)return null}let c=0,u=0,f=!1,d=0,p=-1,m=-1,g=/[a-z]/.test(e),k=!0;for(let i=0,h=Math.min(e.length,200),v=0;i<h&&u<a;){let h=(0,o.gm)(e,i);l<0&&(c<a&&h==t[c]&&(r[c++]=i),d<a&&(h==t[d]||h==n[d]?(0==d&&(p=i),m=i+1,d++):d=0));let y,x=h<255?h>=48&&h<=57||h>=97&&h<=122?2:h>=65&&h<=90?1:0:(y=(0,o.bg)(h))!=y.toLowerCase()?1:y!=y.toUpperCase()?2:0;(!i||1==x&&g||0==v&&0!=x)&&(t[u]==h||n[u]==h&&(f=!0)?s[u++]=i:s.length&&(k=!1)),v=x,i+=(0,o.nZ)(h)}return u==a&&0==s[0]&&k?this.result(-100+(f?-200:0),s,e):d==a&&0==p?this.ret(-200-e.length+(m==e.length?0:-100),[0,m]):l>-1?this.ret(-700-e.length,[l,l+this.pattern.length]):d==a?this.ret(-900-e.length,[p,m]):u==a?this.result(-100+(f?-200:0)+-700+(k?0:-1100),s,e):2==t.length?null:this.result((i[0]?-700:0)+-200+-1100,i,e)}result(e,t,n){let i=[],r=0;for(let e of t){let t=e+(this.astral?(0,o.nZ)((0,o.gm)(n,e)):1);r&&i[r-1]==e?i[r-1]=t:(i[r++]=e,i[r++]=t)}return this.ret(e-n.length,i)}}class tN{constructor(e){this.pattern=e,this.matched=[],this.score=0,this.folded=e.toLowerCase()}match(e){if(e.length<this.pattern.length)return null;let t=e.slice(0,this.pattern.length),n=t==this.pattern?0:t.toLowerCase()==this.folded?-200:null;return null==n?null:(this.matched=[0,t.length],this.score=n+(e.length==this.pattern.length?0:-100),this)}}let tE=o.r$.define({combine:e=>(0,o.BO)(e,{activateOnTyping:!0,activateOnCompletion:()=>!1,activateOnTypingDelay:100,selectOnOpen:!0,override:null,closeOnBlur:!0,maxRenderedOptions:100,defaultKeymap:!0,tooltipClass:()=>"",optionClass:()=>"",aboveCursor:!1,icons:!0,addToOptions:[],positionInfo:tL,filterStrict:!1,compareCompletions:(e,t)=>(e.sortText||e.label).localeCompare(t.sortText||t.label),interactionDelay:75,updateSyncTime:100},{defaultKeymap:(e,t)=>e&&t,closeOnBlur:(e,t)=>e&&t,icons:(e,t)=>e&&t,tooltipClass:(e,t)=>n=>tB(e(n),t(n)),optionClass:(e,t)=>n=>tB(e(n),t(n)),addToOptions:(e,t)=>e.concat(t),filterStrict:(e,t)=>e||t})});function tB(e,t){return e?t?e+" "+t:e:t}function tL(e,t,n,i,r,s){let o=e.textDirection==l.Nm.RTL,a=o,h=!1,c="top",u,f,d=t.left-r.left,p=r.right-t.right,m=i.right-i.left,g=i.bottom-i.top;if(a&&d<Math.min(m,p)?a=!1:!a&&p<Math.min(m,d)&&(a=!0),m<=(a?d:p))u=Math.max(r.top,Math.min(n.top,r.bottom-g))-t.top,f=Math.min(400,a?d:p);else{h=!0,f=Math.min(400,(o?t.right:r.right-t.left)-30);let e=r.bottom-t.bottom;e>=g||e>t.top?u=n.bottom-t.top:(c="bottom",u=t.bottom-n.top)}let k=(t.bottom-t.top)/s.offsetHeight,v=(t.right-t.left)/s.offsetWidth;return{style:`${c}: ${u/k}px; max-width: ${f/v}px`,class:"cm-completionInfo-"+(h?o?"left-narrow":"right-narrow":a?"left":"right")}}function t$(e,t,n){if(e<=n)return{from:0,to:e};if(t<0&&(t=0),t<=e>>1){let e=Math.floor(t/n);return{from:e*n,to:(e+1)*n}}let i=Math.floor((e-t)/n);return{from:e-(i+1)*n,to:e-i*n}}class tj{constructor(e,t,n){let i;this.view=e,this.stateField=t,this.applyCompletion=n,this.info=null,this.infoDestroy=null,this.placeInfoReq={read:()=>this.measureInfo(),write:e=>this.placeInfo(e),key:this},this.space=null,this.currentClass="";let r=e.state.field(t),{options:s,selected:o}=r.open,l=e.state.facet(tE);this.optionContent=(i=l.addToOptions.slice(),l.icons&&i.push({render(e){let t=document.createElement("div");return t.classList.add("cm-completionIcon"),e.type&&t.classList.add(...e.type.split(/\s+/g).map(e=>"cm-completionIcon-"+e)),t.setAttribute("aria-hidden","true"),t},position:20}),i.push({render(e,t,n,i){let r=document.createElement("span");r.className="cm-completionLabel";let s=e.displayLabel||e.label,o=0;for(let e=0;e<i.length;){let t=i[e++],n=i[e++];t>o&&r.appendChild(document.createTextNode(s.slice(o,t)));let l=r.appendChild(document.createElement("span"));l.appendChild(document.createTextNode(s.slice(t,n))),l.className="cm-completionMatchedText",o=n}return o<s.length&&r.appendChild(document.createTextNode(s.slice(o))),r},position:50},{render(e){if(!e.detail)return null;let t=document.createElement("span");return t.className="cm-completionDetail",t.textContent=e.detail,t},position:80}),i.sort((e,t)=>e.position-t.position).map(e=>e.render)),this.optionClass=l.optionClass,this.tooltipClass=l.tooltipClass,this.range=t$(s.length,o,l.maxRenderedOptions),this.dom=document.createElement("div"),this.dom.className="cm-tooltip-autocomplete",this.updateTooltipClass(e.state),this.dom.addEventListener("mousedown",n=>{let{options:i}=e.state.field(t).open;for(let t=n.target,r;t&&t!=this.dom;t=t.parentNode)if("LI"==t.nodeName&&(r=/-(\d+)$/.exec(t.id))&&+r[1]<i.length){this.applyCompletion(e,i[+r[1]]),n.preventDefault();return}}),this.dom.addEventListener("focusout",t=>{let n=e.state.field(this.stateField,!1);n&&n.tooltip&&e.state.facet(tE).closeOnBlur&&t.relatedTarget!=e.contentDOM&&e.dispatch({effects:tI.of(null)})}),this.showOptions(s,r.id)}mount(){this.updateSel()}showOptions(e,t){this.list&&this.list.remove(),this.list=this.dom.appendChild(this.createListBox(e,t,this.range)),this.list.addEventListener("scroll",()=>{this.info&&this.view.requestMeasure(this.placeInfoReq)})}update(e){var t;let n=e.state.field(this.stateField),i=e.startState.field(this.stateField);if(this.updateTooltipClass(e.state),n!=i){let{options:r,selected:s,disabled:o}=n.open;i.open&&i.open.options==r||(this.range=t$(r.length,s,e.state.facet(tE).maxRenderedOptions),this.showOptions(r,n.id)),this.updateSel(),o!=(null===(t=i.open)||void 0===t?void 0:t.disabled)&&this.dom.classList.toggle("cm-tooltip-autocomplete-disabled",!!o)}}updateTooltipClass(e){let t=this.tooltipClass(e);if(t!=this.currentClass){for(let e of this.currentClass.split(" "))e&&this.dom.classList.remove(e);for(let e of t.split(" "))e&&this.dom.classList.add(e);this.currentClass=t}}positioned(e){this.space=e,this.info&&this.view.requestMeasure(this.placeInfoReq)}updateSel(){let e=this.view.state.field(this.stateField),t=e.open;(t.selected>-1&&t.selected<this.range.from||t.selected>=this.range.to)&&(this.range=t$(t.options.length,t.selected,this.view.state.facet(tE).maxRenderedOptions),this.showOptions(t.options,e.id));let n=this.updateSelectedOption(t.selected);if(n){this.destroyInfo();let{completion:i}=t.options[t.selected],{info:r}=i;if(!r)return;let s="string"==typeof r?document.createTextNode(r):r(i);if(!s)return;"then"in s?s.then(t=>{t&&this.view.state.field(this.stateField,!1)==e&&this.addInfoPane(t,i)}).catch(e=>(0,l.OO)(this.view.state,e,"completion info")):(this.addInfoPane(s,i),n.setAttribute("aria-describedby",this.info.id))}}addInfoPane(e,t){this.destroyInfo();let n=this.info=document.createElement("div");if(n.className="cm-tooltip cm-completionInfo",n.id="cm-completionInfo-"+Math.floor(65535*Math.random()).toString(16),null!=e.nodeType)n.appendChild(e),this.infoDestroy=null;else{let{dom:t,destroy:i}=e;n.appendChild(t),this.infoDestroy=i||null}this.dom.appendChild(n),this.view.requestMeasure(this.placeInfoReq)}updateSelectedOption(e){var t,n;let i,r,s,o=null;for(let t=this.list.firstChild,n=this.range.from;t;t=t.nextSibling,n++)"LI"==t.nodeName&&t.id?n==e?t.hasAttribute("aria-selected")||(t.setAttribute("aria-selected","true"),o=t):t.hasAttribute("aria-selected")&&(t.removeAttribute("aria-selected"),t.removeAttribute("aria-describedby")):n--;return o&&(t=this.list,n=o,i=t.getBoundingClientRect(),r=n.getBoundingClientRect(),s=i.height/t.offsetHeight,r.top<i.top?t.scrollTop-=(i.top-r.top)/s:r.bottom>i.bottom&&(t.scrollTop+=(r.bottom-i.bottom)/s)),o}measureInfo(){let e=this.dom.querySelector("[aria-selected]");if(!e||!this.info)return null;let t=this.dom.getBoundingClientRect(),n=this.info.getBoundingClientRect(),i=e.getBoundingClientRect(),r=this.space;if(!r){let e=this.dom.ownerDocument.documentElement;r={left:0,top:0,right:e.clientWidth,bottom:e.clientHeight}}return i.top>Math.min(r.bottom,t.bottom)-10||i.bottom<Math.max(r.top,t.top)+10?null:this.view.state.facet(tE).positionInfo(this.view,t,i,n,r,this.dom)}placeInfo(e){this.info&&(e?(e.style&&(this.info.style.cssText=e.style),this.info.className="cm-tooltip cm-completionInfo "+(e.class||"")):this.info.style.cssText="top: -1e6px")}createListBox(e,t,n){let i=document.createElement("ul");i.id=t,i.setAttribute("role","listbox"),i.setAttribute("aria-expanded","true"),i.setAttribute("aria-label",this.view.state.phrase("Completions")),i.addEventListener("mousedown",e=>{e.target==i&&e.preventDefault()});let r=null;for(let s=n.from;s<n.to;s++){let{completion:o,match:l}=e[s],{section:a}=o;if(a){let e="string"==typeof a?a:a.name;e!=r&&(s>n.from||0==n.from)&&(r=e,"string"!=typeof a&&a.header?i.appendChild(a.header(a)):i.appendChild(document.createElement("completion-section")).textContent=e)}let h=i.appendChild(document.createElement("li"));h.id=t+"-"+s,h.setAttribute("role","option");let c=this.optionClass(o);for(let e of(c&&(h.className=c),this.optionContent)){let t=e(o,this.view.state,this.view,l);t&&h.appendChild(t)}}return n.from&&i.classList.add("cm-completionListIncompleteTop"),n.to<e.length&&i.classList.add("cm-completionListIncompleteBottom"),i}destroyInfo(){this.info&&(this.infoDestroy&&this.infoDestroy(),this.info.remove(),this.info=null)}destroy(){this.destroyInfo()}}function tz(e){return 100*(e.boost||0)+(e.apply?10:0)+(e.info?5:0)+(e.type?1:0)}class tX{constructor(e,t,n,i,r,s){this.options=e,this.attrs=t,this.tooltip=n,this.timestamp=i,this.selected=r,this.disabled=s}setSelected(e,t){return e==this.selected||e>=this.options.length?this:new tX(this.options,t_(t,e),this.tooltip,this.timestamp,e,this.disabled)}static build(e,t,n,i,r,s){if(i&&!s&&e.some(e=>e.isPending))return i.setDisabled();let o=function(e,t){let n=[],i=null,r=null,s=e=>{n.push(e);let{section:t}=e.completion;if(t){i||(i=[]);let e="string"==typeof t?t:t.name;i.some(t=>t.name==e)||i.push("string"==typeof t?{name:e}:t)}},o=t.facet(tE);for(let i of e)if(i.hasResult()){let e=i.result.getMatch;if(!1===i.result.filter)for(let t of i.result.options)s(new tP(t,i.source,e?e(t):[],1e9-n.length));else{let n=t.sliceDoc(i.from,i.to),l,a=o.filterStrict?new tN(n):new tQ(n);for(let t of i.result.options)if(l=a.match(t.label)){let n=t.displayLabel?e?e(t,l.matched):[]:l.matched,o=l.score+(t.boost||0);if(s(new tP(t,i.source,n,o)),"object"==typeof t.section&&"dynamic"===t.section.rank){let{name:e}=t.section;r||(r=Object.create(null)),r[e]=Math.max(o,r[e]||-1e9)}}}}if(i){let e=Object.create(null),t=0;for(let n of i.sort((e,t)=>("dynamic"===e.rank&&"dynamic"===t.rank?r[t.name]-r[e.name]:0)||("number"==typeof e.rank?e.rank:1e9)-("number"==typeof t.rank?t.rank:1e9)||(e.name<t.name?-1:1)))t-=1e5,e[n.name]=t;for(let t of n){let{section:n}=t.completion;n&&(t.score+=e["string"==typeof n?n:n.name])}}let l=[],a=null,h=o.compareCompletions;for(let e of n.sort((e,t)=>t.score-e.score||h(e.completion,t.completion))){let t=e.completion;a&&a.label==t.label&&a.detail==t.detail&&(null==a.type||null==t.type||a.type==t.type)&&a.apply==t.apply&&a.boost==t.boost?tz(e.completion)>tz(a)&&(l[l.length-1]=e):l.push(e),a=e.completion}return l}(e,t);if(!o.length)return i&&e.some(e=>e.isPending)?i.setDisabled():null;let l=t.facet(tE).selectOnOpen?0:-1;if(i&&i.selected!=l&&-1!=i.selected){let e=i.options[i.selected].completion;for(let t=0;t<o.length;t++)if(o[t].completion==e){l=t;break}}return new tX(o,t_(n,l),{pos:e.reduce((e,t)=>t.hasResult()?Math.min(e,t.from):e,1e8),create:t0,above:r.aboveCursor},i?i.timestamp:Date.now(),l,!1)}map(e){return new tX(this.options,this.attrs,{...this.tooltip,pos:e.mapPos(this.tooltip.pos)},this.timestamp,this.selected,this.disabled)}setDisabled(){return new tX(this.options,this.attrs,this.tooltip,this.timestamp,this.selected,!0)}}class tq{constructor(e,t,n){this.active=e,this.id=t,this.open=n}static start(){return new tq(tF,"cm-ac-"+Math.floor(2e6*Math.random()).toString(36),null)}update(e){let{state:t}=e,n=t.facet(tE),i=(n.override||t.languageDataAt("autocomplete",tC(t)).map(tD)).map(t=>(this.active.find(e=>e.source==t)||new tV(t,this.active.some(e=>0!=e.state)?1:0)).update(e,n));i.length==this.active.length&&i.every((e,t)=>e==this.active[t])&&(i=this.active);let r=this.open,s=e.effects.some(e=>e.is(tG));for(let o of(r&&e.docChanged&&(r=r.map(e.changes)),e.selection||i.some(t=>t.hasResult()&&e.changes.touchesRange(t.from,t.to))||!function(e,t){if(e==t)return!0;for(let n=0,i=0;;){for(;n<e.length&&!e[n].hasResult();)n++;for(;i<t.length&&!t[i].hasResult();)i++;let r=n==e.length,s=i==t.length;if(r||s)return r==s;if(e[n++].result!=t[i++].result)return!1}}(i,this.active)||s?r=tX.build(i,t,this.id,r,n,s):r&&r.disabled&&!i.some(e=>e.isPending)&&(r=null),!r&&i.every(e=>!e.isPending)&&i.some(e=>e.hasResult())&&(i=i.map(e=>e.hasResult()?new tV(e.source,0):e)),e.effects))o.is(tY)&&(r=r&&r.setSelected(o.value,this.id));return i==this.active&&r==this.open?this:new tq(i,this.id,r)}get tooltip(){return this.open?this.open.tooltip:null}get attrs(){return this.open?this.open.attrs:this.active.length?tJ:tW}}let tJ={"aria-autocomplete":"list"},tW={};function t_(e,t){let n={"aria-autocomplete":"list","aria-haspopup":"listbox","aria-controls":e};return t>-1&&(n["aria-activedescendant"]=e+"-"+t),n}let tF=[];function tU(e,t){if(e.isUserEvent("input.complete")){let n=e.annotation(tT);if(n&&t.activateOnCompletion(n))return 12}let n=e.isUserEvent("input.type");return n&&t.activateOnTyping?5:n?1:e.isUserEvent("delete.backward")?2:e.selection?8:e.docChanged?16:0}class tV{constructor(e,t,n=!1){this.source=e,this.state=t,this.explicit=n}hasResult(){return!1}get isPending(){return 1==this.state}update(e,t){let n=tU(e,t),i=this;for(let t of((8&n||16&n&&this.touches(e))&&(i=new tV(i.source,0)),4&n&&0==i.state&&(i=new tV(this.source,1)),i=i.updateFor(e,n),e.effects))if(t.is(tM))i=new tV(i.source,1,t.value);else if(t.is(tI))i=new tV(i.source,0);else if(t.is(tG))for(let e of t.value)e.source==i.source&&(i=e);return i}updateFor(e,t){return this.map(e.changes)}map(e){return this}touches(e){return e.changes.touchesRange(tC(e.state))}}class tZ extends tV{constructor(e,t,n,i,r,s){super(e,3,t),this.limit=n,this.result=i,this.from=r,this.to=s}hasResult(){return!0}updateFor(e,t){var n;if(!(3&t))return this.map(e.changes);let i=this.result;i.map&&!e.changes.empty&&(i=i.map(i,e.changes));let r=e.changes.mapPos(this.from),s=e.changes.mapPos(this.to,1),o=tC(e.state);if(o>s||!i||2&t&&(tC(e.startState)==this.from||o<this.limit))return new tV(this.source,4&t?1:0);let l=e.changes.mapPos(this.limit);return!function(e,t,n,i){if(!e)return!1;let r=t.sliceDoc(n,i);return"function"==typeof e?e(r,n,i,t):tA(e,!0).test(r)}(i.validFor,e.state,r,s)?i.update&&(i=i.update(i,r,s,new tw(e.state,o,!1)))?new tZ(this.source,this.explicit,l,i,i.from,null!==(n=i.to)&&void 0!==n?n:tC(e.state)):new tV(this.source,1,this.explicit):new tZ(this.source,this.explicit,l,i,r,s)}map(e){return e.empty?this:(this.result.map?this.result.map(this.result,e):this.result)?new tZ(this.source,this.explicit,e.mapPos(this.limit),this.result,e.mapPos(this.from),e.mapPos(this.to,1)):new tV(this.source,0)}touches(e){return e.changes.touchesRange(this.from,this.to)}}let tG=o.Py.define({map:(e,t)=>e.map(e=>e.map(t))}),tY=o.Py.define(),tH=o.QQ.define({create:()=>tq.start(),update:(e,t)=>e.update(t),provide:e=>[l.hJ.from(e,e=>e.tooltip),l.tk.contentAttributes.from(e,e=>e.attrs)]});function tK(e,t){let n=t.completion.apply||t.completion.label,i=e.state.field(tH).active.find(e=>e.source==t.source);return i instanceof tZ&&("string"==typeof n?e.dispatch({...function(e,t,n,i){let{main:r}=e.selection,s=n-r.from,l=i-r.from;return{...e.changeByRange(a=>{if(a!=r&&n!=i&&e.sliceDoc(a.from+s,a.from+l)!=e.sliceDoc(n,i))return{range:a};let h=e.toText(t);return{changes:{from:a.from+s,to:i==r.from?a.to:a.from+l,insert:h},range:o.jT.cursor(a.from+s+h.length)}}),scrollIntoView:!0,userEvent:"input.complete"}}(e.state,n,i.from,i.to),annotations:tT.of(t.completion)}):n(e,t.completion,i.from,i.to),!0)}let t0=e=>new tj(e,tH,tK);function t1(e,t="option"){return n=>{let i=n.state.field(tH,!1);if(!i||!i.open||i.open.disabled||Date.now()-i.open.timestamp<n.state.facet(tE).interactionDelay)return!1;let r=1,s;"page"==t&&(s=(0,l.gB)(n,i.open.tooltip))&&(r=Math.max(2,Math.floor(s.dom.offsetHeight/s.dom.querySelector("li").offsetHeight)-1));let{length:o}=i.open.options,a=i.open.selected>-1?i.open.selected+r*(e?1:-1):e?0:o-1;return a<0?a="page"==t?0:o-1:a>=o&&(a="page"==t?o-1:0),n.dispatch({effects:tY.of(a)}),!0}}let t2=e=>!!e.state.field(tH,!1)&&(e.dispatch({effects:tM.of(!0)}),!0);class t5{constructor(e,t){this.active=e,this.context=t,this.time=Date.now(),this.updates=[],this.done=void 0}}let t3=l.lg.fromClass(class{constructor(e){for(let t of(this.view=e,this.debounceUpdate=-1,this.running=[],this.debounceAccept=-1,this.pendingStart=!1,this.composing=0,e.state.field(tH).active))t.isPending&&this.startQuery(t)}update(e){let t=e.state.field(tH),n=e.state.facet(tE);if(!e.selectionSet&&!e.docChanged&&e.startState.field(tH)==t)return;let i=e.transactions.some(e=>{let t=tU(e,n);return 8&t||(e.selection||e.docChanged)&&!(3&t)});for(let t=0;t<this.running.length;t++){let n=this.running[t];if(i||n.context.abortOnDocChange&&e.docChanged||n.updates.length+e.transactions.length>50&&Date.now()-n.time>1e3){for(let e of n.context.abortListeners)try{e()}catch(e){(0,l.OO)(this.view.state,e)}n.context.abortListeners=null,this.running.splice(t--,1)}else n.updates.push(...e.transactions)}this.debounceUpdate>-1&&clearTimeout(this.debounceUpdate),e.transactions.some(e=>e.effects.some(e=>e.is(tM)))&&(this.pendingStart=!0);let r=this.pendingStart?50:n.activateOnTypingDelay;if(this.debounceUpdate=t.active.some(e=>e.isPending&&!this.running.some(t=>t.active.source==e.source))?setTimeout(()=>this.startUpdate(),r):-1,0!=this.composing)for(let t of e.transactions)t.isUserEvent("input.type")?this.composing=2:2==this.composing&&t.selection&&(this.composing=3)}startUpdate(){this.debounceUpdate=-1,this.pendingStart=!1;let{state:e}=this.view,t=e.field(tH);for(let e of t.active)e.isPending&&!this.running.some(t=>t.active.source==e.source)&&this.startQuery(e);this.running.length&&t.open&&t.open.disabled&&(this.debounceAccept=setTimeout(()=>this.accept(),this.view.state.facet(tE).updateSyncTime))}startQuery(e){let{state:t}=this.view,n=tC(t),i=new tw(t,n,e.explicit,this.view),r=new t5(e,i);this.running.push(r),Promise.resolve(e.source(i)).then(e=>{r.context.aborted||(r.done=e||null,this.scheduleAccept())},e=>{this.view.dispatch({effects:tI.of(null)}),(0,l.OO)(this.view.state,e)})}scheduleAccept(){this.running.every(e=>void 0!==e.done)?this.accept():this.debounceAccept<0&&(this.debounceAccept=setTimeout(()=>this.accept(),this.view.state.facet(tE).updateSyncTime))}accept(){var e;this.debounceAccept>-1&&clearTimeout(this.debounceAccept),this.debounceAccept=-1;let t=[],n=this.view.state.facet(tE),i=this.view.state.field(tH);for(let r=0;r<this.running.length;r++){let s=this.running[r];if(void 0===s.done)continue;if(this.running.splice(r--,1),s.done){let i=tC(s.updates.length?s.updates[0].startState:this.view.state),r=Math.min(i,s.done.from+(s.active.explicit?0:1)),o=new tZ(s.active.source,s.active.explicit,r,s.done,s.done.from,null!==(e=s.done.to)&&void 0!==e?e:i);for(let e of s.updates)o=o.update(e,n);if(o.hasResult()){t.push(o);continue}}let o=i.active.find(e=>e.source==s.active.source);if(o&&o.isPending){if(null==s.done){let e=new tV(s.active.source,0);for(let t of s.updates)e=e.update(t,n);e.isPending||t.push(e)}else this.startQuery(o)}}(t.length||i.open&&i.open.disabled)&&this.view.dispatch({effects:tG.of(t)})}},{eventHandlers:{blur(e){let t=this.view.state.field(tH,!1);if(t&&t.tooltip&&this.view.state.facet(tE).closeOnBlur){let n=t.open&&(0,l.gB)(this.view,t.open.tooltip);n&&n.dom.contains(e.relatedTarget)||setTimeout(()=>this.view.dispatch({effects:tI.of(null)}),10)}},compositionstart(){this.composing=1},compositionend(){3==this.composing&&setTimeout(()=>this.view.dispatch({effects:tM.of(!1)}),20),this.composing=0}}}),t4="object"==typeof navigator&&/Win/.test(navigator.platform),t9=o.Wl.highest(l.tk.domEventHandlers({keydown(e,t){let n=t.state.field(tH,!1);if(!n||!n.open||n.open.disabled||n.open.selected<0||e.key.length>1||e.ctrlKey&&!(t4&&e.altKey)||e.metaKey)return!1;let i=n.open.options[n.open.selected],r=n.active.find(e=>e.source==i.source),s=i.completion.commitCharacters||r.result.commitCharacters;return s&&s.indexOf(e.key)>-1&&tK(t,i),!1}})),t6=l.tk.baseTheme({".cm-tooltip.cm-tooltip-autocomplete":{"& > ul":{fontFamily:"monospace",whiteSpace:"nowrap",overflow:"hidden auto",maxWidth_fallback:"700px",maxWidth:"min(700px, 95vw)",minWidth:"250px",maxHeight:"10em",height:"100%",listStyle:"none",margin:0,padding:0,"& > li, & > completion-section":{padding:"1px 3px",lineHeight:1.2},"& > li":{overflowX:"hidden",textOverflow:"ellipsis",cursor:"pointer"},"& > completion-section":{display:"list-item",borderBottom:"1px solid silver",paddingLeft:"0.5em",opacity:.7}}},"&light .cm-tooltip-autocomplete ul li[aria-selected]":{background:"#17c",color:"white"},"&light .cm-tooltip-autocomplete-disabled ul li[aria-selected]":{background:"#777"},"&dark .cm-tooltip-autocomplete ul li[aria-selected]":{background:"#347",color:"white"},"&dark .cm-tooltip-autocomplete-disabled ul li[aria-selected]":{background:"#444"},".cm-completionListIncompleteTop:before, .cm-completionListIncompleteBottom:after":{content:'"\xb7\xb7\xb7"',opacity:.5,display:"block",textAlign:"center"},".cm-tooltip.cm-completionInfo":{position:"absolute",padding:"3px 9px",width:"max-content",maxWidth:"400px",boxSizing:"border-box",whiteSpace:"pre-line"},".cm-completionInfo.cm-completionInfo-left":{right:"100%"},".cm-completionInfo.cm-completionInfo-right":{left:"100%"},".cm-completionInfo.cm-completionInfo-left-narrow":{right:"30px"},".cm-completionInfo.cm-completionInfo-right-narrow":{left:"30px"},"&light .cm-snippetField":{backgroundColor:"#00000022"},"&dark .cm-snippetField":{backgroundColor:"#ffffff22"},".cm-snippetFieldPosition":{verticalAlign:"text-top",width:0,height:"1.15em",display:"inline-block",margin:"0 -0.7px -.7em",borderLeft:"1.4px dotted #888"},".cm-completionMatchedText":{textDecoration:"underline"},".cm-completionDetail":{marginLeft:"0.5em",fontStyle:"italic"},".cm-completionIcon":{fontSize:"90%",width:".8em",display:"inline-block",textAlign:"center",paddingRight:".6em",opacity:"0.6",boxSizing:"content-box"},".cm-completionIcon-function, .cm-completionIcon-method":{"&:after":{content:"'ƒ'"}},".cm-completionIcon-class":{"&:after":{content:"'○'"}},".cm-completionIcon-interface":{"&:after":{content:"'◌'"}},".cm-completionIcon-variable":{"&:after":{content:"'\uD835\uDC65'"}},".cm-completionIcon-constant":{"&:after":{content:"'\uD835\uDC36'"}},".cm-completionIcon-type":{"&:after":{content:"'\uD835\uDC61'"}},".cm-completionIcon-enum":{"&:after":{content:"'∪'"}},".cm-completionIcon-property":{"&:after":{content:"'□'"}},".cm-completionIcon-keyword":{"&:after":{content:"'\uD83D\uDD11︎'"}},".cm-completionIcon-namespace":{"&:after":{content:"'▢'"}},".cm-completionIcon-text":{"&:after":{content:"'abc'",fontSize:"50%",verticalAlign:"middle"}}}),t7=l.p.widget({widget:new class extends l.l9{toDOM(){let e=document.createElement("span");return e.className="cm-snippetFieldPosition",e}ignoreEvent(){return!1}}}),t8=l.p.mark({class:"cm-snippetField"});class ne{constructor(e,t){this.ranges=e,this.active=t,this.deco=l.p.set(e.map(e=>(e.from==e.to?t7:t8).range(e.from,e.to)),!0)}map(e){let t=[];for(let n of this.ranges){let i=n.map(e);if(!i)return null;t.push(i)}return new ne(t,this.active)}selectionInsideField(e){return e.ranges.every(e=>this.ranges.some(t=>t.field==this.active&&t.from<=e.from&&t.to>=e.to))}}let nt=o.Py.define({map:(e,t)=>e&&e.map(t)}),nn=o.Py.define(),ni=o.QQ.define({create:()=>null,update(e,t){for(let n of t.effects){if(n.is(nt))return n.value;if(n.is(nn)&&e)return new ne(e.ranges,n.value)}return e&&t.docChanged&&(e=e.map(t.changes)),e&&t.selection&&!e.selectionInsideField(t.selection)&&(e=null),e},provide:e=>l.tk.decorations.from(e,e=>e?e.deco:l.p.none)});function nr(e){return({state:t,dispatch:n})=>{var i;let r=t.field(ni,!1);if(!r||e<0&&0==r.active)return!1;let s=r.active+e,l=e>0&&!r.ranges.some(t=>t.field==s+e);return n(t.update({selection:(i=r.ranges,o.jT.create(i.filter(e=>e.field==s).map(e=>o.jT.range(e.from,e.to)))),effects:nt.of(l?null:new ne(r.ranges,s)),scrollIntoView:!0})),!0}}({state:e,dispatch:t})=>!!e.field(ni,!1)&&(t(e.update({effects:nt.of(null)})),!0);let ns={brackets:["(","[","{","'",'"'],before:")]}:;>",stringPrefixes:[]},no=o.Py.define({map(e,t){let n=t.mapPos(e,-1,o.gc.TrackAfter);return null==n?void 0:n}}),nl=new class extends o.uU{};nl.startSide=1,nl.endSide=-1;let na=o.QQ.define({create:()=>o.Xs.empty,update(e,t){if(e=e.map(t.changes),t.selection){let n=t.state.doc.lineAt(t.selection.main.head);e=e.update({filter:e=>e>=n.from&&e<=n.to})}for(let n of t.effects)n.is(no)&&(e=e.update({add:[nl.range(n.value,n.value+1)]}));return e}}),nh="()[]{}<>\xab\xbb\xbb\xab［］｛｝";function nc(e){for(let t=0;t<nh.length;t+=2)if(nh.charCodeAt(t)==e)return nh.charAt(t+1);return(0,o.bg)(e<128?e:e+1)}function nu(e,t){return e.languageDataAt("closeBrackets",t)[0]||ns}let nf="object"==typeof navigator&&/Android\b/.test(navigator.userAgent),nd=l.tk.inputHandler.of((e,t,n,i)=>{if((nf?e.composing:e.compositionStarted)||e.state.readOnly)return!1;let r=e.state.selection.main;if(i.length>2||2==i.length&&1==(0,o.nZ)((0,o.gm)(i,0))||t!=r.from||n!=r.to)return!1;let s=function(e,t){let n=nu(e,e.selection.main.head),i=n.brackets||ns.brackets;for(let r of i){let s=nc((0,o.gm)(r,0));if(t==r)return s==r?function(e,t,n,i){let r=i.stringPrefixes||ns.stringPrefixes,s=null,l=e.changeByRange(i=>{if(!i.empty)return{changes:[{insert:t,from:i.from},{insert:t,from:i.to}],effects:no.of(i.to+t.length),range:o.jT.range(i.anchor+t.length,i.head+t.length)};let l=i.head,h=ng(e.doc,l),c;if(h==t){if(nk(e,l))return{changes:{insert:t+t,from:l},effects:no.of(l+t.length),range:o.jT.cursor(l+t.length)};if(nm(e,l)){let i=n&&e.sliceDoc(l,l+3*t.length)==t+t+t?t+t+t:t;return{changes:{from:l,to:l+i.length,insert:i},range:o.jT.cursor(l+i.length)}}}else if(n&&e.sliceDoc(l-2*t.length,l)==t+t&&(c=nv(e,l-2*t.length,r))>-1&&nk(e,c))return{changes:{insert:t+t+t+t,from:l},effects:no.of(l+t.length),range:o.jT.cursor(l+t.length)};else if(e.charCategorizer(l)(h)!=o.D0.Word&&nv(e,l,r)>-1&&!function(e,t,n,i){let r=(0,a.qz)(e).resolveInner(t,-1),s=i.reduce((e,t)=>Math.max(e,t.length),0);for(let o=0;o<5;o++){let o=e.sliceDoc(r.from,Math.min(r.to,r.from+n.length+s)),l=o.indexOf(n);if(!l||l>-1&&i.indexOf(o.slice(0,l))>-1){let t=r.firstChild;for(;t&&t.from==r.from&&t.to-t.from>n.length+l;){if(e.sliceDoc(t.to-n.length,t.to)==n)return!1;t=t.firstChild}return!0}let a=r.to==t&&r.parent;if(!a)break;r=a}return!1}(e,l,t,r))return{changes:{insert:t+t,from:l},effects:no.of(l+t.length),range:o.jT.cursor(l+t.length)};return{range:s=i}});return s?null:e.update(l,{scrollIntoView:!0,userEvent:"input.type"})}(e,r,i.indexOf(r+r+r)>-1,n):function(e,t,n,i){let r=null,s=e.changeByRange(s=>{if(!s.empty)return{changes:[{insert:t,from:s.from},{insert:n,from:s.to}],effects:no.of(s.to+t.length),range:o.jT.range(s.anchor+t.length,s.head+t.length)};let l=ng(e.doc,s.head);return!l||/\s/.test(l)||i.indexOf(l)>-1?{changes:{insert:t+n,from:s.head},effects:no.of(s.head+t.length),range:o.jT.cursor(s.head+t.length)}:{range:r=s}});return r?null:e.update(s,{scrollIntoView:!0,userEvent:"input.type"})}(e,r,s,n.before||ns.before);if(t==s&&nm(e,e.selection.main.from))return function(e,t,n){let i=null,r=e.changeByRange(t=>t.empty&&ng(e.doc,t.head)==n?{changes:{from:t.head,to:t.head+n.length,insert:n},range:o.jT.cursor(t.head+n.length)}:i={range:t});return i?null:e.update(r,{scrollIntoView:!0,userEvent:"input.type"})}(e,0,s)}return null}(e.state,i);return!!s&&(e.dispatch(s),!0)}),np=[{key:"Backspace",run:({state:e,dispatch:t})=>{if(e.readOnly)return!1;let n=nu(e,e.selection.main.head).brackets||ns.brackets,i=null,r=e.changeByRange(t=>{if(t.empty){var r,s;let i;let l=(r=e.doc,s=t.head,i=r.sliceString(s-2,s),(0,o.nZ)((0,o.gm)(i,0))==i.length?i:i.slice(1));for(let i of n)if(i==l&&ng(e.doc,t.head)==nc((0,o.gm)(i,0)))return{changes:{from:t.head-i.length,to:t.head+i.length},range:o.jT.cursor(t.head-i.length)}}return{range:i=t}});return i||t(e.update(r,{scrollIntoView:!0,userEvent:"delete.backward"})),!i}}];function nm(e,t){let n=!1;return e.field(na).between(0,e.doc.length,e=>{e==t&&(n=!0)}),n}function ng(e,t){let n=e.sliceString(t,t+2);return n.slice(0,(0,o.nZ)((0,o.gm)(n,0)))}function nk(e,t){let n=(0,a.qz)(e).resolveInner(t+1);return n.parent&&n.from==t}function nv(e,t,n){let i=e.charCategorizer(t);if(i(e.sliceDoc(t-1,t))!=o.D0.Word)return t;for(let r of n){let n=t-r.length;if(e.sliceDoc(n,t)==r&&i(e.sliceDoc(n-1,n))!=o.D0.Word)return n}return -1}let ny=[{key:"Ctrl-Space",run:t2},{mac:"Alt-`",run:t2},{mac:"Alt-i",run:t2},{key:"Escape",run:e=>{let t=e.state.field(tH,!1);return!!(t&&t.active.some(e=>0!=e.state))&&(e.dispatch({effects:tI.of(null)}),!0)}},{key:"ArrowDown",run:t1(!0)},{key:"ArrowUp",run:t1(!1)},{key:"PageDown",run:t1(!0,"page")},{key:"PageUp",run:t1(!1,"page")},{key:"Enter",run:e=>{let t=e.state.field(tH,!1);return!(e.state.readOnly||!t||!t.open||t.open.selected<0||t.open.disabled||Date.now()-t.open.timestamp<e.state.facet(tE).interactionDelay)&&tK(e,t.open.options[t.open.selected])}}],nx=o.Wl.highest(l.$f.computeN([tE],e=>e.facet(tE).defaultKeymap?[ny]:[]));class nb{constructor(e,t,n){this.from=e,this.to=t,this.diagnostic=n}}class nO{constructor(e,t,n){this.diagnostics=e,this.panel=t,this.selected=n}static init(e,t,n){let i=n.facet(nI).markerFilter;i&&(e=i(e,n));let r=e.slice().sort((e,t)=>e.from-t.from||e.to-t.to),s=new o.f_,a=[],h=0,c=n.doc.iter(),u=0,f=n.doc.length;for(let e=0;;){let t,n,i=e==r.length?null:r[e];if(!i&&!a.length)break;if(a.length)t=h,n=a.reduce((e,t)=>Math.min(e,t.to),i&&i.from>t?i.from:1e8);else{if((t=i.from)>f)break;n=i.to,a.push(i),e++}for(;e<r.length;){let i=r[e];if(i.from==t&&(i.to>i.from||i.to==t))a.push(i),e++,n=Math.min(i.to,n);else{n=Math.min(i.from,n);break}}n=Math.min(n,f);let o=!1;if(a.some(e=>e.from==t&&(e.to==n||n==f))&&!(o=t==n)&&n-t<10){let e=t-(u+c.value.length);e>0&&(c.next(e),u=t);for(let e=t;;){if(e>=n){o=!0;break}if(!c.lineBreak&&u+c.value.length>e)break;e=u+c.value.length,u+=c.value.length,c.next()}}let d=function(e){let t="hint",n=1;for(let r of e){var i;let e="error"==(i=r.severity)?4:"warning"==i?3:"info"==i?2:1;e>n&&(n=e,t=r.severity)}return t}(a);if(o)s.add(t,t,l.p.widget({widget:new nB(d),diagnostics:a.slice()}));else{let e=a.reduce((e,t)=>t.markClass?e+" "+t.markClass:e,"");s.add(t,n,l.p.mark({class:"cm-lintRange cm-lintRange-"+d+e,diagnostics:a.slice(),inclusiveEnd:a.some(e=>e.to>n)}))}if((h=n)==f)break;for(let e=0;e<a.length;e++)a[e].to<=h&&a.splice(e--,1)}let d=s.finish();return new nO(d,t,nw(d))}}function nw(e,t=null,n=0){let i=null;return e.between(n,1e9,(e,n,{spec:r})=>{if(!(t&&0>r.diagnostics.indexOf(t))){if(i){if(0>r.diagnostics.indexOf(i.diagnostic))return!1;i=new nb(i.from,n,i.diagnostic)}else i=new nb(e,n,t||r.diagnostics[0])}}),i}function nS(e,t){let n=t.pos,i=t.end||n,r=e.state.facet(nI).hideOn(e,n,i);if(null!=r)return r;let s=e.startState.doc.lineAt(t.pos);return!!(e.effects.some(e=>e.is(nP))||e.changes.touchesRange(s.from,Math.max(s.to,i)))}let nP=o.Py.define(),nC=o.Py.define(),nA=o.Py.define(),nT=o.QQ.define({create:()=>new nO(l.p.none,null,null),update(e,t){if(t.docChanged&&e.diagnostics.size){let n=e.diagnostics.map(t.changes),i=null,r=e.panel;if(e.selected){let r=t.changes.mapPos(e.selected.from,1);i=nw(n,e.selected.diagnostic,r)||nw(n,null,r)}!n.size&&r&&t.state.facet(nI).autoPanel&&(r=null),e=new nO(n,r,i)}for(let n of t.effects)if(n.is(nP)){let i=t.state.facet(nI).autoPanel?n.value.length?n$.open:null:e.panel;e=nO.init(n.value,i,t.state)}else n.is(nC)?e=new nO(e.diagnostics,n.value?n$.open:null,e.selected):n.is(nA)&&(e=new nO(e.diagnostics,e.panel,n.value));return e},provide:e=>[l.mH.from(e,e=>e.panel),l.tk.decorations.from(e,e=>e.diagnostics)]}),nR=l.p.mark({class:"cm-lintRange cm-lintRange-active"}),nD=e=>{let t=e.state.field(nT,!1);return!!t&&!!t.panel&&(e.dispatch({effects:nC.of(!1)}),!0)},nM=[{key:"Mod-Shift-m",run:e=>{var t,n;let i=e.state.field(nT,!1);i&&i.panel||e.dispatch({effects:(t=e.state,n=[nC.of(!0)],t.field(nT,!1)?n:n.concat(o.Py.appendConfig.of(nq)))});let r=(0,l.Sd)(e,n$.open);return r&&r.dom.querySelector(".cm-panel-lint ul").focus(),!0},preventDefault:!0},{key:"F8",run:e=>{let t=e.state.field(nT,!1);if(!t)return!1;let n=e.state.selection.main,i=t.diagnostics.iter(n.to+1);return(!!i.value||!!(i=t.diagnostics.iter(0)).value&&(i.from!=n.from||i.to!=n.to))&&(e.dispatch({selection:{anchor:i.from,head:i.to},scrollIntoView:!0}),!0)}}],nI=o.r$.define({combine:e=>({sources:e.map(e=>e.source).filter(e=>null!=e),...(0,o.BO)(e.map(e=>e.config),{delay:750,markerFilter:null,tooltipFilter:null,needsRefresh:null,hideOn:()=>null},{delay:Math.max,markerFilter:nQ,tooltipFilter:nQ,needsRefresh:(e,t)=>e?t?n=>e(n)||t(n):e:t,hideOn:(e,t)=>e?t?(n,i,r)=>e(n,i,r)||t(n,i,r):e:t,autoPanel:(e,t)=>e||t})})});function nQ(e,t){return e?t?(n,i)=>t(e(n,i),i):e:t}function nN(e){let t=[];if(e)i:for(let{name:n}of e){for(let e=0;e<n.length;e++){let i=n[e];if(/[a-zA-Z]/.test(i)&&!t.some(e=>e.toLowerCase()==i.toLowerCase())){t.push(i);continue i}}t.push("")}return t}function nE(e,t,n){var i;let r=n?nN(t.actions):[];return(0,eN.Z)("li",{class:"cm-diagnostic cm-diagnostic-"+t.severity},(0,eN.Z)("span",{class:"cm-diagnosticText"},t.renderMessage?t.renderMessage(e):t.message),null===(i=t.actions)||void 0===i?void 0:i.map((n,i)=>{let s=!1,o=i=>{if(i.preventDefault(),s)return;s=!0;let r=nw(e.state.field(nT).diagnostics,t);r&&n.apply(e,r.from,r.to)},{name:l}=n,a=r[i]?l.indexOf(r[i]):-1,h=a<0?l:[l.slice(0,a),(0,eN.Z)("u",l.slice(a,a+1)),l.slice(a+1)],c=n.markClass?" "+n.markClass:"";return(0,eN.Z)("button",{type:"button",class:"cm-diagnosticAction"+c,onclick:o,onmousedown:o,"aria-label":` Action: ${l}${a<0?"":` (access key "${r[i]})"`}.`},h)}),t.source&&(0,eN.Z)("div",{class:"cm-diagnosticSource"},t.source))}class nB extends l.l9{constructor(e){super(),this.sev=e}eq(e){return e.sev==this.sev}toDOM(){return(0,eN.Z)("span",{class:"cm-lintPoint cm-lintPoint-"+this.sev})}}class nL{constructor(e,t){this.diagnostic=t,this.id="item_"+Math.floor(4294967295*Math.random()).toString(16),this.dom=nE(e,t,!0),this.dom.id=this.id,this.dom.setAttribute("role","option")}}class n${constructor(e){this.view=e,this.items=[],this.list=(0,eN.Z)("ul",{tabIndex:0,role:"listbox","aria-label":this.view.state.phrase("Diagnostics"),onkeydown:t=>{if(!t.ctrlKey&&!t.altKey&&!t.metaKey){if(27==t.keyCode)nD(this.view),this.view.focus();else if(38==t.keyCode||33==t.keyCode)this.moveSelection((this.selectedIndex-1+this.items.length)%this.items.length);else if(40==t.keyCode||34==t.keyCode)this.moveSelection((this.selectedIndex+1)%this.items.length);else if(36==t.keyCode)this.moveSelection(0);else if(35==t.keyCode)this.moveSelection(this.items.length-1);else if(13==t.keyCode)this.view.focus();else{if(!(t.keyCode>=65)||!(t.keyCode<=90)||!(this.selectedIndex>=0))return;let{diagnostic:n}=this.items[this.selectedIndex],i=nN(n.actions);for(let r=0;r<i.length;r++)if(i[r].toUpperCase().charCodeAt(0)==t.keyCode){let t=nw(this.view.state.field(nT).diagnostics,n);t&&n.actions[r].apply(e,t.from,t.to)}}t.preventDefault()}},onclick:e=>{for(let t=0;t<this.items.length;t++)this.items[t].dom.contains(e.target)&&this.moveSelection(t)}}),this.dom=(0,eN.Z)("div",{class:"cm-panel-lint"},this.list,(0,eN.Z)("button",{type:"button",name:"close","aria-label":this.view.state.phrase("close"),onclick:()=>nD(this.view)},"\xd7")),this.update()}get selectedIndex(){let e=this.view.state.field(nT).selected;if(!e)return -1;for(let t=0;t<this.items.length;t++)if(this.items[t].diagnostic==e.diagnostic)return t;return -1}update(){let{diagnostics:e,selected:t}=this.view.state.field(nT),n=0,i=!1,r=null,s=new Set;for(e.between(0,this.view.state.doc.length,(e,o,{spec:l})=>{for(let e of l.diagnostics){if(s.has(e))continue;s.add(e);let o=-1,l;for(let t=n;t<this.items.length;t++)if(this.items[t].diagnostic==e){o=t;break}o<0?(l=new nL(this.view,e),this.items.splice(n,0,l),i=!0):(l=this.items[o],o>n&&(this.items.splice(n,o-n),i=!0)),t&&l.diagnostic==t.diagnostic?l.dom.hasAttribute("aria-selected")||(l.dom.setAttribute("aria-selected","true"),r=l):l.dom.hasAttribute("aria-selected")&&l.dom.removeAttribute("aria-selected"),n++}});n<this.items.length&&!(1==this.items.length&&this.items[0].diagnostic.from<0);)i=!0,this.items.pop();0==this.items.length&&(this.items.push(new nL(this.view,{from:-1,to:-1,severity:"info",message:this.view.state.phrase("No diagnostics")})),i=!0),r?(this.list.setAttribute("aria-activedescendant",r.id),this.view.requestMeasure({key:this,read:()=>({sel:r.dom.getBoundingClientRect(),panel:this.list.getBoundingClientRect()}),write:({sel:e,panel:t})=>{let n=t.height/this.list.offsetHeight;e.top<t.top?this.list.scrollTop-=(t.top-e.top)/n:e.bottom>t.bottom&&(this.list.scrollTop+=(e.bottom-t.bottom)/n)}})):this.selectedIndex<0&&this.list.removeAttribute("aria-activedescendant"),i&&this.sync()}sync(){let e=this.list.firstChild;function t(){let t=e;e=t.nextSibling,t.remove()}for(let n of this.items)if(n.dom.parentNode==this.list){for(;e!=n.dom;)t();e=n.dom.nextSibling}else this.list.insertBefore(n.dom,e);for(;e;)t()}moveSelection(e){if(this.selectedIndex<0)return;let t=nw(this.view.state.field(nT).diagnostics,this.items[e].diagnostic);t&&this.view.dispatch({selection:{anchor:t.from,head:t.to},scrollIntoView:!0,effects:nA.of(t)})}static open(e){return new n$(e)}}function nj(e){return function(e,t='viewBox="0 0 40 40"'){return`url('data:image/svg+xml,<svg xmlns="http://www.w3.org/2000/svg" ${t}>${encodeURIComponent(e)}</svg>')`}(`<path d="m0 2.5 l2 -1.5 l1 0 l2 1.5 l1 0" stroke="${e}" fill="none" stroke-width=".7"/>`,'width="6" height="3"')}let nz=l.tk.baseTheme({".cm-diagnostic":{padding:"3px 6px 3px 8px",marginLeft:"-1px",display:"block",whiteSpace:"pre-wrap"},".cm-diagnostic-error":{borderLeft:"5px solid #d11"},".cm-diagnostic-warning":{borderLeft:"5px solid orange"},".cm-diagnostic-info":{borderLeft:"5px solid #999"},".cm-diagnostic-hint":{borderLeft:"5px solid #66d"},".cm-diagnosticAction":{font:"inherit",border:"none",padding:"2px 4px",backgroundColor:"#444",color:"white",borderRadius:"3px",marginLeft:"8px",cursor:"pointer"},".cm-diagnosticSource":{fontSize:"70%",opacity:.7},".cm-lintRange":{backgroundPosition:"left bottom",backgroundRepeat:"repeat-x",paddingBottom:"0.7px"},".cm-lintRange-error":{backgroundImage:nj("#d11")},".cm-lintRange-warning":{backgroundImage:nj("orange")},".cm-lintRange-info":{backgroundImage:nj("#999")},".cm-lintRange-hint":{backgroundImage:nj("#66d")},".cm-lintRange-active":{backgroundColor:"#ffdd9980"},".cm-tooltip-lint":{padding:0,margin:0},".cm-lintPoint":{position:"relative","&:after":{content:'""',position:"absolute",bottom:0,left:"-2px",borderLeft:"3px solid transparent",borderRight:"3px solid transparent",borderBottom:"4px solid #d11"}},".cm-lintPoint-warning":{"&:after":{borderBottomColor:"orange"}},".cm-lintPoint-info":{"&:after":{borderBottomColor:"#999"}},".cm-lintPoint-hint":{"&:after":{borderBottomColor:"#66d"}},".cm-panel.cm-panel-lint":{position:"relative","& ul":{maxHeight:"100px",overflowY:"auto","& [aria-selected]":{backgroundColor:"#ddd","& u":{textDecoration:"underline"}},"&:focus [aria-selected]":{background_fallback:"#bdf",backgroundColor:"Highlight",color_fallback:"white",color:"HighlightText"},"& u":{textDecoration:"none"},padding:0,margin:0},"& [name=close]":{position:"absolute",top:"0",right:"2px",background:"inherit",border:"none",font:"inherit",padding:0,margin:0}}});l.SJ;let nX=o.Py.define(),nq=[nT,l.tk.decorations.compute([nT],e=>{let{selected:t,panel:n}=e.field(nT);return t&&n&&t.from!=t.to?l.p.set([nR.range(t.from,t.to)]):l.p.none}),(0,l.bF)(function(e,t,n){let{diagnostics:i}=e.state.field(nT),r,s=-1,o=-1;i.between(t-(n<0?1:0),t+(n>0?1:0),(e,i,{spec:l})=>{if(t>=e&&t<=i&&(e==i||(t>e||n>0)&&(t<i||n<0)))return r=l.diagnostics,s=e,o=i,!1});let l=e.state.facet(nI).tooltipFilter;return(r&&l&&(r=l(r,e.state)),r)?{pos:s,end:o,above:e.state.doc.lineAt(s).to<o,create:()=>{var t,n;return{dom:(t=e,n=r,(0,eN.Z)("ul",{class:"cm-tooltip-lint"},n.map(e=>nE(t,e,!1))))}}}:null},{hideOn:nS}),nz];var nJ=function(e){void 0===e&&(e={});var{crosshairCursor:t=!1}=e,n=[];!1!==e.closeBracketsKeymap&&(n=n.concat(np)),!1!==e.defaultKeymap&&(n=n.concat(eI)),!1!==e.searchKeymap&&(n=n.concat(tg)),!1!==e.historyKeymap&&(n=n.concat(N)),!1!==e.foldKeymap&&(n=n.concat(a.e7)),!1!==e.completionKeymap&&(n=n.concat(ny)),!1!==e.lintKeymap&&(n=n.concat(nM));var i=[];if(!1!==e.lineNumbers&&i.push((0,l.Eu)()),!1!==e.highlightActiveLineGutter&&i.push((0,l.HQ)()),!1!==e.highlightSpecialChars&&i.push((0,l.AE)()),!1!==e.history&&i.push(function(e={}){return[x,y.of(e),l.tk.domEventHandlers({beforeinput(e,t){let n="historyUndo"==e.inputType?O:"historyRedo"==e.inputType?w:null;return!!n&&(e.preventDefault(),n(t))}})]}()),!1!==e.foldGutter&&i.push((0,a.mi)()),!1!==e.drawSelection&&i.push((0,l.Uw)()),!1!==e.dropCursor&&i.push((0,l.qr)()),!1!==e.allowMultipleSelections&&i.push(o.yy.allowMultipleSelections.of(!0)),!1!==e.indentOnInput&&i.push((0,a.nY)()),!1!==e.syntaxHighlighting&&i.push((0,a.nF)(a.R_,{fallback:!0})),!1!==e.bracketMatching&&i.push((0,a.n$)()),!1!==e.closeBrackets&&i.push([nd,na]),!1!==e.autocompletion&&i.push(function(e={}){return[t9,tH,tE.of(e),t3,nx,t6]}()),!1!==e.rectangularSelection&&i.push((0,l.Zs)()),!1!==t&&i.push((0,l.S2)()),!1!==e.highlightActiveLine&&i.push((0,l.ZO)()),!1!==e.highlightSelectionMatches)i.push([eG,eZ]);return e.tabSize&&"number"==typeof e.tabSize&&i.push(a.c.of(" ".repeat(e.tabSize))),i.concat([l.$f.of(n.flat())]).filter(Boolean)},nW=n(35524);let n_="#e06c75",nF="#abb2bf",nU="#7d8799",nV="#d19a66",nZ="#2c313a",nG="#282c34",nY="#353a42",nH="#528bff",nK=l.tk.theme({"&":{color:nF,backgroundColor:nG},".cm-content":{caretColor:nH},".cm-cursor, .cm-dropCursor":{borderLeftColor:nH},"&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection":{backgroundColor:"#3E4451"},".cm-panels":{backgroundColor:"#21252b",color:nF},".cm-panels.cm-panels-top":{borderBottom:"2px solid black"},".cm-panels.cm-panels-bottom":{borderTop:"2px solid black"},".cm-searchMatch":{backgroundColor:"#72a1ff59",outline:"1px solid #457dff"},".cm-searchMatch.cm-searchMatch-selected":{backgroundColor:"#6199ff2f"},".cm-activeLine":{backgroundColor:"#6699ff0b"},".cm-selectionMatch":{backgroundColor:"#aafe661a"},"&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket":{backgroundColor:"#bad0f847"},".cm-gutters":{backgroundColor:nG,color:nU,border:"none"},".cm-activeLineGutter":{backgroundColor:nZ},".cm-foldPlaceholder":{backgroundColor:"transparent",border:"none",color:"#ddd"},".cm-tooltip":{border:"none",backgroundColor:nY},".cm-tooltip .cm-tooltip-arrow:before":{borderTopColor:"transparent",borderBottomColor:"transparent"},".cm-tooltip .cm-tooltip-arrow:after":{borderTopColor:nY,borderBottomColor:nY},".cm-tooltip-autocomplete":{"& > ul > li[aria-selected]":{backgroundColor:nZ,color:nF}}},{dark:!0}),n0=a.Qf.define([{tag:nW.pJ.keyword,color:"#c678dd"},{tag:[nW.pJ.name,nW.pJ.deleted,nW.pJ.character,nW.pJ.propertyName,nW.pJ.macroName],color:n_},{tag:[nW.pJ.function(nW.pJ.variableName),nW.pJ.labelName],color:"#61afef"},{tag:[nW.pJ.color,nW.pJ.constant(nW.pJ.name),nW.pJ.standard(nW.pJ.name)],color:nV},{tag:[nW.pJ.definition(nW.pJ.name),nW.pJ.separator],color:nF},{tag:[nW.pJ.typeName,nW.pJ.className,nW.pJ.number,nW.pJ.changed,nW.pJ.annotation,nW.pJ.modifier,nW.pJ.self,nW.pJ.namespace],color:"#e5c07b"},{tag:[nW.pJ.operator,nW.pJ.operatorKeyword,nW.pJ.url,nW.pJ.escape,nW.pJ.regexp,nW.pJ.link,nW.pJ.special(nW.pJ.string)],color:"#56b6c2"},{tag:[nW.pJ.meta,nW.pJ.comment],color:nU},{tag:nW.pJ.strong,fontWeight:"bold"},{tag:nW.pJ.emphasis,fontStyle:"italic"},{tag:nW.pJ.strikethrough,textDecoration:"line-through"},{tag:nW.pJ.link,color:nU,textDecoration:"underline"},{tag:nW.pJ.heading,fontWeight:"bold",color:n_},{tag:[nW.pJ.atom,nW.pJ.bool,nW.pJ.special(nW.pJ.variableName)],color:nV},{tag:[nW.pJ.processingInstruction,nW.pJ.string,nW.pJ.inserted],color:"#98c379"},{tag:nW.pJ.invalid,color:"#ffffff"}]),n1=[nK,(0,a.nF)(n0)];var n2=l.tk.theme({"&":{backgroundColor:"#fff"}},{dark:!1}),n5=function(e){void 0===e&&(e={});var{indentWithTab:t=!0,editable:n=!0,readOnly:i=!1,theme:r="light",placeholder:s="",basicSetup:a=!0}=e,h=[];switch(t&&h.unshift(l.$f.of([eQ])),a&&("boolean"==typeof a?h.unshift(nJ()):h.unshift(nJ(a))),s&&h.unshift((0,l.W$)(s)),r){case"light":h.push(n2);break;case"dark":h.push(n1);break;case"none":break;default:h.push(r)}return!1===n&&h.push(l.tk.editable.of(!1)),i&&h.push(o.yy.readOnly.of(!0)),[...h]},n3=e=>({line:e.state.doc.lineAt(e.state.selection.main.from),lineCount:e.state.doc.lines,lineBreak:e.state.lineBreak,length:e.state.doc.length,readOnly:e.state.readOnly,tabSize:e.state.tabSize,selection:e.state.selection,selectionAsSingle:e.state.selection.asSingle().main,ranges:e.state.selection.ranges,selectionCode:e.state.sliceDoc(e.state.selection.main.from,e.state.selection.main.to),selections:e.state.selection.ranges.map(t=>e.state.sliceDoc(t.from,t.to)),selectedText:e.state.selection.ranges.some(e=>!e.empty)});class n4{constructor(e,t){this.timeLeftMS=void 0,this.timeoutMS=void 0,this.isCancelled=!1,this.isTimeExhausted=!1,this.callbacks=[],this.timeLeftMS=t,this.timeoutMS=t,this.callbacks.push(e)}tick(){if(!this.isCancelled&&!this.isTimeExhausted&&(this.timeLeftMS--,this.timeLeftMS<=0)){this.isTimeExhausted=!0;var e=this.callbacks.slice();this.callbacks.length=0,e.forEach(e=>{try{e()}catch(e){console.error("TimeoutLatch callback error:",e)}})}}cancel(){this.isCancelled=!0,this.callbacks.length=0}reset(){this.timeLeftMS=this.timeoutMS,this.isCancelled=!1,this.isTimeExhausted=!1}get isDone(){return this.isCancelled||this.isTimeExhausted}}class n9{constructor(){this.interval=null,this.latches=new Set}add(e){this.latches.add(e),this.start()}remove(e){this.latches.delete(e),0===this.latches.size&&this.stop()}start(){null===this.interval&&(this.interval=setInterval(()=>{this.latches.forEach(e=>{e.tick(),e.isDone&&this.remove(e)})},1))}stop(){null!==this.interval&&(clearInterval(this.interval),this.interval=null)}}var n6=null,n7=()=>"undefined"==typeof window?new n9:(n6||(n6=new n9),n6),n8=o.q6.define(),ie=[],it=n(85893),ii=["className","value","selection","extensions","onChange","onStatistics","onCreateEditor","onUpdate","autoFocus","theme","height","minHeight","maxHeight","width","minWidth","maxWidth","basicSetup","placeholder","indentWithTab","editable","readOnly","root","initialState"],ir=(0,s.forwardRef)((e,t)=>{var{className:n,value:a="",selection:h,extensions:c=[],onChange:u,onStatistics:f,onCreateEditor:d,onUpdate:p,autoFocus:m,theme:g="light",height:k,minHeight:v,maxHeight:y,width:x,minWidth:b,maxWidth:O,basicSetup:w,placeholder:S,indentWithTab:P,editable:C,readOnly:A,root:T,initialState:R}=e,D=(0,r.Z)(e,ii),M=(0,s.useRef)(null),{state:I,view:Q,container:N,setContainer:E}=function(e){var{value:t,selection:n,onChange:i,onStatistics:r,onCreateEditor:a,onUpdate:h,extensions:c=ie,autoFocus:u,theme:f="light",height:d=null,minHeight:p=null,maxHeight:m=null,width:g=null,minWidth:k=null,maxWidth:v=null,placeholder:y="",editable:x=!0,readOnly:b=!1,indentWithTab:O=!0,basicSetup:w=!0,root:S,initialState:P}=e,[C,A]=(0,s.useState)(),[T,R]=(0,s.useState)(),[D,M]=(0,s.useState)(),I=(0,s.useState)(()=>({current:null}))[0],Q=(0,s.useState)(()=>({current:null}))[0],N=l.tk.theme({"&":{height:d,minHeight:p,maxHeight:m,width:g,minWidth:k,maxWidth:v},"& .cm-scroller":{height:"100% !important"}}),E=[l.tk.updateListener.of(e=>{e.docChanged&&"function"==typeof i&&!e.transactions.some(e=>e.annotation(n8))&&(I.current?I.current.reset():(I.current=new n4(()=>{if(Q.current){var e=Q.current;Q.current=null,e()}I.current=null},200),n7().add(I.current)),i(e.state.doc.toString(),e)),r&&r(n3(e))}),N,...n5({theme:f,editable:x,readOnly:b,placeholder:y,indentWithTab:O,basicSetup:w})];return h&&"function"==typeof h&&E.push(l.tk.updateListener.of(h)),E=E.concat(c),(0,s.useLayoutEffect)(()=>{if(C&&!D){var e={doc:t,selection:n,extensions:E},i=P?o.yy.fromJSON(P.json,e,P.fields):o.yy.create(e);if(M(i),!T){var r=new l.tk({state:i,parent:C,root:S});R(r),a&&a(r,i)}}return()=>{T&&(M(void 0),R(void 0))}},[C,D]),(0,s.useEffect)(()=>{e.container&&A(e.container)},[e.container]),(0,s.useEffect)(()=>()=>{T&&(T.destroy(),R(void 0)),I.current&&(I.current.cancel(),I.current=null)},[T]),(0,s.useEffect)(()=>{u&&T&&T.focus()},[u,T]),(0,s.useEffect)(()=>{T&&T.dispatch({effects:o.Py.reconfigure.of(E)})},[f,c,d,p,m,g,k,v,y,x,b,O,w,i,h]),(0,s.useEffect)(()=>{if(void 0!==t){var e=T?T.state.doc.toString():"";if(T&&t!==e){var n=I.current&&!I.current.isDone,i=()=>{T&&t!==T.state.doc.toString()&&T.dispatch({changes:{from:0,to:T.state.doc.toString().length,insert:t||""},annotations:[n8.of(!0)]})};n?Q.current=i:i()}}},[t,T]),{state:D,setState:M,view:T,setView:R,container:C,setContainer:A}}({root:T,value:a,autoFocus:m,theme:g,height:k,minHeight:v,maxHeight:y,width:x,minWidth:b,maxWidth:O,basicSetup:w,placeholder:S,indentWithTab:P,editable:C,readOnly:A,selection:h,onChange:u,onStatistics:f,onCreateEditor:d,onUpdate:p,extensions:c,initialState:R});(0,s.useImperativeHandle)(t,()=>({editor:M.current,state:I,view:Q}),[M,N,I,Q]);var B=(0,s.useCallback)(e=>{M.current=e,E(e)},[E]);if("string"!=typeof a)throw Error("value must be typeof string but got "+typeof a);var L="string"==typeof g?"cm-theme-"+g:"cm-theme";return(0,it.jsx)("div",(0,i.Z)({ref:B,className:""+L+(n?" "+n:"")},D))});ir.displayName="CodeMirror";var is=ir},12003:function(e,t,n){n.d(t,{j:function(){return o}});var i=n(90512);let r=e=>"boolean"==typeof e?`${e}`:0===e?"0":e,s=i.W,o=(e,t)=>n=>{var i;if((null==t?void 0:t.variants)==null)return s(e,null==n?void 0:n.class,null==n?void 0:n.className);let{variants:o,defaultVariants:l}=t,a=Object.keys(o).map(e=>{let t=null==n?void 0:n[e],i=null==l?void 0:l[e];if(null===t)return null;let s=r(t)||r(i);return o[e][s]}),h=n&&Object.entries(n).reduce((e,t)=>{let[n,i]=t;return void 0===i||(e[n]=i),e},{});return s(e,a,null==t?void 0:null===(i=t.compoundVariants)||void 0===i?void 0:i.reduce((e,t)=>{let{class:n,className:i,...r}=t;return Object.entries(r).every(e=>{let[t,n]=e;return Array.isArray(n)?n.includes({...l,...h}[t]):({...l,...h})[t]===n})?[...e,n,i]:e},[]),null==n?void 0:n.class,null==n?void 0:n.className)}},18580:function(e,t,n){function i(){var e=arguments[0];"string"==typeof e&&(e=document.createElement(e));var t=1,n=arguments[1];if(n&&"object"==typeof n&&null==n.nodeType&&!Array.isArray(n)){for(var i in n)if(Object.prototype.hasOwnProperty.call(n,i)){var r=n[i];"string"==typeof r?e.setAttribute(i,r):null!=r&&(e[i]=r)}t++}for(;t<arguments.length;t++)!function e(t,n){if("string"==typeof n)t.appendChild(document.createTextNode(n));else if(null==n);else if(null!=n.nodeType)t.appendChild(n);else if(Array.isArray(n))for(var i=0;i<n.length;i++)e(t,n[i]);else throw RangeError("Unsupported child node: "+n)}(e,arguments[t]);return e}n.d(t,{Z:function(){return i}})},28699:function(e,t,n){n.d(t,{V:function(){return o}});let i="undefined"==typeof Symbol?"__ͼ":Symbol.for("ͼ"),r="undefined"==typeof Symbol?"__styleSet"+Math.floor(1e8*Math.random()):Symbol("styleSet"),s="undefined"!=typeof globalThis?globalThis:"undefined"!=typeof window?window:{};class o{constructor(e,t){this.rules=[];let{finish:n}=t||{};function i(e){return/^@/.test(e)?[e]:e.split(/,\s*/)}for(let t in e)!function e(t,r,s,o){let l=[],a=/^@(\w+)\b/.exec(t[0]),h=a&&"keyframes"==a[1];if(a&&null==r)return s.push(t[0]+";");for(let n in r){let o=r[n];if(/&/.test(n))e(n.split(/,\s*/).map(e=>t.map(t=>e.replace(/&/,t))).reduce((e,t)=>e.concat(t)),o,s);else if(o&&"object"==typeof o){if(!a)throw RangeError("The value of a property ("+n+") should be a primitive value.");e(i(n),o,l,h)}else null!=o&&l.push(n.replace(/_.*/,"").replace(/[A-Z]/g,e=>"-"+e.toLowerCase())+": "+o+";")}(l.length||h)&&s.push((!n||a||o?t:t.map(n)).join(", ")+" {"+l.join(" ")+"}")}(i(t),e[t],this.rules)}getRules(){return this.rules.join("\n")}static newName(){let e=s[i]||1;return s[i]=e+1,"ͼ"+e.toString(36)}static mount(e,t,n){let i=e[r],s=n&&n.nonce;i?s&&i.setNonce(s):i=new a(e,s),i.mount(Array.isArray(t)?t:[t],e)}}let l=new Map;class a{constructor(e,t){let n=e.ownerDocument||e,i=n.defaultView;if(!e.head&&e.adoptedStyleSheets&&i.CSSStyleSheet){let t=l.get(n);if(t)return e[r]=t;this.sheet=new i.CSSStyleSheet,l.set(n,this)}else this.styleTag=n.createElement("style"),t&&this.styleTag.setAttribute("nonce",t);this.modules=[],e[r]=this}mount(e,t){let n=this.sheet,i=0,r=0;for(let t=0;t<e.length;t++){let s=e[t],o=this.modules.indexOf(s);if(o<r&&o>-1&&(this.modules.splice(o,1),r--,o=-1),-1==o){if(this.modules.splice(r++,0,s),n)for(let e=0;e<s.rules.length;e++)n.insertRule(s.rules[e],i++)}else{for(;r<o;)i+=this.modules[r++].rules.length;i+=s.rules.length,r++}}if(n)0>t.adoptedStyleSheets.indexOf(this.sheet)&&(t.adoptedStyleSheets=[this.sheet,...t.adoptedStyleSheets]);else{let e="";for(let t=0;t<this.modules.length;t++)e+=this.modules[t].getRules()+"\n";this.styleTag.textContent=e;let n=t.head||t;this.styleTag.parentNode!=n&&n.insertBefore(this.styleTag,n.firstChild)}}setNonce(e){this.styleTag&&this.styleTag.getAttribute("nonce")!=e&&this.styleTag.setAttribute("nonce",e)}}},81754:function(e,t,n){n.d(t,{YG:function(){return h},uY:function(){return r},ue:function(){return i}});for(var i={8:"Backspace",9:"Tab",10:"Enter",12:"NumLock",13:"Enter",16:"Shift",17:"Control",18:"Alt",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",44:"PrintScreen",45:"Insert",46:"Delete",59:";",61:"=",91:"Meta",92:"Meta",106:"*",107:"+",108:",",109:"-",110:".",111:"/",144:"NumLock",145:"ScrollLock",160:"Shift",161:"Shift",162:"Control",163:"Control",164:"Alt",165:"Alt",173:"-",186:";",187:"=",188:",",189:"-",190:".",191:"/",192:"`",219:"[",220:"\\",221:"]",222:"'"},r={48:")",49:"!",50:"@",51:"#",52:"$",53:"%",54:"^",55:"&",56:"*",57:"(",59:":",61:"+",173:"_",186:":",187:"+",188:"<",189:"_",190:">",191:"?",192:"~",219:"{",220:"|",221:"}",222:'"'},s="undefined"!=typeof navigator&&/Mac/.test(navigator.platform),o="undefined"!=typeof navigator&&/MSIE \d|Trident\/(?:[7-9]|\d{2,})\..*rv:(\d+)/.exec(navigator.userAgent),l=0;l<10;l++)i[48+l]=i[96+l]=String(l);for(var l=1;l<=24;l++)i[l+111]="F"+l;for(var l=65;l<=90;l++)i[l]=String.fromCharCode(l+32),r[l]=String.fromCharCode(l);for(var a in i)r.hasOwnProperty(a)||(r[a]=i[a]);function h(e){var t=!(s&&e.metaKey&&e.shiftKey&&!e.ctrlKey&&!e.altKey||o&&e.shiftKey&&e.key&&1==e.key.length||"Unidentified"==e.key)&&e.key||(e.shiftKey?r:i)[e.keyCode]||e.key||"Unidentified";return"Esc"==t&&(t="Escape"),"Del"==t&&(t="Delete"),"Left"==t&&(t="ArrowLeft"),"Up"==t&&(t="ArrowUp"),"Right"==t&&(t="ArrowRight"),"Down"==t&&(t="ArrowDown"),t}}}]);